@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2018 by the deal.II authors
+// Copyright (C) 1998 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -53,7 +53,7 @@ in whatever directory you set up the library to be compiled in, to indent all
 source files that have been changed recently. If you want to make sure that
 the indenting is correct for all your commits, you might want to set up a
 pre-commit hook. One way to do so, is to copy
-<code>\${SOURCE_DIR}/contrib/scripts/pre-commit-clang-format</code> to
+<code>\${SOURCE_DIR}/contrib/git-hooks/pre-commit</code> to
 <code>\${SOURCE_DIR}/.git/hooks/pre-commit</code> and make sure it is
 executable.
 </p>
@@ -63,11 +63,11 @@ executable.
 <ol>
 <li> %Functions which return the number of something (number of cells,
   degrees of freedom, etc) should start with <code>n_*</code>. Example:
-  SparsityPattern::n_nonzero_entries().</li>
+  SparsityPatternBase::n_nonzero_elements().</li>
 
 <li> %Functions which set a bit or flag should start with <code>set_*</code>;
   functions which clear bits or flags should be named <code>clear_*</code>.
-  Example: CellIterator::set_refine_flag().</li>
+  Example: CellAccessor::set_refine_flag().</li>
 
 <li> Traditional logical operators should be used instead of their English
   equivalents (i.e., use <code>&&</code>, <code>||</code>, and <code>!</code>
@@ -119,7 +119,36 @@ executable.
   then functions.
   <br>
   Exceptions shall be declared at the end of the public section
-  before the non-public sections start.</li>
+  before the non-public sections start.
+  <br>
+  We do not use the C++11-style class member initialization for member variables
+  that are neither <code>static const</code> nor <code>static constexpr</code>;
+  i.e., instead of
+@code
+  class Foo
+  {
+    int a = 42;
+    int *b = nullptr;
+  };
+@endcode
+  write
+@code
+  class Foo
+  {
+    Foo();
+
+    int a;
+    int *b;
+  };
+
+
+
+  inline Foo::Foo()
+  : a(42)
+  , b(nullptr)
+  {}
+@endcode
+  </li>
 
 <li> If a function has both input and output parameters, usually the
   input parameters shall precede the output parameters, unless there
@@ -196,12 +225,12 @@ instantiate these functions for any other template arguments anyway. </li>
 <li> If we can not enumerate all possible template arguments (e.g., vector
 types -- because users might want to define their own vector kinds) but at
 least know a few common usage cases, then the function is put into a
-<code>.templates.h</code> file. We #include it into the <code>.cc</code> file
+<code>.templates.h</code> file. We \#include it into the <code>.cc</code> file
 and instantiate the functions for all of the common arguments. For almost all
 users, this will be just fine -- they only use the (vector, matrix, ...) types
 we already instantiate, and for them the <code>.templates.h</code> file will not
 be of any interest. It will also not slow down their compilations because
-nothing they see will #include the <code>.templates.h</code> file. But users who
+nothing they see will \#include the <code>.templates.h</code> file. But users who
 define their own (vector, matrix, ...) types can instantiate the template
 functions with their own user-defined types by including the
 <code>.templates.h</code> files.
@@ -209,8 +238,8 @@ functions with their own user-defined types by including the
 <li> Finally, if we can not assume in advance which values template arguments
 will take (e.g., any class derived from Subscriptor can be used as an argument),
 the definitions of functions are provided at the bottom of the header
-file with declarations. The definitions should be guarded with <code>#ifndef
-DOXYGEN ... #endif</code> to prevent Doxygen from picking them up.</li>
+file with declarations. The definitions should be guarded with <code>\#ifndef
+DOXYGEN ... \#endif</code> to prevent Doxygen from picking them up.</li>
 
 </ol>
 
@@ -219,7 +248,7 @@ DOXYGEN ... #endif</code> to prevent Doxygen from picking them up.</li>
 expand_instantiations (built from
 <code>cmake/scripts/expand_instantiations.cc</code>) and the parameters are
 defined dynamically through cmake depending on your configuration (see
-<code>share/deal.II/template-arguments</code> in your build directory).
+<code>cmake/config/template-arguments.in</code> in your build directory).
 It is those <code>.inst</code> files that are eventually included from the
 corresponding <code>.cc</code> files. </p>
 
@@ -246,8 +275,8 @@ we list here:
   consider a trivial implementation of vector addition:
   @code
     Vector &
-    operator += (Vector       &lhs,
-                 const Vector &rhs)
+    operator+=(Vector       &lhs,
+               const Vector &rhs)
     {
       for (unsigned int i=0; i<lhs.size(); ++i)
         lhs(i) += rhs(i);
@@ -268,11 +297,11 @@ we list here:
   implementation will do exactly this:
   @code
     Vector &
-    operator += (Vector       &lhs,
-                 const Vector &rhs)
+    operator+=(Vector       &lhs,
+               const Vector &rhs)
     {
       Assert (lhs.size() == rhs.size(),
-              ExcDimensionMismatch (lhs.size(), rhs.size());
+              ExcDimensionMismatch(lhs.size(), rhs.size());
       for (unsigned int i=0; i<lhs.size(); ++i)
         lhs(i) += rhs(i);
       return lhs;
@@ -312,7 +341,7 @@ we list here:
   a vector would expect the norm to be positive. You can write this as
   follows:
   @code
-    double norm (const Vector &v)
+    double norm(const Vector &v)
     {
       double s = 0;
       for (unsigned int i=0; i<v.size(); ++i)
@@ -347,8 +376,8 @@ we list here:
             { ... }
           else
             {
-                // we have a cell whose neighbor must
-                // be at the boundary if we got here
+              // we have a cell whose neighbor must
+              // be at the boundary if we got here
             }
         }
   @endcode
@@ -431,8 +460,8 @@ we list here:
       ... // something lengthy and complicated
       for (const auto &cell = dof_handler.active_cell_iterators())
         {
-          <b>const</b> Point<dim> cell_center = (cell->vertex(0) +
-                                                 cell->vertex(1)) / 2;
+          const Point<dim> cell_center = (cell->vertex(0) +
+                                          cell->vertex(1)) / 2;
           ...
         }
       ...
@@ -460,7 +489,7 @@ we list here:
   @code
      template <int dim>
      typename Triangulation<dim>::cell_iterator
-     CellAccessor<dim>::child (const unsigned int child_no)
+     CellAccessor<dim>::child(const unsigned int child_no)
      {
        ...
        return something;
