@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2006 - 2019 by the deal.II authors
+ * Copyright (C) 2006 - 2020 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -66,6 +66,10 @@
 // such functionality:
 #include <deal.II/base/tensor_function.h>
 
+// Additionally, we use the class <code>DiscreteTime</code> to perform
+// operations related to time incrementation.
+#include <deal.II/base/discrete_time.h>
+
 // The last step is as in all previous programs:
 namespace Step21
 {
@@ -91,7 +95,9 @@ namespace Step21
   //
   // The rest of the class should be pretty much obvious. The
   // <code>viscosity</code> variable stores the viscosity $\mu$ that enters
-  // several of the formulas in the nonlinear equations.
+  // several of the formulas in the nonlinear equations. The variable
+  // <code>time</code> keeps track of the time information within the
+  // simulation.
   template <int dim>
   class TwoPhaseFlowProblem
   {
@@ -119,8 +125,7 @@ namespace Step21
 
     const unsigned int n_refinement_steps;
 
-    double       time_step;
-    unsigned int timestep_number;
+    DiscreteTime time;
     double       viscosity;
 
     BlockVector<double> solution;
@@ -424,7 +429,7 @@ namespace Step21
     {
       SolverControl solver_control(std::max<unsigned int>(src.size(), 200),
                                    1e-8 * src.l2_norm());
-      SolverCG<>    cg(solver_control);
+      SolverCG<Vector<double>> cg(solver_control);
 
       dst = 0;
 
@@ -498,9 +503,13 @@ namespace Step21
   // @sect4{TwoPhaseFlowProblem::TwoPhaseFlowProblem}
 
   // First for the constructor. We use $RT_k \times DQ_k \times DQ_k$
-  // spaces. The time step is set to zero initially, but will be computed
-  // before it is needed first, as described in a subsection of the
-  // introduction.
+  // spaces. For initializing the DiscreteTime object, we don't set the time
+  // step size in the constructor because we don't have its value yet.
+  // The time step size is initially set to zero, but it will be computed
+  // before it is needed to increment time, as described in a subsection of
+  // the introduction. The time object internally prevents itself from being
+  // incremented when $dt = 0$, forcing us to set a non-zero desired size for
+  // $dt$ before advancing time.
   template <int dim>
   TwoPhaseFlowProblem<dim>::TwoPhaseFlowProblem(const unsigned int degree)
     : degree(degree)
@@ -512,8 +521,7 @@ namespace Step21
          1)
     , dof_handler(triangulation)
     , n_refinement_steps(5)
-    , time_step(0)
-    , timestep_number(1)
+    , time(/*start time*/ 0., /*end time*/ 1.)
     , viscosity(0.2)
   {}
 
@@ -534,8 +542,8 @@ namespace Step21
     dof_handler.distribute_dofs(fe);
     DoFRenumbering::component_wise(dof_handler);
 
-    std::vector<types::global_dof_index> dofs_per_component(dim + 2);
-    DoFTools::count_dofs_per_component(dof_handler, dofs_per_component);
+    const std::vector<types::global_dof_index> dofs_per_component =
+      DoFTools::count_dofs_per_fe_component(dof_handler);
     const unsigned int n_u = dofs_per_component[0],
                        n_p = dofs_per_component[dim],
                        n_s = dofs_per_component[dim + 1];
@@ -714,12 +722,10 @@ namespace Step21
 
         // Next, we also have to deal with the pressure boundary values. This,
         // again is as in step-20:
-        for (unsigned int face_no = 0;
-             face_no < GeometryInfo<dim>::faces_per_cell;
-             ++face_no)
-          if (cell->at_boundary(face_no))
+        for (const auto &face : cell->face_iterators())
+          if (face->at_boundary())
             {
-              fe_face_values.reinit(cell, face_no);
+              fe_face_values.reinit(cell, face);
 
               pressure_boundary_values.value_list(
                 fe_face_values.get_quadrature_points(), boundary_values);
@@ -829,10 +835,11 @@ namespace Step21
               const Tensor<1, dim> grad_phi_i_s =
                 fe_values[saturation].gradient(i, q);
 
-              local_rhs(i) += (time_step * fractional_flow(old_s, viscosity) *
-                                 present_u * grad_phi_i_s +
-                               old_s * phi_i_s) *
-                              fe_values.JxW(q);
+              local_rhs(i) +=
+                (time.get_next_step_size() * fractional_flow(old_s, viscosity) *
+                   present_u * grad_phi_i_s +
+                 old_s * phi_i_s) *
+                fe_values.JxW(q);
             }
 
         // Secondly, we have to deal with the flux parts on the face
@@ -844,9 +851,7 @@ namespace Step21
         //
         // All this is a bit tricky, but has been explained in some detail
         // already in step-9. Take a look there how this is supposed to work!
-        for (unsigned int face_no = 0;
-             face_no < GeometryInfo<dim>::faces_per_cell;
-             ++face_no)
+        for (unsigned int face_no : GeometryInfo<dim>::face_indices())
           {
             fe_face_values.reinit(cell, face_no);
 
@@ -888,7 +893,7 @@ namespace Step21
 
                 for (unsigned int i = 0; i < dofs_per_cell; ++i)
                   local_rhs(i) -=
-                    time_step * normal_flux *
+                    time.get_next_step_size() * normal_flux *
                     fractional_flow((is_outflow_q_point == true ?
                                        old_solution_values_face[q](dim + 1) :
                                        neighbor_saturation[q]),
@@ -937,9 +942,9 @@ namespace Step21
         approximate_schur_complement);
 
 
-      SolverControl solver_control(solution.block(1).size(),
+      SolverControl            solver_control(solution.block(1).size(),
                                    1e-12 * schur_rhs.l2_norm());
-      SolverCG<>    cg(solver_control);
+      SolverCG<Vector<double>> cg(solver_control);
 
       cg.solve(schur_complement, solution.block(1), schur_rhs, preconditioner);
 
@@ -967,18 +972,24 @@ namespace Step21
     //
     // The maximal velocity we compute using a helper function to compute the
     // maximal velocity defined below, and with all this we can evaluate our
-    // new time step length:
-    time_step =
-      std::pow(0.5, double(n_refinement_steps)) / get_maximal_velocity();
+    // new time step length. We use the method
+    // DiscreteTime::set_desired_next_time_step() to suggest the new
+    // calculated value of the time step to the DiscreteTime object. In most
+    // cases, the time object uses the exact provided value to increment time.
+    // It some case, the step size may be modified further by the time object.
+    // For example, if the calculated time increment overshoots the end time,
+    // it is truncated accordingly.
+    time.set_desired_next_step_size(std::pow(0.5, double(n_refinement_steps)) /
+                                    get_maximal_velocity());
 
     // The next step is to assemble the right hand side, and then to pass
     // everything on for solution. At the end, we project back saturations
     // onto the physically reasonable range:
     assemble_rhs_S();
     {
-      SolverControl solver_control(system_matrix.block(2, 2).m(),
+      SolverControl            solver_control(system_matrix.block(2, 2).m(),
                                    1e-8 * system_rhs.block(2).l2_norm());
-      SolverCG<>    cg(solver_control);
+      SolverCG<Vector<double>> cg(solver_control);
       cg.solve(system_matrix.block(2, 2),
                solution.block(2),
                system_rhs.block(2),
@@ -1009,7 +1020,7 @@ namespace Step21
   template <int dim>
   void TwoPhaseFlowProblem<dim>::output_results() const
   {
-    if (timestep_number % 5 != 0)
+    if (time.get_step_number() % 5 != 0)
       return;
 
     std::vector<std::string> solution_names;
@@ -1035,7 +1046,8 @@ namespace Step21
     data_out.build_patches(degree + 1);
 
     std::ofstream output("solution-" +
-                         Utilities::int_to_string(timestep_number, 4) + ".vtk");
+                         Utilities::int_to_string(time.get_step_number(), 4) +
+                         ".vtk");
     data_out.write_vtk(output);
   }
 
@@ -1122,13 +1134,15 @@ namespace Step21
   //
   // The second point worth mentioning is that we only compute the length of
   // the present time step in the middle of solving the linear system
-  // corresponding to each time step. We can therefore output the present end
+  // corresponding to each time step. We can therefore output the present
   // time of a time step only at the end of the time step.
-  //
-  // The function as it is here does actually not compute the results
-  // found on the web page. The reason is, that even on a decent
-  // computer it runs more than a day. If you want to reproduce these
-  // results, set the final time at the end of the do loop to 250.
+  // We increment time by calling the method DiscreteTime::advance_time()
+  // inside the loop. Since we are reporting the time and dt after we
+  // increment it, we have to call the method
+  // DiscreteTime::get_previous_step_size() instead of
+  // DiscreteTime::get_next_step_size(). After many steps, when the simulation
+  // reaches the end time, the last dt is chosen by the DiscreteTime class in
+  // such a way that the last step finishes exactly at the end time.
   template <int dim>
   void TwoPhaseFlowProblem<dim>::run()
   {
@@ -1145,11 +1159,9 @@ namespace Step21
                            old_solution);
     }
 
-    double time = 0;
-
     do
       {
-        std::cout << "Timestep " << timestep_number << std::endl;
+        std::cout << "Timestep " << time.get_step_number() + 1 << std::endl;
 
         assemble_system();
 
@@ -1157,13 +1169,13 @@ namespace Step21
 
         output_results();
 
-        time += time_step;
-        ++timestep_number;
-        std::cout << "   Now at t=" << time << ", dt=" << time_step << '.'
+        time.advance_time();
+        std::cout << "   Now at t=" << time.get_current_time()
+                  << ", dt=" << time.get_previous_step_size() << '.'
                   << std::endl
                   << std::endl;
       }
-    while (time <= 1.);
+    while (time.is_at_end() == false);
   }
 } // namespace Step21
 
@@ -1178,7 +1190,6 @@ int main()
 {
   try
     {
-      using namespace dealii;
       using namespace Step21;
 
       TwoPhaseFlowProblem<2> two_phase_flow_problem(0);
