@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2019 by the deal.II authors
+ * Copyright (C) 2009 - 2020 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -392,7 +392,7 @@ namespace Step28
   class EnergyGroup
   {
   public:
-    // @sect5{Public member functions}
+    // @sect5{<code>EnergyGroup</code> public member functions}
     //
     // The class has a good number of public member functions, since its the
     // way it operates is controlled from the outside, and therefore all
@@ -458,7 +458,7 @@ namespace Step28
                      const double         refine_threshold,
                      const double         coarsen_threshold);
 
-    // @sect5{Public data members}
+    // @sect5{<code>EnergyGroup</code> public data members}
     //
     // As is good practice in object oriented programming, we hide most data
     // members by making them private. However, we have to grant the class
@@ -471,7 +471,7 @@ namespace Step28
     Vector<double> solution_old;
 
 
-    // @sect5{Private data members}
+    // @sect5{<code>EnergyGroup</code> private data members}
     //
     // The rest of the data members are private. Compared to all the previous
     // tutorial programs, the only new data members are an integer storing
@@ -501,7 +501,7 @@ namespace Step28
     AffineConstraints<double>                 hanging_node_constraints;
 
 
-    // @sect5{Private member functions}
+    // @sect5{<code>EnergyGroup</code> private member functions}
     //
     // There is one private member function in this class. It recursively
     // walks over cells of two meshes to compute the cross-group right hand
@@ -1014,11 +1014,11 @@ namespace Step28
                                        solution,
                                        system_rhs);
 
-    SolverControl solver_control(system_matrix.m(),
+    SolverControl            solver_control(system_matrix.m(),
                                  1e-12 * system_rhs.l2_norm());
-    SolverCG<>    cg(solver_control);
+    SolverCG<Vector<double>> cg(solver_control);
 
-    PreconditionSSOR<> preconditioner;
+    PreconditionSSOR<SparseMatrix<double>> preconditioner;
     preconditioner.initialize(system_matrix, 1.2);
 
     cg.solve(system_matrix, solution, system_rhs, preconditioner);
@@ -1141,10 +1141,11 @@ namespace Step28
   // used to determine when convergence of the inverse power iteration has
   // occurred. In addition, we have a constructor of this class that sets all
   // these values to their default values, a function
-  // <code>declare_parameters</code> that described to the ParameterHandler
-  // class already used in step-19 what parameters are accepted in the input
-  // file, and a function <code>get_parameters</code> that can extract the
-  // values of these parameters from a ParameterHandler object.
+  // <code>declare_parameters</code> that describes to the ParameterHandler
+  // class what parameters are accepted in the input file, and a function
+  // <code>get_parameters</code> that can extract the values of these
+  // parameters from a ParameterHandler object. See also step-29 for another
+  // example of using ParameterHandler.
   template <int dim>
   class NeutronDiffusionProblem
   {
@@ -1170,7 +1171,7 @@ namespace Step28
     void run();
 
   private:
-    // @sect5{Private member functions}
+    // @sect5{<code>NeutronDiffusionProblem</code> private member functions}
 
     // There are not that many member functions in this class since most of
     // the functionality has been moved into the <code>EnergyGroup</code>
@@ -1183,7 +1184,7 @@ namespace Step28
     double get_total_fission_source() const;
 
 
-    // @sect5{Private member variables}
+    // @sect5{<code>NeutronDiffusionProblem</code> private member variables}
 
     // Next, we have a few member variables. In particular, these are (i) a
     // reference to the parameter object (owned by the main function of this
@@ -1524,7 +1525,7 @@ namespace Step28
     BlockVector<float> group_error_indicators(n_cells);
 
     {
-      Threads::ThreadGroup<> threads;
+      Threads::ThreadGroup<void> threads;
       for (unsigned int group = 0; group < parameters.n_groups; ++group)
         threads += Threads::new_thread(&EnergyGroup<dim>::estimate_errors,
                                        *energy_groups[group],
@@ -1537,7 +1538,7 @@ namespace Step28
     const float coarsen_threshold = 0.01 * max_error;
 
     {
-      Threads::ThreadGroup<> threads;
+      Threads::ThreadGroup<void> threads;
       for (unsigned int group = 0; group < parameters.n_groups; ++group)
         threads += Threads::new_thread(&EnergyGroup<dim>::refine_grid,
                                        *energy_groups[group],
@@ -1611,7 +1612,7 @@ namespace Step28
         std::cout << std::endl << std::endl;
 
 
-        Threads::ThreadGroup<> threads;
+        Threads::ThreadGroup<void> threads;
         for (unsigned int group = 0; group < parameters.n_groups; ++group)
           threads +=
             Threads::new_thread(&EnergyGroup<dim>::assemble_system_matrix,
