@@ -1,6 +1,6 @@
 /* ---------------------------------------------------------------------
  *
- * Copyright (C) 2009 - 2019 by the deal.II authors
+ * Copyright (C) 2009 - 2020 by the deal.II authors
  *
  * This file is part of the deal.II library.
  *
@@ -30,9 +30,11 @@
 
 #include <deal.II/lac/generic_linear_algebra.h>
 
-// uncomment the following #define if you have PETSc and Trilinos installed
+// uncomment the following \#define if you have PETSc and Trilinos installed
 // and you prefer using Trilinos in this example:
+// @code
 // #define FORCE_USE_OF_TRILINOS
+// @endcode
 
 // This will either import PETSc or TrilinosWrappers into the namespace
 // LA. Note that we are defining the macro USE_PETSC_LA so that we can detect
@@ -314,11 +316,10 @@ namespace Step40
     DynamicSparsityPattern dsp(locally_relevant_dofs);
 
     DoFTools::make_sparsity_pattern(dof_handler, dsp, constraints, false);
-    SparsityTools::distribute_sparsity_pattern(
-      dsp,
-      dof_handler.n_locally_owned_dofs_per_processor(),
-      mpi_communicator,
-      locally_relevant_dofs);
+    SparsityTools::distribute_sparsity_pattern(dsp,
+                                               dof_handler.locally_owned_dofs(),
+                                               mpi_communicator,
+                                               locally_relevant_dofs);
 
     system_matrix.reinit(locally_owned_dofs,
                          locally_owned_dofs,
@@ -572,40 +573,11 @@ namespace Step40
 
     data_out.build_patches();
 
-    // The next step is to write this data to disk. We choose file names of
-    // the form <code>solution-XX.PPPP.vtu</code> where <code>XX</code>
-    // indicates the refinement cycle, <code>PPPP</code> refers to the
-    // processor number (enough for up to 10,000 processors, though we hope
-    // that nobody ever tries to generate this much data -- you would likely
-    // overflow all file system quotas), and <code>.vtu</code> indicates the
-    // XML-based Visualization Toolkit for Unstructured grids (VTU) file
-    // format.
-    const std::string filename =
-      ("solution-" + Utilities::int_to_string(cycle, 2) + "." +
-       Utilities::int_to_string(triangulation.locally_owned_subdomain(), 4));
-    std::ofstream output(filename + ".vtu");
-    data_out.write_vtu(output);
-
-    // The last step is to write a "master record" that lists for the
-    // visualization program the names of the various files that combined
-    // represents the graphical data for the entire domain. The
-    // DataOutBase::write_pvtu_record does this, and it needs a list of
-    // filenames that we create first. Note that only one processor needs to
-    // generate this file; we arbitrarily choose processor zero to take over
-    // this job.
-    if (Utilities::MPI::this_mpi_process(mpi_communicator) == 0)
-      {
-        std::vector<std::string> filenames;
-        for (unsigned int i = 0;
-             i < Utilities::MPI::n_mpi_processes(mpi_communicator);
-             ++i)
-          filenames.push_back("solution-" + Utilities::int_to_string(cycle, 2) +
-                              "." + Utilities::int_to_string(i, 4) + ".vtu");
-
-        std::ofstream master_output(
-          "solution-" + Utilities::int_to_string(cycle, 2) + ".pvtu");
-        data_out.write_pvtu_record(master_output, filenames);
-      }
+    // The next step is to write this data to disk. We write up to 8 VTU files
+    // in parallel with the help of MPI-IO. Additionally a PVTU record is
+    // generated, which groups the written VTU files.
+    data_out.write_vtu_with_pvtu_record(
+      "./", "solution", cycle, mpi_communicator, 2, 8);
   }
 
 
