@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2017 by the deal.II authors
+// Copyright (C) 2004 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -49,13 +49,11 @@ DEAL_II_NAMESPACE_OPEN
  * @code
  * ConditionalOStream pout(std::cout, this_mpi_process==0);
  *
- *                                  // all processes print following
- *                                  // information to standard output
+ * // all processes print the following information to standard output
  * std::cout << "Reading parameter file on process "
  *           << this_mpi_process << std::endl;
  *
- *                                  // following is printed by
- *                                  // process 0 only
+ * // following is printed by process 0 only
  * pout << "Solving ..." << std::endl;
  * solve();
  * pout << "done" << std::endl;
@@ -88,25 +86,27 @@ public:
    * based on which writes are actually forwarded. Per default the condition
    * of an object is active.
    */
-  ConditionalOStream (std::ostream &stream,
-                      const bool    active = true);
+  ConditionalOStream(std::ostream &stream, const bool active = true);
 
   /**
    * Depending on the <tt>active</tt> flag set the condition of this stream to
    * active (true) or non-active (false). An object of this class prints to
    * <tt>cout</tt> if and only if its condition is active.
    */
-  void set_condition (const bool active);
+  void
+  set_condition(const bool active);
 
   /**
    * Return the condition of the object.
    */
-  bool is_active() const;
+  bool
+  is_active() const;
 
   /**
    * Return a reference to the stream currently in use.
    */
-  std::ostream &get_stream () const;
+  std::ostream &
+  get_stream() const;
 
   /**
    * Output a constant something through this stream. This function must be @p
@@ -115,7 +115,7 @@ public:
    */
   template <typename T>
   const ConditionalOStream &
-  operator << (const T &t) const;
+  operator<<(const T &t) const;
 
   /**
    * Treat ostream manipulators. This function must be @p const so that member
@@ -127,13 +127,13 @@ public:
    * and can't be bound directly to a template type.
    */
   const ConditionalOStream &
-  operator<< (std::ostream& (*p) (std::ostream &)) const;
+  operator<<(std::ostream &(*p)(std::ostream &)) const;
 
 private:
   /**
    * Reference to the stream we want to write to.
    */
-  std::ostream  &output_stream;
+  std::ostream &output_stream;
 
   /**
    * Stores the actual condition the object is in.
@@ -145,9 +145,8 @@ private:
 // --------------------------- inline and template functions -----------
 
 template <class T>
-inline
-const ConditionalOStream &
-ConditionalOStream::operator<< (const T &t) const
+inline const ConditionalOStream &
+ConditionalOStream::operator<<(const T &t) const
 {
   if (active_flag == true)
     output_stream << t;
@@ -156,9 +155,8 @@ ConditionalOStream::operator<< (const T &t) const
 }
 
 
-inline
-const ConditionalOStream &
-ConditionalOStream::operator<< (std::ostream& (*p) (std::ostream &)) const
+inline const ConditionalOStream &
+ConditionalOStream::operator<<(std::ostream &(*p)(std::ostream &)) const
 {
   if (active_flag == true)
     output_stream << p;
@@ -167,9 +165,8 @@ ConditionalOStream::operator<< (std::ostream& (*p) (std::ostream &)) const
 }
 
 
-inline
-std::ostream &
-ConditionalOStream::get_stream () const
+inline std::ostream &
+ConditionalOStream::get_stream() const
 {
   return output_stream;
 }
