@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2013 - 2019 by the deal.II authors
+// Copyright (C) 2013 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,6 +16,8 @@
 #ifndef dealii_derivative_form_h
 #define dealii_derivative_form_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/tensor.h>
 
 DEAL_II_NAMESPACE_OPEN
@@ -218,7 +220,7 @@ template <int order, int dim, int spacedim, typename Number>
 inline Tensor<order, dim, Number> &
   DerivativeForm<order, dim, spacedim, Number>::operator[](const unsigned int i)
 {
-  Assert(i < spacedim, ExcIndexRange(i, 0, spacedim));
+  AssertIndexRange(i, spacedim);
 
   return tensor[i];
 }
@@ -230,7 +232,7 @@ inline const Tensor<order, dim, Number> &
   DerivativeForm<order, dim, spacedim, Number>::
   operator[](const unsigned int i) const
 {
-  Assert(i < spacedim, ExcIndexRange(i, 0, spacedim));
+  AssertIndexRange(i, spacedim);
 
   return tensor[i];
 }
@@ -344,12 +346,12 @@ DerivativeForm<order, dim, spacedim, Number>::covariant_form() const
     {
       const Tensor<2, dim, Number> DF_t =
         dealii::transpose(invert(static_cast<Tensor<2, dim, Number>>(*this)));
-      return DerivativeForm<1, dim, spacedim>(DF_t);
+      return DerivativeForm<1, dim, spacedim, Number>(DF_t);
     }
   else
     {
-      const DerivativeForm<1, spacedim, dim> DF_t = this->transpose();
-      Tensor<2, dim, Number>                 G; // First fundamental form
+      const DerivativeForm<1, spacedim, dim, Number> DF_t = this->transpose();
+      Tensor<2, dim, Number> G; // First fundamental form
       for (unsigned int i = 0; i < dim; ++i)
         for (unsigned int j = 0; j < dim; ++j)
           G[i][j] = DF_t[i] * DF_t[j];
@@ -415,11 +417,11 @@ apply_transformation(const DerivativeForm<1, dim, spacedim, Number> &grad_F,
  */
 // rank=2
 template <int spacedim, int dim, typename Number>
-inline DerivativeForm<1, spacedim, dim>
+inline DerivativeForm<1, spacedim, dim, Number>
 apply_transformation(const DerivativeForm<1, dim, spacedim, Number> &grad_F,
                      const Tensor<2, dim, Number> &                  D_X)
 {
-  DerivativeForm<1, spacedim, dim> dest;
+  DerivativeForm<1, spacedim, dim, Number> dest;
   for (unsigned int i = 0; i < dim; ++i)
     dest[i] = apply_transformation(grad_F, D_X[i]);
 
