@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -17,8 +17,8 @@
 #define dealii_geometric_utilities_h
 
 #include <deal.II/base/config.h>
-#include <deal.II/base/point.h>
 
+#include <deal.II/base/point.h>
 
 #include <array>
 
@@ -27,22 +27,20 @@ DEAL_II_NAMESPACE_OPEN
 
 
 /**
- * A namespace for geometric utility functions that are not particularly specific
- * to finite element computing or numerical programs, but nevertheless are needed
- * in various contexts when writing applications.
+ * A namespace for geometric utility functions that are not particularly
+ * specific to finite element computing or numerical programs, but nevertheless
+ * are needed in various contexts when writing applications.
  *
  * @ingroup utilities
  * @author Denis Davydov, 2016
  */
 namespace GeometricUtilities
 {
-
   /**
    * A namespace for coordinate transformations.
    */
   namespace Coordinates
   {
-
     /**
      * Return spherical coordinates of a Cartesian point @p point.
      * The returned array is filled with radius, azimuth angle $\in [0,2 \pi)$
@@ -56,7 +54,7 @@ namespace GeometricUtilities
      * @f}
      */
     template <int dim>
-    std::array<double,dim>
+    std::array<double, dim>
     to_spherical(const Point<dim> &point);
 
     /**
@@ -74,10 +72,10 @@ namespace GeometricUtilities
      */
     template <std::size_t dim>
     Point<dim>
-    from_spherical(const std::array<double,dim> &scoord);
+    from_spherical(const std::array<double, dim> &scoord);
 
-  }
-}
+  } // namespace Coordinates
+} // namespace GeometricUtilities
 
 DEAL_II_NAMESPACE_CLOSE
 
