@@ -1,7 +1,7 @@
 
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2013 - 2019 by the deal.II authors
+// Copyright (C) 2013 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -24,13 +24,10 @@
 
 #  include <deal.II/lac/sparsity_tools.h>
 
-DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
-#  include <boost/unordered_map.hpp>
-#  include <boost/unordered_set.hpp>
-DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
-
 #  include <functional>
 #  include <set>
+#  include <unordered_map>
+#  include <unordered_set>
 #  include <vector>
 
 
@@ -121,7 +118,7 @@ namespace GraphColoring
       unsigned int n_iterators = 0;
 
       // Create a map from conflict indices to iterators
-      boost::unordered_map<types::global_dof_index, std::vector<Iterator>>
+      std::unordered_map<types::global_dof_index, std::vector<Iterator>>
         indices_to_iterators;
       for (Iterator it = begin; it != end; ++it)
         {
@@ -202,12 +199,15 @@ namespace GraphColoring
 
     /**
      * This function uses DSATUR (Degree SATURation) to color the elements of
-     * a set. DSATUR works as follows: -# Arrange the vertices by decreasing
-     * order of degrees. -# Color a vertex of maximal degree with color 1. -#
-     * Choose a vertex with a maximal saturation degree. If there is equality,
-     * choose any vertex of maximal degree in the uncolored subgraph. -# Color
-     * the chosen vertex with the least possible (lowest numbered) color. -#
-     * If all the vertices are colored, stop. Otherwise, return to 3.
+     * a set. DSATUR works as follows:
+     * -# Arrange the vertices by decreasing order of degrees.
+     * -# Color a vertex of maximal degree with color 1.
+     * -# Choose a vertex with a maximal saturation degree. If there is
+     *    equality, choose any vertex of maximal degree in the uncolored
+     *    subgraph.
+     * -# Color the chosen vertex with the least possible (lowest numbered)
+     *    color.
+     * -# If all the vertices are colored, stop. Otherwise, return to 3.
      *
      * @param[in] partition The set of iterators that should be colored.
      * @param[in] get_conflict_indices A user defined function object
@@ -273,7 +273,7 @@ namespace GraphColoring
         }
 
       // Color the graph.
-      std::vector<boost::unordered_set<unsigned int>> colors_used;
+      std::vector<std::unordered_set<unsigned int>> colors_used;
       for (unsigned int i = 0; i < partition_size; ++i)
         {
           const unsigned int current_vertex(sorted_vertices[i]);
@@ -306,7 +306,7 @@ namespace GraphColoring
             {
               partition_coloring.push_back(
                 std::vector<Iterator>(1, partition[current_vertex]));
-              boost::unordered_set<unsigned int> tmp;
+              std::unordered_set<unsigned int> tmp;
               tmp.insert(current_vertex);
               colors_used.push_back(tmp);
             }
@@ -363,7 +363,7 @@ namespace GraphColoring
         {
           if (i != i_color)
             {
-              boost::unordered_set<unsigned int> used_k;
+              std::unordered_set<unsigned int> used_k;
               for (unsigned int j = 0; j < colors_counter[i].size(); ++j)
                 {
                   // Find the color in the current partition with the largest
@@ -418,7 +418,7 @@ namespace GraphColoring
             {
               if (i != i_color)
                 {
-                  boost::unordered_set<unsigned int> used_k;
+                  std::unordered_set<unsigned int> used_k;
                   for (unsigned int j = 0; j < colors_counter[i].size(); ++j)
                     {
                       // Find the color in the current partition with the
