@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2018 by the deal.II authors
+// Copyright (C) 2003 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,6 +16,8 @@
 #ifndef dealii_mg_level_object_h
 #define dealii_mg_level_object_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/subscriptor.h>
 
 #include <memory>
@@ -106,20 +108,6 @@ public:
   MGLevelObject<Object> &
   operator=(const double d);
 
-  /**
-   * Call @p clear on all objects stored by this object. This function
-   * is only implemented for some @p Object classes, e.g., matrix
-   * types or the PreconditionBlockSOR and similar classes. Using this
-   * function will fail with a compiler error if the @p Object
-   * template type to this class does not provide a
-   * <code>clear()</code> member function.
-   *
-   * @deprecated Use clear_elements () instead
-   */
-  DEAL_II_DEPRECATED
-  void
-  clear();
-
   /**
    * Call @p clear on all objects stored by this object. This function
    * is only implemented for some @p Object classes, e.g., matrix
@@ -235,15 +223,6 @@ MGLevelObject<Object>::operator=(const double d)
 }
 
 
-template <class Object>
-void
-MGLevelObject<Object>::clear() // DEPRECATED
-{
-  // Avoid code duplication in deprecated call by calling replacing function
-  clear_elements();
-}
-
-
 template <class Object>
 void
 MGLevelObject<Object>::clear_elements()
