@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2019 by the deal.II authors
+// Copyright (C) 2011 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,10 +19,12 @@
 #include <deal.II/base/config.h>
 
 #include <deal.II/base/array_view.h>
+#include <deal.II/base/mpi_tags.h>
 #include <deal.II/base/numbers.h>
 
 #include <map>
 #include <numeric>
+#include <set>
 #include <vector>
 
 #if !defined(DEAL_II_WITH_MPI) && !defined(DEAL_II_WITH_PETSC)
@@ -30,6 +32,7 @@
 // some constructs with MPI data
 // types. Therefore, create some dummies
 using MPI_Comm     = int;
+using MPI_Request  = int;
 using MPI_Datatype = int;
 using MPI_Op       = int;
 #  ifndef MPI_COMM_WORLD
@@ -38,6 +41,9 @@ using MPI_Op       = int;
 #  ifndef MPI_COMM_SELF
 #    define MPI_COMM_SELF 0
 #  endif
+#  ifndef MPI_REQUEST_NULL
+#    define MPI_REQUEST_NULL 0
+#  endif
 #  ifndef MPI_MIN
 #    define MPI_MIN 0
 #  endif
@@ -86,6 +92,7 @@ DEAL_II_NAMESPACE_OPEN
 
 
 // Forward type declarations to allow MPI sums over tensorial types
+#ifndef DOXYGEN
 template <int rank, int dim, typename Number>
 class Tensor;
 template <int rank, int dim, typename Number>
@@ -93,9 +100,27 @@ class SymmetricTensor;
 template <typename Number>
 class SparseMatrix;
 class IndexSet;
+#endif
 
 namespace Utilities
 {
+  /**
+   * Given the total number of elements @p total_size, create an evenly
+   * distributed 1:1 partitioning of the elements for across @p n_partitions.
+   * The local sizes will be equal to the @p total_size divided by the number
+   * of partitions plus the remainder being divided amongst the first
+   * processes. Each process will store a contiguous subset of indices, and the
+   * index set on process p+1 starts at the index one larger than the last one
+   * stored on process p.
+   * For example, a @p total_size of 11 with 3 processes will result
+   * in the IndexSets { [0,4), [4,8), [8,11)] }, and this function will
+   * return the @p my_partition_id 's IndexSet.
+   */
+  IndexSet
+  create_evenly_distributed_partitioning(const unsigned int my_partition_id,
+                                         const unsigned int n_partitions,
+                                         const IndexSet::size_type total_size);
+
   /**
    * A namespace for utility functions that abstract certain operations using
    * the Message Passing Interface (MPI) or provide fallback operations in
@@ -188,11 +213,194 @@ namespace Utilities
      * can interact without interfering with each other.
      *
      * When no longer needed, the communicator created here needs to be
-     * destroyed using <code>MPI_Comm_free</code>.
+     * destroyed using free_communicator().
+     *
+     * This function is equivalent to calling
+     * <code>MPI_Comm_dup(mpi_communicator, &return_value);</code>.
      */
     MPI_Comm
     duplicate_communicator(const MPI_Comm &mpi_communicator);
 
+    /**
+     * Free the given
+     * @ref GlossMPICommunicator "communicator"
+     * @p mpi_communicator that was duplicated using duplicate_communicator().
+     *
+     * The argument is passed by reference and will be invalidated and set to
+     * the MPI null handle. This function is equivalent to calling
+     * <code>MPI_Comm_free(&mpi_communicator);</code>.
+     */
+    void
+    free_communicator(MPI_Comm &mpi_communicator);
+
+    /**
+     * Helper class to automatically duplicate and free an MPI
+     * @ref GlossMPICommunicator "communicator".
+     *
+     * This class duplicates the communicator given in the constructor
+     * using duplicate_communicator() and frees it automatically when
+     * this object gets destroyed by calling free_communicator(). You
+     * can access the wrapped communicator using operator*.
+     *
+     * This class exists to easily allow duplicating communicators without
+     * having to worry when and how to free it after usage.
+     */
+    class DuplicatedCommunicator
+    {
+    public:
+      /**
+       * Create a duplicate of the given @p communicator.
+       */
+      explicit DuplicatedCommunicator(const MPI_Comm &communicator)
+        : comm(duplicate_communicator(communicator))
+      {}
+
+      /**
+       * Do not allow making copies.
+       */
+      DuplicatedCommunicator(const DuplicatedCommunicator &) = delete;
+
+      /**
+       * The destructor will free the communicator automatically.
+       */
+      ~DuplicatedCommunicator()
+      {
+        free_communicator(comm);
+      }
+
+      /**
+       * Access the stored communicator.
+       */
+      const MPI_Comm &operator*() const
+      {
+        return comm;
+      }
+
+
+      /**
+       * Do not allow assignment of this class.
+       */
+      DuplicatedCommunicator &
+      operator=(const DuplicatedCommunicator &) = delete;
+
+    private:
+      /**
+       * The communicator of course.
+       */
+      MPI_Comm comm;
+    };
+
+    /**
+     * This class represents a mutex to guard a critical section for a set of
+     * processors in a parallel computation using MPI.
+     *
+     * The lock() commands waits until all MPI ranks in the communicator have
+     * released a previous lock using unlock().
+     *
+     * A typical usage involves guarding a critical section using a lock guard:
+     * @code
+     * {
+     *   static CollectiveMutex      mutex;
+     *   CollectiveMutex::ScopedLock lock(mutex, comm);
+     *   // [ critical code to be guarded]
+     * }
+     * @endcode
+     *
+     * Here, the critical code will finish on all processors before the mutex
+     * can be acquired again (for example by a second execution of the block
+     * above. The critical code block typically involves MPI communication that
+     * would yield incorrect results without the lock. For example, if the code
+     * contains nonblocking receives with MPI_ANY_SOURCE, packets can be
+     * confused between iterations.
+     *
+     * Note that the mutex needs to be the same instance between calls to the
+     * same critical region. While not required, this can be achieved by making
+     * the instance static (like in the example above). The variable can also be
+     * a global variable, or a member variable of the object to which the
+     * executing function belongs.
+     */
+    class CollectiveMutex
+    {
+    public:
+      /**
+       * This helper class provides a scoped lock for the CollectiveMutex.
+       *
+       * See the class documentation of CollectiveMutex for details.
+       */
+      class ScopedLock
+      {
+      public:
+        /**
+         * Constructor. Blocks until it can acquire the lock.
+         */
+        explicit ScopedLock(CollectiveMutex &mutex, const MPI_Comm &comm)
+          : mutex(mutex)
+          , comm(comm)
+        {
+          mutex.lock(comm);
+        }
+
+        /**
+         * Destructor. Releases the lock.
+         */
+        ~ScopedLock()
+        {
+          mutex.unlock(comm);
+        }
+
+      private:
+        /**
+         * A reference to the mutex.
+         */
+        CollectiveMutex &mutex;
+        /**
+         * The communicator.
+         */
+        const MPI_Comm comm;
+      };
+
+      /**
+       * Constructor of this class.
+       */
+      explicit CollectiveMutex();
+
+      /**
+       * Destroy the mutex. Assumes the lock is not currently held.
+       */
+      ~CollectiveMutex();
+
+      /**
+       * Acquire the mutex and, if necessary, wait until we can do so.
+       *
+       * This is a collective call that needs to be executed by all processors
+       * in the communicator.
+       */
+      void
+      lock(MPI_Comm comm);
+
+      /**
+       * Release the lock.
+       *
+       * This is a collective call that needs to be executed by all processors
+       * in the communicator.
+       */
+      void
+      unlock(MPI_Comm comm);
+
+    private:
+      /**
+       * Keep track if we have this lock right now.
+       */
+      bool locked;
+
+      /**
+       * The request to keep track of the non-blocking barrier.
+       */
+      MPI_Request request;
+    };
+
+
+
     /**
      * If @p comm is an intracommunicator, this function returns a new
      * communicator @p newcomm with communication group defined by the
@@ -237,8 +445,20 @@ namespace Utilities
      * process p.
      */
     std::vector<IndexSet>
-    create_ascending_partitioning(const MPI_Comm &           comm,
-                                  const IndexSet::size_type &local_size);
+    create_ascending_partitioning(const MPI_Comm &          comm,
+                                  const IndexSet::size_type local_size);
+
+    /**
+     * Given the total number of elements @p total_size, create an evenly
+     * distributed 1:1 partitioning of the elements across the
+     * MPI communicator @p comm.
+     * Uses @p comm to determine number of partitions and processor ID to call the
+     * @p create_evenly_distributed_partitioning() function above.
+     */
+    IndexSet
+    create_evenly_distributed_partitioning(
+      const MPI_Comm &          comm,
+      const IndexSet::size_type total_size);
 
 #ifdef DEAL_II_WITH_MPI
     /**
@@ -528,6 +748,40 @@ namespace Utilities
     MinMaxAvg
     min_max_avg(const double my_value, const MPI_Comm &mpi_communicator);
 
+    /**
+     * Same as above but returning the sum, average, minimum, maximum,
+     * process id of minimum and maximum as a collective operation on the
+     * given MPI
+     * @ref GlossMPICommunicator "communicator"
+     * @p mpi_communicator for each entry of the vector.
+     *
+     * @note This function performs a single reduction sweep.
+     *
+     * @pre Size of the input vector has to be the same on all processes.
+     */
+    std::vector<MinMaxAvg>
+    min_max_avg(const std::vector<double> &my_value,
+                const MPI_Comm &           mpi_communicator);
+
+
+    /**
+     * Same as above but returning the sum, average, minimum, maximum,
+     * process id of minimum and maximum as a collective operation on the
+     * given MPI
+     * @ref GlossMPICommunicator "communicator"
+     * @p mpi_communicator for each entry of the ArrayView.
+     *
+     * @note This function performs a single reduction sweep.
+     *
+     * @pre Size of the input ArrayView has to be the same on all processes
+     *   and the input and output ArrayVew have to have the same size.
+     */
+    void
+    min_max_avg(const ArrayView<const double> &my_values,
+                const ArrayView<MinMaxAvg> &   result,
+                const MPI_Comm &               mpi_communicator);
+
+
     /**
      * A class that is used to initialize the MPI system at the beginning of a
      * program and to shut it down again at the end. It also allows you to
@@ -629,6 +883,47 @@ namespace Utilities
        * MPI process.
        */
       ~MPI_InitFinalize();
+
+      /**
+       * Register a reference to an MPI_Request
+       * on which we need to call `MPI_Wait` before calling `MPI_Finalize`.
+       *
+       * The object @p request needs to exist when MPI_Finalize is called, which means the
+       * request is typically statically allocated. Otherwise, you need to call
+       * unregister_request() before the request goes out of scope. Note that it
+       * is acceptable for a request to be already waited on (and consequently
+       * reset to MPI_REQUEST_NULL).
+       *
+       * It is acceptable to call this function more than once with the same
+       * instance (as it is done in the example below).
+       *
+       * Typically, this function is used by CollectiveMutex and not directly,
+       * but it can also be used directly like this:
+       * @code
+       * void my_fancy_communication()
+       * {
+       *   static MPI_Request request = MPI_REQUEST_NULL;
+       *   MPI_InitFinalize::register_request(request);
+       *   MPI_Wait(&request, MPI_STATUS_IGNORE);
+       *   // [some algorithm that is not safe to be executed twice in a row.]
+       *   MPI_IBarrier(comm, &request);
+       * }
+       * @endcode
+       */
+      static void
+      register_request(MPI_Request &request);
+
+      /**
+       * Unregister a request previously added using register_request().
+       */
+      static void
+      unregister_request(MPI_Request &request);
+
+    private:
+      /**
+       * Requests to MPI_Wait before finalizing
+       */
+      static std::set<MPI_Request *> requests;
     };
 
     /**
@@ -653,8 +948,11 @@ namespace Utilities
      * @param[in] comm MPI communicator.
      *
      * @param[in] objects_to_send A map from the rank (unsigned int) of the
-     *  process meant to receive the data and the object to send (the type T
-     *  must be serializable for this function to work properly).
+     *  process meant to receive the data and the object to send (the type `T`
+     *  must be serializable for this function to work properly). If this
+     *  map contains an entry with a key equal to the rank of the current
+     *  process (i.e., an instruction to a process to send data to itself),
+     *  then this data item is simply copied to the returned object.
      *
      * @return A map from the rank (unsigned int) of the process
      *  which sent the data and object received.
@@ -708,6 +1006,73 @@ namespace Utilities
            const T &          object_to_send,
            const unsigned int root_process = 0);
 
+    /**
+     * Given a partitioned index set space, compute the owning MPI process rank
+     * of each element of a second index set according to the partitioned index
+     * set. A natural usage of this function is to compute for each ghosted
+     * degree of freedom the MPI rank of the process owning that index.
+     *
+     * One might think: "But we know which rank a ghost DoF belongs to based on
+     * the subdomain id of the cell it is on". But this heuristic fails for DoFs
+     * on interfaces between ghost cells with different subdomain_ids, or
+     * between a ghost cell and an artificial cell. Furthermore, this function
+     * enables a completely abstract exchange of information without the help of
+     * the mesh in terms of neighbors.
+     *
+     * The first argument passed to this function, @p owned_indices, must
+     * uniquely partition an index space between all processes.
+     * Otherwise, there are no limitations on this argument: In particular,
+     * there is no need in partitioning
+     * the index space into contiguous subsets. Furthermore, there are no
+     * limitations
+     * on the second index set @p indices_to_look_up as long as the size matches
+     * the first one. It can be chosen arbitrarily and independently on each
+     * process. In the case that the second index set also contains locally
+     * owned indices, these indices will be treated correctly and the rank of
+     * this process is returned for those entries.
+     *
+     * @note This is a collective operation: all processes within the given
+     * communicator have to call this function. Since this function does not
+     * use MPI_Alltoall or MPI_Allgather, but instead uses non-blocking
+     * point-to-point communication instead, and only a single non-blocking
+     * barrier, it reduces the memory consumption significantly. This function
+     * is suited for large-scale simulations with >100k MPI ranks.
+     *
+     * @param[in] owned_indices Index set with indices locally owned by this
+     *            process.
+     * @param[in] indices_to_look_up Index set containing indices of which the
+     *            user is interested the rank of the owning process.
+     * @param[in] comm MPI communicator.
+     *
+     * @return List containing the MPI process rank for each entry in the index
+     *         set @p indices_to_look_up. The order coincides with the order
+     *         within the ElementIterator.
+     *
+     * @author Peter Munch, 2019
+     */
+    std::vector<unsigned int>
+    compute_index_owner(const IndexSet &owned_indices,
+                        const IndexSet &indices_to_look_up,
+                        const MPI_Comm &comm);
+
+    /**
+     * Compute the union of the input vectors @p vec of all processes in the
+     *   MPI communicator @p comm.
+     *
+     * @note This is a collective operation. The result will available on all
+     *   processes.
+     */
+    template <typename T>
+    std::vector<T>
+    compute_set_union(const std::vector<T> &vec, const MPI_Comm &comm);
+
+    /**
+     * The same as above but for std::set.
+     */
+    template <typename T>
+    std::set<T>
+    compute_set_union(const std::set<T> &set, const MPI_Comm &comm);
+
 #ifndef DOXYGEN
     // declaration for an internal function that lives in mpi.templates.h
     namespace internal
@@ -758,25 +1123,41 @@ namespace Utilities
     {
 #  ifndef DEAL_II_WITH_MPI
       (void)comm;
-      Assert(objects_to_send.size() == 0,
+      Assert(objects_to_send.size() < 2,
              ExcMessage("Cannot send to more than one processor."));
       Assert(objects_to_send.find(0) != objects_to_send.end() ||
                objects_to_send.size() == 0,
              ExcMessage("Can only send to myself or to nobody."));
       return objects_to_send;
 #  else
+      const auto my_proc = this_mpi_process(comm);
 
-      std::vector<unsigned int> send_to(objects_to_send.size());
-      {
-        unsigned int i = 0;
-        for (const auto &m : objects_to_send)
-          send_to[i++] = m.first;
-      }
-      AssertDimension(send_to.size(), objects_to_send.size());
+      std::map<unsigned int, T> received_objects;
 
-      const auto receive_from =
-        Utilities::MPI::compute_point_to_point_communication_pattern(comm,
-                                                                     send_to);
+      std::vector<unsigned int> send_to;
+      send_to.reserve(objects_to_send.size());
+      for (const auto &m : objects_to_send)
+        if (m.first == my_proc)
+          received_objects[my_proc] = m.second;
+        else
+          send_to.emplace_back(m.first);
+
+      const unsigned int n_point_point_communications =
+        Utilities::MPI::compute_n_point_to_point_communications(comm, send_to);
+
+      // Protect the following communication:
+      static CollectiveMutex      mutex;
+      CollectiveMutex::ScopedLock lock(mutex, comm);
+
+      // If we have something to send, or we expect something from other
+      // processors, we need to visit one of the two scopes below. Otherwise,
+      // no other action is required by this mpi process, and we can safely
+      // return.
+      if (send_to.size() == 0 && n_point_point_communications == 0)
+        return received_objects;
+
+      const int mpi_tag =
+        internal::Tags::compute_point_to_point_communication_pattern;
 
       // Sending buffers
       std::vector<std::vector<char>> buffers_to_send(send_to.size());
@@ -784,31 +1165,31 @@ namespace Utilities
       {
         unsigned int i = 0;
         for (const auto &rank_obj : objects_to_send)
-          {
-            const auto &rank   = rank_obj.first;
-            buffers_to_send[i] = Utilities::pack(rank_obj.second);
-            const int ierr     = MPI_Isend(buffers_to_send[i].data(),
-                                       buffers_to_send[i].size(),
-                                       MPI_CHAR,
-                                       rank,
-                                       21,
-                                       comm,
-                                       &buffer_send_requests[i]);
-            AssertThrowMPI(ierr);
-            ++i;
-          }
+          if (rank_obj.first != my_proc)
+            {
+              const auto &rank   = rank_obj.first;
+              buffers_to_send[i] = Utilities::pack(rank_obj.second);
+              const int ierr     = MPI_Isend(buffers_to_send[i].data(),
+                                         buffers_to_send[i].size(),
+                                         MPI_CHAR,
+                                         rank,
+                                         mpi_tag,
+                                         comm,
+                                         &buffer_send_requests[i]);
+              AssertThrowMPI(ierr);
+              ++i;
+            }
       }
 
-      // Receiving buffers
-      std::map<unsigned int, T> received_objects;
+      // Fill the output map
       {
         std::vector<char> buffer;
         // We do this on a first come/first served basis
-        for (unsigned int i = 0; i < receive_from.size(); ++i)
+        for (unsigned int i = 0; i < n_point_point_communications; ++i)
           {
             // Probe what's going on. Take data from the first available sender
             MPI_Status status;
-            int        ierr = MPI_Probe(MPI_ANY_SOURCE, 21, comm, &status);
+            int        ierr = MPI_Probe(MPI_ANY_SOURCE, mpi_tag, comm, &status);
             AssertThrowMPI(ierr);
 
             // Length of the message
@@ -821,8 +1202,13 @@ namespace Utilities
             const unsigned int rank = status.MPI_SOURCE;
 
             // Actually receive the message
-            ierr = MPI_Recv(
-              buffer.data(), len, MPI_CHAR, rank, 21, comm, MPI_STATUS_IGNORE);
+            ierr = MPI_Recv(buffer.data(),
+                            len,
+                            MPI_CHAR,
+                            status.MPI_SOURCE,
+                            status.MPI_TAG,
+                            comm,
+                            MPI_STATUS_IGNORE);
             AssertThrowMPI(ierr);
             Assert(received_objects.find(rank) == received_objects.end(),
                    ExcInternalError(
@@ -832,9 +1218,10 @@ namespace Utilities
       }
 
       // Wait to have sent all objects.
-      MPI_Waitall(send_to.size(),
-                  buffer_send_requests.data(),
-                  MPI_STATUSES_IGNORE);
+      const int ierr = MPI_Waitall(send_to.size(),
+                                   buffer_send_requests.data(),
+                                   MPI_STATUSES_IGNORE);
+      AssertThrowMPI(ierr);
 
       return received_objects;
 #  endif // deal.II with MPI
@@ -844,6 +1231,9 @@ namespace Utilities
     std::vector<T>
     all_gather(const MPI_Comm &comm, const T &object)
     {
+      if (job_supports_mpi() == false)
+        return {object};
+
 #  ifndef DEAL_II_WITH_MPI
       (void)comm;
       std::vector<T> v(1, object);
@@ -911,7 +1301,7 @@ namespace Utilities
       const auto n_procs = dealii::Utilities::MPI::n_mpi_processes(comm);
       const auto my_rank = dealii::Utilities::MPI::this_mpi_process(comm);
 
-      Assert(root_process < n_procs, ExcIndexRange(root_process, 0, n_procs));
+      AssertIndexRange(root_process, n_procs);
 
       std::vector<char> buffer       = Utilities::pack(object_to_send);
       int               n_local_data = buffer.size();
