@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2017 - 2019 by the deal.II authors
+// Copyright (C) 2017 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -47,6 +47,7 @@ namespace Utilities
       std::vector<MPI_Request> &                      requests) const
     {
       AssertDimension(temporary_storage.size(), n_import_indices());
+      AssertIndexRange(communication_channel, 200);
       Assert(ghost_array.size() == n_ghost_indices() ||
                ghost_array.size() == n_ghost_indices_in_larger_set,
              ExcGhostIndexArrayHasWrongSize(ghost_array.size(),
@@ -63,6 +64,12 @@ namespace Utilities
              ExcMessage("Another operation seems to still be running. "
                         "Call update_ghost_values_finish() first."));
 
+      const unsigned int mpi_tag =
+        Utilities::MPI::internal::Tags::partitioner_export_start +
+        communication_channel;
+      Assert(mpi_tag <= Utilities::MPI::internal::Tags::partitioner_export_end,
+             ExcInternalError());
+
       // Need to send and receive the data. Use non-blocking communication,
       // where it is usually less overhead to first initiate the receive and
       // then actually send the data
@@ -90,7 +97,7 @@ namespace Utilities
                       ghost_targets_data[i].second * sizeof(Number),
                       MPI_BYTE,
                       ghost_targets_data[i].first,
-                      ghost_targets_data[i].first + communication_channel,
+                      mpi_tag,
                       communicator,
                       &requests[i]);
           AssertThrowMPI(ierr);
@@ -126,6 +133,7 @@ namespace Utilities
                   import_indices_plain_dev[i].first.get(),
                   locally_owned_array.data(),
                   chunk_size);
+              cudaDeviceSynchronize();
             }
           else
 #    endif
@@ -158,7 +166,7 @@ namespace Utilities
                       import_targets_data[i].second * sizeof(Number),
                       MPI_BYTE,
                       import_targets_data[i].first,
-                      my_pid + communication_channel,
+                      mpi_tag,
                       communicator,
                       &requests[n_ghost_targets + i]);
           AssertThrowMPI(ierr);
@@ -263,6 +271,7 @@ namespace Utilities
       std::vector<MPI_Request> &                requests) const
     {
       AssertDimension(temporary_storage.size(), n_import_indices());
+      AssertIndexRange(communication_channel, 200);
       Assert(ghost_array.size() == n_ghost_indices() ||
                ghost_array.size() == n_ghost_indices_in_larger_set,
              ExcGhostIndexArrayHasWrongSize(ghost_array.size(),
@@ -297,8 +306,11 @@ namespace Utilities
       // where it is generally less overhead to first initiate the receive and
       // then actually send the data
 
-      // set channels in different range from update_ghost_values channels
-      const unsigned int channel = communication_channel + 401;
+      const unsigned int mpi_tag =
+        Utilities::MPI::internal::Tags::partitioner_import_start +
+        communication_channel;
+      Assert(mpi_tag <= Utilities::MPI::internal::Tags::partitioner_import_end,
+             ExcInternalError());
       requests.resize(n_import_targets + n_ghost_targets);
 
       // initiate the receive operations
@@ -317,7 +329,7 @@ namespace Utilities
                       import_targets_data[i].second * sizeof(Number),
                       MPI_BYTE,
                       import_targets_data[i].first,
-                      import_targets_data[i].first + channel,
+                      mpi_tag,
                       communicator,
                       &requests[i]);
           AssertThrowMPI(ierr);
@@ -400,12 +412,17 @@ namespace Utilities
             ExcMessage("Index overflow: Maximum message size in MPI is 2GB. "
                        "The number of ghost entries times the size of 'Number' "
                        "exceeds this value. This is not supported."));
+#    if defined(DEAL_II_COMPILER_CUDA_AWARE) && \
+      defined(DEAL_II_MPI_WITH_CUDA_SUPPORT)
+          if (std::is_same<MemorySpaceType, MemorySpace::CUDA>::value)
+            cudaDeviceSynchronize();
+#    endif
           const int ierr =
             MPI_Isend(ghost_array_ptr,
                       ghost_targets_data[i].second * sizeof(Number),
                       MPI_BYTE,
                       ghost_targets_data[i].first,
-                      this_mpi_process() + channel,
+                      mpi_tag,
                       communicator,
                       &requests[n_import_targets + i]);
           AssertThrowMPI(ierr);
@@ -507,18 +524,30 @@ namespace Utilities
                    "vector_operation argument was passed to "
                    "import_from_ghosted_array_start as is passed "
                    "to import_from_ghosted_array_finish."));
-#      ifdef DEAL_II_WITH_CXX17
-          if constexpr (std::is_trivial<Number>::value)
-#      else
-          if (std::is_trivial<Number>::value)
-#      endif
-            std::memset(ghost_array.data(),
-                        0,
-                        sizeof(Number) * ghost_array.size());
+
+#      if defined(DEAL_II_COMPILER_CUDA_AWARE)
+          if (std::is_same<MemorySpaceType, MemorySpace::CUDA>::value)
+            {
+              cudaMemset(ghost_array.data(),
+                         0,
+                         sizeof(Number) * ghost_array.size());
+            }
           else
-            std::fill(ghost_array.data(),
-                      ghost_array.data() + ghost_array.size(),
-                      0);
+#      endif
+            {
+#      ifdef DEAL_II_WITH_CXX17
+              if constexpr (std::is_trivial<Number>::value)
+#      else
+            if (std::is_trivial<Number>::value)
+#      endif
+                std::memset(ghost_array.data(),
+                            0,
+                            sizeof(Number) * ghost_array.size());
+              else
+                std::fill(ghost_array.data(),
+                          ghost_array.data() + ghost_array.size(),
+                          0);
+            }
           return;
         }
 #    endif
@@ -600,7 +629,7 @@ namespace Utilities
                              100000. *
                              std::numeric_limits<typename numbers::NumberTraits<
                                Number>::real_type>::epsilon(),
-                       typename LinearAlgebra::distributed::Vector<
+                       typename dealii::LinearAlgebra::distributed::Vector<
                          Number>::ExcNonMatchingElements(*read_position,
                                                          locally_owned_array[j],
                                                          my_pid));
@@ -669,17 +698,9 @@ namespace Utilities
             {
               for (auto const &import_indices_plain : import_indices_plain_dev)
                 {
+                  // We can't easily assert here, so we just move the pointer
+                  // matching the host code.
                   const auto chunk_size = import_indices_plain.second;
-                  const int n_blocks =
-                    1 + chunk_size / (::dealii::CUDAWrappers::chunk_size *
-                                      ::dealii::CUDAWrappers::block_size);
-                  dealii::LinearAlgebra::CUDAWrappers::kernel::
-                    set_permutated<<<n_blocks,
-                                     dealii::CUDAWrappers::block_size>>>(
-                      import_indices_plain.first.get(),
-                      locally_owned_array.data(),
-                      read_position,
-                      chunk_size);
                   read_position += chunk_size;
                 }
             }
