@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2019 by the deal.II authors
+// Copyright (C) 1998 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -54,10 +54,12 @@ DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 DEAL_II_NAMESPACE_OPEN
 
 // forward declarations for interfaces and friendship
+#ifndef DOXYGEN
 class LogStream;
 class MultipleParameterLoop;
 template <int dim>
 class FunctionParser;
+#endif
 
 /**
  * Namespace for a few classes that act as patterns for the ParameterHandler
@@ -1409,9 +1411,9 @@ namespace Patterns
      */
     DeclException2(ExcNoMatch,
                    std::string,
-                   const Patterns::PatternBase *,
+                   std::string,
                    << "The string " << arg1 << " does not match the pattern \""
-                   << arg2->description() << "\"");
+                   << arg2 << "\"");
     //@}
   } // namespace Tools
 } // namespace Patterns
@@ -1437,7 +1439,7 @@ namespace Patterns
                   "are derived from PatternBase");
     static_assert(sizeof...(ps) > 0,
                   "The number of PatternTypes must be greater than zero!");
-    auto pattern_pointers = {(static_cast<const PatternBase *>(&ps))...};
+    const auto pattern_pointers = {(static_cast<const PatternBase *>(&ps))...};
     for (const auto p : pattern_pointers)
       patterns.push_back(p->clone());
   }
@@ -1542,7 +1544,8 @@ namespace Patterns
           str << (static_cast<bool>(value) ? "true" : "false");
         else
           str << value;
-        AssertThrow(p->match(str.str()), ExcNoMatch(str.str(), p.get()));
+        AssertThrow(p->match(str.str()),
+                    ExcNoMatch(str.str(), p->description()));
         return str.str();
       }
 
@@ -1551,7 +1554,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &p =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(p->match(s), ExcNoMatch(s, p.get()));
+        AssertThrow(p->match(s), ExcNoMatch(s, p->description()));
         T value;
         if (std::is_same<T, bool>::value)
           value = (s == "true");
@@ -1814,7 +1817,7 @@ namespace Patterns
         for (unsigned int i = 1; i < vec.size(); ++i)
           s += p->get_separator() + " " + vec[i];
 
-        AssertThrow(pattern->match(s), ExcNoMatch(s, p));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, p->description()));
         return s;
       }
 
@@ -1823,7 +1826,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::List *>(pattern.get());
         AssertThrow(p,
@@ -1890,7 +1893,7 @@ namespace Patterns
         for (unsigned int i = 1; i < vec.size(); ++i)
           s += p->get_separator() + " " + vec[i];
 
-        AssertThrow(p->match(s), ExcNoMatch(s, p));
+        AssertThrow(p->match(s), ExcNoMatch(s, p->description()));
         return s;
       }
 
@@ -1899,7 +1902,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::Map *>(pattern.get());
         AssertThrow(p,
@@ -1964,7 +1967,7 @@ namespace Patterns
         for (unsigned int i = 1; i < vec.size(); ++i)
           s += p->get_separator() + " " + vec[i];
 
-        AssertThrow(p->match(s), ExcNoMatch(s, p));
+        AssertThrow(p->match(s), ExcNoMatch(s, p->description()));
         return s;
       }
 
@@ -1973,7 +1976,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::List *>(pattern.get());
         AssertThrow(p,
@@ -2060,7 +2063,7 @@ namespace Patterns
         for (unsigned int i = 1; i < expressions.size(); ++i)
           s = s + p->get_separator() + expressions[i];
 
-        AssertThrow(pattern->match(s), ExcNoMatch(s, p));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, p->description()));
         return s;
       }
 
@@ -2069,7 +2072,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::List *>(pattern.get());
         AssertThrow(p,
@@ -2157,7 +2160,7 @@ namespace Patterns
           p->get_separator() + " " +
           Convert<typename T::value_type>::to_string(t.imag(), base_p);
 
-        AssertThrow(pattern->match(s), ExcNoMatch(s, p));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, p->description()));
         return s;
       }
 
@@ -2169,7 +2172,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::List *>(pattern.get());
         AssertThrow(p,
@@ -2203,7 +2206,7 @@ namespace Patterns
                 const std::unique_ptr<Patterns::PatternBase> &pattern =
                   Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(t), ExcNoMatch(t, pattern.get()));
+        AssertThrow(pattern->match(t), ExcNoMatch(t, pattern->description()));
         return t;
       }
 
@@ -2212,7 +2215,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
         return s;
       }
     };
@@ -2228,15 +2231,10 @@ namespace Patterns
       {
         static_assert(internal::RankInfo<T>::map_rank > 0,
                       "Cannot use this class for non Map-compatible types.");
-        return std_cxx14::make_unique<Patterns::Map>(
+        return std_cxx14::make_unique<Patterns::Tuple>(
+          internal::default_map_separator[internal::RankInfo<T>::map_rank - 1],
           *Convert<Key>::to_pattern(),
-          *Convert<Value>::to_pattern(),
-          1,
-          1,
-          // We keep the same list separator of the previous level, as this is
-          // a map with only 1 possible entry
-          internal::default_list_separator[internal::RankInfo<T>::list_rank],
-          internal::default_map_separator[internal::RankInfo<T>::map_rank - 1]);
+          *Convert<Value>::to_pattern());
       }
 
       static std::string
@@ -2244,10 +2242,9 @@ namespace Patterns
                 const std::unique_ptr<Patterns::PatternBase> &pattern =
                   Convert<T>::to_pattern())
       {
-        std::unordered_map<Key, Value> m;
-        m.insert(t);
-        std::string s = Convert<decltype(m)>::to_string(m, pattern);
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        std::tuple<Key, Value> m(t);
+        std::string            s = Convert<decltype(m)>::to_string(m, pattern);
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
         return s;
       }
 
@@ -2256,9 +2253,9 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        std::unordered_map<Key, Value> m;
+        std::tuple<Key, Value> m;
         m = Convert<decltype(m)>::to_value(s, pattern);
-        return *m.begin();
+        return std::make_pair(std::get<0>(m), std::get<1>(m));
       }
     };
 
@@ -2292,7 +2289,7 @@ namespace Patterns
         std::string str;
         for (unsigned int i = 0; i < string_array.size(); ++i)
           str += (i ? " " + p->get_separator() + " " : "") + string_array[i];
-        AssertThrow(p->match(str), ExcNoMatch(str, p));
+        AssertThrow(p->match(str), ExcNoMatch(str, p->description()));
         return str;
       }
 
@@ -2301,7 +2298,7 @@ namespace Patterns
                const std::unique_ptr<Patterns::PatternBase> &pattern =
                  Convert<T>::to_pattern())
       {
-        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern.get()));
+        AssertThrow(pattern->match(s), ExcNoMatch(s, pattern->description()));
 
         auto p = dynamic_cast<const Patterns::Tuple *>(pattern.get());
         AssertThrow(p,
