@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2019 by the deal.II authors
+// Copyright (C) 1998 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,7 +23,8 @@
 #include <deal.II/base/tensor.h>
 
 DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
-#include <boost/geometry.hpp>
+#include <boost/geometry/core/cs.hpp>
+#include <boost/geometry/geometries/point.hpp>
 DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
 
 #include <cmath>
@@ -113,21 +114,28 @@ public:
   /**
    * Standard constructor. Creates an object that corresponds to the origin,
    * i.e., all coordinates are set to zero.
+   *
+   * @note This function can also be used in CUDA device code.
    */
+  DEAL_II_CUDA_HOST_DEV
   Point();
 
   /**
    * Convert a tensor to a point.
    */
-  explicit Point(const Tensor<1, dim, Number> &);
+  explicit DEAL_II_CUDA_HOST_DEV
+  Point(const Tensor<1, dim, Number> &);
 
   /**
    * Constructor for one dimensional points. This function is only implemented
    * for <tt>dim==1</tt> since the usage is considered unsafe for points with
    * <tt>dim!=1</tt> as it would leave some components of the point
    * coordinates uninitialized.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  explicit Point(const Number x);
+  explicit DEAL_II_CUDA_HOST_DEV
+  Point(const Number x);
 
   /**
    * Constructor for two dimensional points. This function is only implemented
@@ -135,7 +143,10 @@ public:
    * <tt>dim!=2</tt> as it would leave some components of the point
    * coordinates uninitialized (if dim>2) or would not use some arguments (if
    * dim<2).
+   *
+   * @note This function can also be used in CUDA device code.
    */
+  DEAL_II_CUDA_HOST_DEV
   Point(const Number x, const Number y);
 
   /**
@@ -144,7 +155,10 @@ public:
    * points with <tt>dim!=3</tt> as it would leave some components of the
    * point coordinates uninitialized (if dim>3) or would not use some
    * arguments (if dim<3).
+   *
+   * @note This function can also be used in CUDA device code.
    */
+  DEAL_II_CUDA_HOST_DEV
   Point(const Number x, const Number y, const Number z);
 
   /**
@@ -160,21 +174,36 @@ public:
    * Return a unit vector in coordinate direction <tt>i</tt>, i.e., a vector
    * that is zero in all coordinates except for a single 1 in the <tt>i</tt>th
    * coordinate.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  static Point<dim, Number>
-  unit_vector(const unsigned int i);
+  static DEAL_II_CUDA_HOST_DEV Point<dim, Number>
+                               unit_vector(const unsigned int i);
 
   /**
    * Read access to the <tt>index</tt>th coordinate.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Number
-  operator()(const unsigned int index) const;
+  DEAL_II_CUDA_HOST_DEV Number
+                        operator()(const unsigned int index) const;
 
   /**
    * Read and write access to the <tt>index</tt>th coordinate.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Number &
-  operator()(const unsigned int index);
+  DEAL_II_CUDA_HOST_DEV Number &
+                        operator()(const unsigned int index);
+
+  /**
+   * Assignment operator from Tensor<1, dim, Number> with different underlying
+   * scalar type. This obviously requires that the @p OtherNumber type is
+   * convertible to @p Number.
+   */
+  template <typename OtherNumber>
+  Point<dim, Number> &
+  operator=(const Tensor<1, dim, OtherNumber> &p);
 
   /**
    * @name Addition and subtraction of points.
@@ -183,9 +212,11 @@ public:
 
   /**
    * Add an offset given as Tensor<1,dim,Number> to a point.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Point<dim, Number>
-  operator+(const Tensor<1, dim, Number> &) const;
+  DEAL_II_CUDA_HOST_DEV Point<dim, Number>
+                        operator+(const Tensor<1, dim, Number> &) const;
 
   /**
    * Subtract two points, i.e., obtain the vector that connects the two. As
@@ -193,24 +224,30 @@ public:
    * results in a vector anchored at one of the two points (rather than at the
    * origin) and, consequently, the result is returned as a Tensor@<1,dim@>
    * rather than as a Point@<dim@>.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Tensor<1, dim, Number>
-  operator-(const Point<dim, Number> &) const;
+  DEAL_II_CUDA_HOST_DEV Tensor<1, dim, Number>
+                        operator-(const Point<dim, Number> &) const;
 
   /**
    * Subtract a difference vector (represented by a Tensor@<1,dim@>) from the
    * current point. This results in another point and, as discussed in the
    * documentation of this class, the result is then naturally returned as a
    * Point@<dim@> object rather than as a Tensor@<1,dim@>.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Point<dim, Number>
-  operator-(const Tensor<1, dim, Number> &) const;
+  DEAL_II_CUDA_HOST_DEV Point<dim, Number>
+                        operator-(const Tensor<1, dim, Number> &) const;
 
   /**
    * The opposite vector.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Point<dim, Number>
-  operator-() const;
+  DEAL_II_CUDA_HOST_DEV Point<dim, Number>
+                        operator-() const;
 
   /**
    * @}
@@ -224,27 +261,35 @@ public:
   /**
    * Multiply the current point by a factor.
    *
+   * @note This function can also be used in CUDA device code.
+   *
    * @relatesalso EnableIfScalar
    */
   template <typename OtherNumber>
-  Point<dim,
-        typename ProductType<Number,
-                             typename EnableIfScalar<OtherNumber>::type>::type>
+  DEAL_II_CUDA_HOST_DEV Point<
+    dim,
+    typename ProductType<Number,
+                         typename EnableIfScalar<OtherNumber>::type>::type>
   operator*(const OtherNumber) const;
 
   /**
    * Divide the current point by a factor.
+   *
+   * @note This function can also be used in CUDA device code.
    */
   template <typename OtherNumber>
-  Point<dim,
-        typename ProductType<Number,
-                             typename EnableIfScalar<OtherNumber>::type>::type>
+  DEAL_II_CUDA_HOST_DEV Point<
+    dim,
+    typename ProductType<Number,
+                         typename EnableIfScalar<OtherNumber>::type>::type>
   operator/(const OtherNumber) const;
 
   /**
    * Return the scalar product of the vectors representing two points.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  Number operator*(const Tensor<1, dim, Number> &p) const;
+  DEAL_II_CUDA_HOST_DEV Number operator*(const Tensor<1, dim, Number> &p) const;
 
   /**
    * Return the scalar product of this point vector with itself, i.e. the
@@ -255,23 +300,29 @@ public:
    * @note This function is equivalent to
    * Tensor<rank,dim,Number>::norm_square() which returns the square of the
    * Frobenius norm.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  typename numbers::NumberTraits<Number>::real_type
+  DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
   square() const;
 
   /**
    * Return the Euclidean distance of <tt>this</tt> point to the point
-   * <tt>p</tt>, i.e. the <tt>l_2</tt> norm of the difference between the
+   * <tt>p</tt>, i.e. the $l_2$ norm of the difference between the
    * vectors representing the two points.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  typename numbers::NumberTraits<Number>::real_type
+  DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
   distance(const Point<dim, Number> &p) const;
 
   /**
    * Return the squared Euclidean distance of <tt>this</tt> point to the point
    * <tt>p</tt>.
+   *
+   * @note This function can also be used in CUDA device code.
    */
-  typename numbers::NumberTraits<Number>::real_type
+  DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
   distance_square(const Point<dim, Number> &p) const;
 
   /**
@@ -294,27 +345,32 @@ public:
 // At least clang-3.7 requires us to have a user-defined constructor
 // and we can't use 'Point<dim,Number>::Point () = default' here.
 template <int dim, typename Number>
-inline Point<dim, Number>::Point() // NOLINT
+inline DEAL_II_CUDA_HOST_DEV
+Point<dim, Number>::Point() // NOLINT
 {}
 
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>::Point(const Tensor<1, dim, Number> &t)
+inline DEAL_II_CUDA_HOST_DEV
+Point<dim, Number>::Point(const Tensor<1, dim, Number> &t)
   : Tensor<1, dim, Number>(t)
 {}
 
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>::Point(const Number x)
+inline DEAL_II_CUDA_HOST_DEV
+Point<dim, Number>::Point(const Number x)
 {
+#  ifndef __CUDA_ARCH__
   Assert(dim == 1,
          ExcMessage(
            "You can only initialize Point<1> objects using the constructor "
            "that takes only one argument. Point<dim> objects with dim!=1 "
            "require initialization with the constructor that takes 'dim' "
            "arguments."));
+#  endif
 
   // we can only get here if we pass the assertion. use the switch anyway so
   // as to avoid compiler warnings about uninitialized elements or writing
@@ -332,14 +388,18 @@ inline Point<dim, Number>::Point(const Number x)
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>::Point(const Number x, const Number y)
+inline DEAL_II_CUDA_HOST_DEV
+Point<dim, Number>::Point(const Number x, const Number y)
 {
+#  ifndef __CUDA_ARCH__
   Assert(dim == 2,
          ExcMessage(
            "You can only initialize Point<2> objects using the constructor "
            "that takes two arguments. Point<dim> objects with dim!=2 "
            "require initialization with the constructor that takes 'dim' "
            "arguments."));
+#  endif
+
   // we can only get here if we pass the assertion. use the indirection anyway
   // so as to avoid compiler warnings about uninitialized elements or writing
   // beyond the end of the 'values' array
@@ -351,14 +411,17 @@ inline Point<dim, Number>::Point(const Number x, const Number y)
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>::Point(const Number x, const Number y, const Number z)
+inline DEAL_II_CUDA_HOST_DEV
+Point<dim, Number>::Point(const Number x, const Number y, const Number z)
 {
+#  ifndef __CUDA_ARCH__
   Assert(dim == 3,
          ExcMessage(
            "You can only initialize Point<3> objects using the constructor "
            "that takes three arguments. Point<dim> objects with dim!=3 "
            "require initialization with the constructor that takes 'dim' "
            "arguments."));
+#  endif
 
   // we can only get here if we pass the assertion. use the indirection anyway
   // so as to avoid compiler warnings about uninitialized elements or writing
@@ -395,8 +458,8 @@ inline Point<dim, Number>::Point(
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>
-Point<dim, Number>::unit_vector(unsigned int i)
+inline DEAL_II_CUDA_HOST_DEV Point<dim, Number>
+                             Point<dim, Number>::unit_vector(unsigned int i)
 {
   Point<dim, Number> p;
   p[i] = 1.;
@@ -405,27 +468,42 @@ Point<dim, Number>::unit_vector(unsigned int i)
 
 
 template <int dim, typename Number>
-inline Number
+inline DEAL_II_CUDA_HOST_DEV Number
 Point<dim, Number>::operator()(const unsigned int index) const
 {
+#  ifndef __CUDA_ARCH__
   AssertIndexRange(index, dim);
+#  endif
   return this->values[index];
 }
 
 
 
 template <int dim, typename Number>
-inline Number &
+inline DEAL_II_CUDA_HOST_DEV Number &
 Point<dim, Number>::operator()(const unsigned int index)
 {
+#  ifndef __CUDA_ARCH__
   AssertIndexRange(index, dim);
+#  endif
   return this->values[index];
 }
 
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>
+template <typename OtherNumber>
+inline DEAL_II_ALWAYS_INLINE Point<dim, Number> &
+Point<dim, Number>::operator=(const Tensor<1, dim, OtherNumber> &p)
+{
+  Tensor<1, dim, Number>::operator=(p);
+  return *this;
+}
+
+
+
+template <int dim, typename Number>
+inline DEAL_II_CUDA_HOST_DEV Point<dim, Number>
 Point<dim, Number>::operator+(const Tensor<1, dim, Number> &p) const
 {
   Point<dim, Number> tmp = *this;
@@ -436,7 +514,7 @@ Point<dim, Number>::operator+(const Tensor<1, dim, Number> &p) const
 
 
 template <int dim, typename Number>
-inline Tensor<1, dim, Number>
+inline DEAL_II_CUDA_HOST_DEV Tensor<1, dim, Number>
 Point<dim, Number>::operator-(const Point<dim, Number> &p) const
 {
   return (Tensor<1, dim, Number>(*this) -= p);
@@ -445,7 +523,7 @@ Point<dim, Number>::operator-(const Point<dim, Number> &p) const
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>
+inline DEAL_II_CUDA_HOST_DEV Point<dim, Number>
 Point<dim, Number>::operator-(const Tensor<1, dim, Number> &p) const
 {
   Point<dim, Number> tmp = *this;
@@ -456,7 +534,7 @@ Point<dim, Number>::operator-(const Tensor<1, dim, Number> &p) const
 
 
 template <int dim, typename Number>
-inline Point<dim, Number>
+inline DEAL_II_CUDA_HOST_DEV Point<dim, Number>
 Point<dim, Number>::operator-() const
 {
   Point<dim, Number> result;
@@ -469,11 +547,11 @@ Point<dim, Number>::operator-() const
 
 template <int dim, typename Number>
 template <typename OtherNumber>
-inline Point<
-  dim,
-  typename ProductType<Number,
-                       typename EnableIfScalar<OtherNumber>::type>::type>
-  Point<dim, Number>::operator*(const OtherNumber factor) const
+inline DEAL_II_CUDA_HOST_DEV
+    Point<dim,
+        typename ProductType<Number,
+                             typename EnableIfScalar<OtherNumber>::type>::type>
+    Point<dim, Number>::operator*(const OtherNumber factor) const
 {
   Point<dim, typename ProductType<Number, OtherNumber>::type> tmp;
   for (unsigned int i = 0; i < dim; ++i)
@@ -485,23 +563,25 @@ inline Point<
 
 template <int dim, typename Number>
 template <typename OtherNumber>
-inline Point<
-  dim,
-  typename ProductType<Number,
-                       typename EnableIfScalar<OtherNumber>::type>::type>
-Point<dim, Number>::operator/(const OtherNumber factor) const
+inline DEAL_II_CUDA_HOST_DEV
+  Point<dim,
+        typename ProductType<Number,
+                             typename EnableIfScalar<OtherNumber>::type>::type>
+  Point<dim, Number>::operator/(const OtherNumber factor) const
 {
-  Point<dim, typename ProductType<Number, OtherNumber>::type> tmp;
-  for (unsigned int i = 0; i < dim; ++i)
-    tmp[i] = this->operator[](i) / factor;
-  return tmp;
+  const Tensor<1, dim, Number> &base_object = *this;
+  return Point<
+    dim,
+    typename ProductType<Number,
+                         typename EnableIfScalar<OtherNumber>::type>::type>(
+    dealii::operator/(base_object, factor));
 }
 
 
 
 template <int dim, typename Number>
-inline Number Point<dim, Number>::
-              operator*(const Tensor<1, dim, Number> &p) const
+inline DEAL_II_CUDA_HOST_DEV Number Point<dim, Number>::
+                                    operator*(const Tensor<1, dim, Number> &p) const
 {
   Number res = Number();
   for (unsigned int i = 0; i < dim; ++i)
@@ -511,7 +591,7 @@ inline Number Point<dim, Number>::
 
 
 template <int dim, typename Number>
-inline typename numbers::NumberTraits<Number>::real_type
+inline DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
 Point<dim, Number>::square() const
 {
   return this->norm_square();
@@ -520,7 +600,7 @@ Point<dim, Number>::square() const
 
 
 template <int dim, typename Number>
-inline typename numbers::NumberTraits<Number>::real_type
+inline DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
 Point<dim, Number>::distance(const Point<dim, Number> &p) const
 {
   return std::sqrt(distance_square(p));
@@ -529,7 +609,7 @@ Point<dim, Number>::distance(const Point<dim, Number> &p) const
 
 
 template <int dim, typename Number>
-inline typename numbers::NumberTraits<Number>::real_type
+inline DEAL_II_CUDA_HOST_DEV typename numbers::NumberTraits<Number>::real_type
 Point<dim, Number>::distance_square(const Point<dim, Number> &p) const
 {
   Number sum = internal::NumberType<Number>::value(0.0);
@@ -563,15 +643,17 @@ Point<dim, Number>::serialize(Archive &ar, const unsigned int)
 /**
  * Global operator scaling a point vector by a scalar.
  *
+ * @note This function can also be used in CUDA device code.
+ *
  * @relatesalso Point
  * @relatesalso EnableIfScalar
  */
 template <int dim, typename Number, typename OtherNumber>
-inline Point<
-  dim,
-  typename ProductType<Number,
-                       typename EnableIfScalar<OtherNumber>::type>::type>
-operator*(const OtherNumber factor, const Point<dim, Number> &p)
+inline DEAL_II_CUDA_HOST_DEV
+  Point<dim,
+        typename ProductType<Number,
+                             typename EnableIfScalar<OtherNumber>::type>::type>
+  operator*(const OtherNumber factor, const Point<dim, Number> &p)
 {
   return p * factor;
 }
