@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2014 by the deal.II authors
+// Copyright (C) 2009 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,9 +16,12 @@
 #ifndef dealii_std_cxx11_shared_ptr_h
 #define dealii_std_cxx11_shared_ptr_h
 
-
 #include <deal.II/base/config.h>
 
+DEAL_II_WARNING(
+  "This file is deprecated. Simply use the corresponding C++11 header <memory>.")
+
+
 #include <memory>
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx11
@@ -30,10 +33,4 @@ namespace std_cxx11
 } // namespace std_cxx11
 DEAL_II_NAMESPACE_CLOSE
 
-
-// then allow using the old namespace name instead of the new one
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx1x = std_cxx11;
-DEAL_II_NAMESPACE_CLOSE
-
 #endif
