@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2012 - 2015 by the deal.II authors
+// Copyright (C) 2012 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,15 +16,16 @@
 #ifndef dealii_std_cxx11_type_traits_h
 #define dealii_std_cxx11_type_traits_h
 
-
 #include <deal.II/base/config.h>
 
+DEAL_II_WARNING(
+  "This file is deprecated. Simply use the corresponding C++11 header <type_traits>.")
+
+
 #include <type_traits>
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx11
 {
-  // TODO: could fill up with more types from
-  // C++11 type traits
   using std::enable_if;
   using std::false_type;
   using std::is_fundamental;
@@ -36,10 +37,4 @@ namespace std_cxx11
 } // namespace std_cxx11
 DEAL_II_NAMESPACE_CLOSE
 
-
-// then allow using the old namespace name instead of the new one
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx1x = std_cxx11;
-DEAL_II_NAMESPACE_CLOSE
-
 #endif
