@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2015, 2016 by the deal.II authors
+// Copyright (C) 2015 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -19,6 +19,10 @@
 
 #include <deal.II/base/config.h>
 
+DEAL_II_WARNING(
+  "This file is deprecated. Simply use the corresponding C++11 header <memory>.")
+
+
 #include <memory>
 DEAL_II_NAMESPACE_OPEN
 namespace std_cxx11
@@ -27,9 +31,4 @@ namespace std_cxx11
 }
 DEAL_II_NAMESPACE_CLOSE
 
-// then allow using the old namespace name instead of the new one
-DEAL_II_NAMESPACE_OPEN
-namespace std_cxx1x = std_cxx11;
-DEAL_II_NAMESPACE_CLOSE
-
 #endif
