@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2019 by the deal.II authors
+// Copyright (C) 2003 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -320,103 +320,6 @@ struct PointerComparison
 
 
 
-namespace internal
-{
-  /**
-   * A type that is sometimes used for template tricks. For example, in some
-   * situations one would like to do this:
-   *
-   * @code
-   *   template <int dim>
-   *   class X
-   *   {
-   *     // do something on subdim-dimensional sub-objects of the big
-   *     // dim-dimensional thing (for example on vertices/lines/quads of
-   *     // cells):
-   *     template <int subdim> void f();
-   *   };
-   *
-   *   template <int dim>
-   *   template <>
-   *   void X<dim>::f<0> ()
-   *   {
-   *     ...operate on the vertices of a cell...
-   *   }
-   *
-   *   template <int dim, int subdim> void g(X<dim> &x)
-   *   {
-   *     x.f<subdim> ();
-   *   }
-   * @endcode
-   *
-   * The problem is: the language doesn't allow us to specialize
-   * <code>X::f()</code> without specializing the outer class first. One of
-   * the common tricks is therefore to use something like this:
-   *
-   * @code
-   *   template <int N>
-   *   struct int2type
-   *   {};
-   *
-   *   template <int dim>
-   *   class X
-   *   {
-   *     // do something on subdim-dimensional sub-objects of the big
-   *     // dim-dimensional thing (for example on vertices/lines/quads of
-   *     // cells):
-   *     void f(int2type<0>);
-   *     void f(int2type<1>);
-   *     void f(int2type<2>);
-   *     void f(int2type<3>);
-   *   };
-   *
-   *   template <int dim>
-   *   void X<dim>::f (int2type<0>)
-   *   {
-   *     ...operate on the vertices of a cell...
-   *   }
-   *
-   *   template <int dim>
-   *   void X<dim>::f (int2type<1>)
-   *   {
-   *     ...operate on the lines of a cell...
-   *   }
-   *
-   *   template <int dim, int subdim>
-   *   void g(X<dim> &x)
-   *   {
-   *     x.f (int2type<subdim>());
-   *   }
-   * @endcode
-   *
-   * Note that we have replaced specialization of <code>X::f()</code> by
-   * overloading, but that from inside the function <code>g()</code>, we can
-   * still select which of the different <code>X::f()</code> we want based on
-   * the <code>subdim</code> template argument.
-   *
-   * @deprecated Use std::integral_constant<int, N> instead.
-   *
-   * @author Wolfgang Bangerth, 2006
-   */
-  template <int N>
-  struct DEAL_II_DEPRECATED int2type
-  {};
-
-
-  /**
-   * The equivalent of the int2type class for boolean arguments.
-   *
-   * @deprecated Use std::integral_constant<bool, B> instead.
-   *
-   * @author Wolfgang Bangerth, 2009
-   */
-  template <bool B>
-  struct DEAL_II_DEPRECATED bool2type
-  {};
-} // namespace internal
-
-
-
 /**
  * A type that can be used to determine whether two types are equal. It allows
  * to write code like
@@ -606,7 +509,7 @@ namespace internal
  *   multiply(1.234, 2.345);
  * @endcode
  * it will deduce @p T to be @p double, and because
- * <code>EnableIfScalar@<double@>::type</code> equals @p double, the compiler
+ * <code>EnableIfScalar@<double@>::%type</code> equals @p double, the compiler
  * will instantiate a function <code>double multiply(const double, const
  * double)</code> from the template above. On the other hand, in a context
  * like
@@ -615,7 +518,7 @@ namespace internal
  *   multiply(v1, v2);
  * @endcode
  * the compiler will deduce @p T to be <code>std::vector@<char@></code> but
- * because <code>EnableIfScalar@<std::vector@<char@>@>::type</code> does not
+ * because <code>EnableIfScalar@<std::vector@<char@>@>::%type</code> does not
  * exist the compiler does not consider the template for instantiation. This
  * technique is called "Substitution Failure is not an Error (SFINAE)". It
  * makes sure that the template function can not even be called, rather than
