@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2019 by the deal.II authors
+// Copyright (C) 2000 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -21,6 +21,7 @@
 
 #  include <deal.II/base/exceptions.h>
 #  include <deal.II/base/multithread_info.h>
+#  include <deal.II/base/std_cxx17/tuple.h>
 #  include <deal.II/base/template_constraints.h>
 
 #  include <condition_variable>
@@ -38,7 +39,9 @@ DEAL_II_DISABLE_EXTRA_DIAGNOSTICS
 #    ifdef DEAL_II_USE_MT_POSIX
 #      include <pthread.h>
 #    endif
+#    define TBB_SUPPRESS_DEPRECATED_MESSAGES 1
 #    include <tbb/task.h>
+#    undef TBB_SUPPRESS_DEPRECATED_MESSAGES
 #    include <tbb/tbb_stddef.h>
 #  endif
 DEAL_II_ENABLE_EXTRA_DIAGNOSTICS
@@ -1096,8 +1099,9 @@ namespace Threads
   inline Thread<RT>
   new_thread(RT (*fun_ptr)(Args...), typename identity<Args>::type... args)
   {
-    return new_thread(std::function<RT()>(
-      std::bind(fun_ptr, internal::maybe_make_ref<Args>::act(args)...)));
+    auto dummy = std::make_tuple(internal::maybe_make_ref<Args>::act(args)...);
+    return new_thread(
+      [dummy, fun_ptr]() -> RT { return std_cxx17::apply(fun_ptr, dummy); });
   }
 
 
@@ -1113,6 +1117,7 @@ namespace Threads
              typename identity<C>::type &c,
              typename identity<Args>::type... args)
   {
+    // NOLINTNEXTLINE(modernize-avoid-bind) silence clang-tidy
     return new_thread(std::function<RT()>(std::bind(
       fun_ptr, std::ref(c), internal::maybe_make_ref<Args>::act(args)...)));
   }
@@ -1129,6 +1134,7 @@ namespace Threads
              typename identity<const C>::type &c,
              typename identity<Args>::type... args)
   {
+    // NOLINTNEXTLINE(modernize-avoid-bind) silence clang-tidy
     return new_thread(std::function<RT()>(std::bind(
       fun_ptr, std::cref(c), internal::maybe_make_ref<Args>::act(args)...)));
   }
@@ -1292,10 +1298,10 @@ namespace Threads
       TaskDescriptor();
 
       /**
-       * Copy constructor. Throws an exception since we want to make sure that
-       * each TaskDescriptor object corresponds to exactly one task.
+       * Copy constructor. Objects of this type can not be copied, and so this
+       * constructor is `delete`d and can't be called.
        */
-      TaskDescriptor(const TaskDescriptor &);
+      TaskDescriptor(const TaskDescriptor &) = delete;
 
       /**
        * Destructor.
@@ -1303,11 +1309,11 @@ namespace Threads
       ~TaskDescriptor();
 
       /**
-       * Copy operator. Throws an exception since we want to make sure that
-       * each TaskDescriptor object corresponds to exactly one task.
+       * Copy operator. Objects of this type can not be copied, and so this
+       * operator is `delete`d and can't be called.
        */
       TaskDescriptor &
-      operator=(const TaskDescriptor &);
+      operator=(const TaskDescriptor &) = delete;
 
       /**
        * Queue up the task to the scheduler. We need to do this in a separate
@@ -1369,17 +1375,6 @@ namespace Threads
 
 
 
-    template <typename RT>
-    TaskDescriptor<RT>::TaskDescriptor(const TaskDescriptor &)
-      : task_is_done(false)
-    {
-      // we shouldn't be getting here -- task descriptors
-      // can't be copied
-      Assert(false, ExcInternalError());
-    }
-
-
-
     template <typename RT>
     inline TaskDescriptor<RT>::~TaskDescriptor()
     {
@@ -1402,17 +1397,6 @@ namespace Threads
     }
 
 
-    template <typename RT>
-    TaskDescriptor<RT> &
-    TaskDescriptor<RT>::operator=(const TaskDescriptor &)
-    {
-      // we shouldn't be getting here -- task descriptors
-      // can't be copied
-      Assert(false, ExcInternalError());
-      return *this;
-    }
-
-
     template <typename RT>
     inline void
     TaskDescriptor<RT>::join()
@@ -1753,8 +1737,9 @@ namespace Threads
   inline Task<RT>
   new_task(RT (*fun_ptr)(Args...), typename identity<Args>::type... args)
   {
-    return new_task(std::function<RT()>(
-      std::bind(fun_ptr, internal::maybe_make_ref<Args>::act(args)...)));
+    auto dummy = std::make_tuple(internal::maybe_make_ref<Args>::act(args)...);
+    return new_task(
+      [dummy, fun_ptr]() -> RT { return std_cxx17::apply(fun_ptr, dummy); });
   }
 
 
@@ -1770,6 +1755,7 @@ namespace Threads
            typename identity<C>::type &c,
            typename identity<Args>::type... args)
   {
+    // NOLINTNEXTLINE(modernize-avoid-bind) silence clang-tidy
     return new_task(std::function<RT()>(std::bind(
       fun_ptr, std::ref(c), internal::maybe_make_ref<Args>::act(args)...)));
   }
@@ -1786,6 +1772,7 @@ namespace Threads
            typename identity<const C>::type &c,
            typename identity<Args>::type... args)
   {
+    // NOLINTNEXTLINE(modernize-avoid-bind) silence clang-tidy
     return new_task(std::function<RT()>(std::bind(
       fun_ptr, std::cref(c), internal::maybe_make_ref<Args>::act(args)...)));
   }
@@ -1821,6 +1808,21 @@ namespace Threads
       return *this;
     }
 
+
+    /**
+     * Return how many tasks have been put into this group. This
+     * function does not distinguish how many of these tasks have
+     * already run and have finished, are still waiting to be
+     * scheduled to a CPU resource, or are currently running. Tasks
+     * that have been joined already are also still counted.
+     */
+    std::size_t
+    size() const
+    {
+      return tasks.size();
+    }
+
+
     /**
      * Wait for all tasks in the collection to finish. It is not a problem if
      * some of them have already been waited for, i.e. you may call this
@@ -1830,10 +1832,8 @@ namespace Threads
     void
     join_all() const
     {
-      for (typename std::list<Task<RT>>::const_iterator t = tasks.begin();
-           t != tasks.end();
-           ++t)
-        t->join();
+      for (auto &t : tasks)
+        t.join();
     }
 
   private:
