@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2019 by the deal.II authors
+// Copyright (C) 2004 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -129,8 +129,7 @@ inline VectorSlice<VectorType>::VectorSlice(VectorType & v,
   : ArrayViewType(
       make_array_view(std::begin(v) + start, std::begin(v) + start + length))
 {
-  Assert((start + length <= v.size()),
-         ExcIndexRange(length, 0, v.size() - start + 1));
+  AssertIndexRange(length, v.size() - start + 1);
 }
 
 DEAL_II_NAMESPACE_CLOSE
