@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2019 by the deal.II authors
+// Copyright (C) 2008 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,8 +28,6 @@
 #  include <deal.II/base/thread_management.h>
 
 #  ifdef DEAL_II_WITH_THREADS
-#    include <deal.II/base/thread_management.h>
-
 #    include <tbb/pipeline.h>
 #  endif
 
@@ -126,7 +124,11 @@ DEAL_II_NAMESPACE_OPEN
  * Once an item is processed by the copier, it is deleted and the ScratchData
  * and CopyData objects that were used in its computation are considered
  * unused and may be re-used for the next invocation of the worker function,
- * on this or another thread.
+ * on this or another thread. However, the WorkStream functions make no
+ * attempt to reset these objects to any kind of pristine state -- a worker
+ * should assume that the CopyData object it gets handed has prior content
+ * and clear it first in whatever manner seems appropriate, before putting
+ * content into it that can later be processed again by the copier.
  *
  * The member variables in ScratchData and CopyData can be accessed
  * independently of other concurrent uses of copies of these data structures.
@@ -139,7 +141,13 @@ DEAL_II_NAMESPACE_OPEN
  * CopyData can be resized in accordance with the number of local DoFs on the
  * current cell.
  *
- * The functions in this namespace only really work in parallel when
+ * @note For integration over cells and faces, it is often useful to use
+ * methods more specific to the task than the current function (which doesn't
+ * care whether the iterators are over cells, vector elements, or any other
+ * kind of range). An implementation of an interface specifically suited to
+ * integration is the MeshWorker::mesh_loop() function.
+ *
+ * @note The functions in this namespace only really work in parallel when
  * multithread mode was selected during deal.II configuration. Otherwise they
  * simply work on each item sequentially.
  *
@@ -152,11 +160,6 @@ namespace WorkStream
 
   namespace internal
   {
-    // TODO: The following classes all use std::shared_ptr, but the
-    //  correct pointer class would actually be std::unique_ptr. make this
-    //  replacement whenever we have a class that provides these semantics
-    //  and that is available also as a fall-back whenever via boost or similar
-
     /**
      * A namespace for the implementation of details of the WorkStream pattern
      * and function. This namespace holds classes that deal with the second
@@ -193,7 +196,7 @@ namespace WorkStream
            */
           struct ScratchDataObject
           {
-            std::shared_ptr<ScratchData> scratch_data;
+            std::unique_ptr<ScratchData> scratch_data;
             bool                         currently_in_use;
 
             /**
@@ -208,20 +211,7 @@ namespace WorkStream
               , currently_in_use(in_use)
             {}
 
-            // TODO: when we push back an object to the list of scratch objects,
-            // in
-            //  Worker::operator(), we first create an object and then copy
-            //  it to the end of this list. this involves having two objects
-            //      of the current type having pointers to it, each with their
-            //      own currently_in_use flag. there is probably little harm in
-            //      this because the original one goes out of scope right away
-            //      again, but it's certainly awkward. one way to avoid this
-            //      would be to use unique_ptr but we'd need to figure out a way
-            //      to use it in non-C++11 mode
-            ScratchDataObject(const ScratchDataObject &o)
-              : scratch_data(o.scratch_data)
-              , currently_in_use(o.currently_in_use)
-            {}
+            ScratchDataObject(ScratchDataObject &&o) noexcept = default;
           };
 
 
@@ -274,7 +264,7 @@ namespace WorkStream
            *
            * The pointers to scratch objects stored in each of these lists
            * must be so that they are deleted on all threads when the thread
-           * local object is destroyed. This is achieved by using shared_ptr.
+           * local object is destroyed. This is achieved by using unique_ptr.
            *
            * Note that when a worker needs to create a scratch object, it
            * allocates it using sample_scratch_data to copy from. This has the
@@ -432,7 +422,7 @@ namespace WorkStream
          *
          * The pointers to scratch objects stored in each of these lists must
          * be so that they are deleted on all threads when the thread local
-         * object is destroyed. This is achieved by using shared_ptr.
+         * object is destroyed. This is achieved by using unique_ptr.
          *
          * Note that when a worker needs to create a scratch object, it
          * allocates it using sample_scratch_data to copy from. This has the
@@ -536,7 +526,7 @@ namespace WorkStream
 
                 typename ItemType::ScratchDataList::value_type
                   new_scratch_object(scratch_data, true);
-                scratch_data_list.push_back(new_scratch_object);
+                scratch_data_list.push_back(std::move(new_scratch_object));
               }
           }
 
@@ -701,8 +691,8 @@ namespace WorkStream
       template <typename Iterator, typename ScratchData, typename CopyData>
       struct ScratchAndCopyDataObjects
       {
-        std::shared_ptr<ScratchData> scratch_data;
-        std::shared_ptr<CopyData>    copy_data;
+        std::unique_ptr<ScratchData> scratch_data;
+        std::unique_ptr<CopyData>    copy_data;
         bool                         currently_in_use;
 
         /**
@@ -910,7 +900,8 @@ namespace WorkStream
    *
    * This function that can be used for worker and copier objects that are
    * either pointers to non-member functions or objects that allow to be
-   * called with an operator(), for example objects created by std::bind.
+   * called with an operator(), for example objects created by lambda functions
+   * or std::bind.
    *
    * The two data types <tt>ScratchData</tt> and <tt>CopyData</tt> need to
    * have a working copy constructor. <tt>ScratchData</tt> is only used in the
@@ -927,6 +918,18 @@ namespace WorkStream
    * copies of the <tt>ScratchData</tt> object and
    * <tt>queue_length*chunk_size</tt> copies of the <tt>CopyData</tt> object
    * are generated.
+   *
+   * @note In case the copier does not do anything, pass
+   * `std::function<void(const CopyData &)>()` as @p copier to make sure
+   * a more efficient algorithm is used internally. It is important, however,
+   * to recognize that the empty function object created above is *not*
+   * the same as a lambda function with an empty body,
+   * `[](const CopyData &) {}` -- from the perspective of this function,
+   * there is no way to recognize whether a lambda function provided as
+   * a copier does something or does not do something in its body,
+   * and so it needs to be copied. On the other hand, a default-constructed
+   * `std::function` object *can* be recognized, and is then used to select
+   * a more efficient algorithm.
    */
   template <typename Worker,
             typename Copier,
@@ -951,8 +954,11 @@ namespace WorkStream
    *
    * This function that can be used for worker and copier objects that are
    * either pointers to non-member functions or objects that allow to be
-   * called with an operator(), for example objects created by std::bind. If
-   * the copier is an empty function, it is ignored in the pipeline.
+   * called with an operator(), for example lambda functions
+   * or objects created by std::bind. If the copier is an empty function, it is
+   * ignored in the pipeline. (However, a lambda function with an empty body is
+   * *not* equivalent to an empty `std::function` object and will, consequently,
+   * not be ignored.
    *
    * The argument passed as @p end must be convertible to the same type as @p
    * begin, but doesn't have to be of the same type itself. This allows to
@@ -976,6 +982,18 @@ namespace WorkStream
    * copies of the <tt>ScratchData</tt> object and
    * <tt>queue_length*chunk_size</tt> copies of the <tt>CopyData</tt> object
    * are generated.
+   *
+   * @note In case the copier does not do anything, pass
+   * `std::function<void(const CopyData &)>()` as @p copier to make sure
+   * a more efficient algorithm is used internally. It is important, however,
+   * to recognize that the empty function object created above is *not*
+   * the same as a lambda function with an empty body,
+   * `[](const CopyData &) {}` -- from the perspective of this function,
+   * there is no way to recognize whether a lambda function provided as
+   * a copier does something or does not do something in its body,
+   * and so it needs to be copied. On the other hand, a default-constructed
+   * `std::function` object *can* be recognized, and is then used to select
+   * a more efficient algorithm.
    */
   template <typename Worker,
             typename Copier,
@@ -1225,9 +1243,11 @@ namespace WorkStream
               parallel::internal::parallel_for(
                 colored_iterators[color].begin(),
                 colored_iterators[color].end(),
-                std::bind(&WorkerAndCopier::operator(),
-                          std::ref(worker_and_copier),
-                          std::placeholders::_1),
+                [&worker_and_copier](
+                  const tbb::blocked_range<
+                    typename std::vector<Iterator>::const_iterator> &range) {
+                  worker_and_copier(range);
+                },
                 chunk_size);
             }
       }
@@ -1264,6 +1284,18 @@ namespace WorkStream
    * copies of the <tt>ScratchData</tt> object and
    * <tt>queue_length*chunk_size</tt> copies of the <tt>CopyData</tt> object
    * are generated.
+   *
+   * @note In case the copier does not do anything, pass
+   * `std::function<void(const CopyData &)>()` as @p copier to make sure
+   * a more efficient algorithm is used internally. It is important, however,
+   * to recognize that the empty function object created above is *not*
+   * the same as a lambda function with an empty body,
+   * `[](const CopyData &) {}` -- from the perspective of this function,
+   * there is no way to recognize whether a lambda function provided as
+   * a copier does something or does not do something in its body,
+   * and so it needs to be copied. On the other hand, a default-constructed
+   * `std::function` object *can* be recognized, and is then used to select
+   * a more efficient algorithm.
    */
   template <typename MainClass,
             typename Iterator,
@@ -1283,12 +1315,14 @@ namespace WorkStream
     // forward to the other function
     run(begin,
         end,
-        std::bind(worker,
-                  std::ref(main_object),
-                  std::placeholders::_1,
-                  std::placeholders::_2,
-                  std::placeholders::_3),
-        std::bind(copier, std::ref(main_object), std::placeholders::_1),
+        [&main_object, worker](const Iterator &iterator,
+                               ScratchData &   scratch_data,
+                               CopyData &      copy_data) {
+          (main_object.*worker)(iterator, scratch_data, copy_data);
+        },
+        [&main_object, copier](const CopyData &copy_data) {
+          (main_object.*copier)(copy_data);
+        },
         sample_scratch_data,
         sample_copy_data,
         queue_length,
@@ -1314,12 +1348,14 @@ namespace WorkStream
     // forward to the other function
     run(begin,
         end,
-        std::bind(worker,
-                  std::ref(main_object),
-                  std::placeholders::_1,
-                  std::placeholders::_2,
-                  std::placeholders::_3),
-        std::bind(copier, std::ref(main_object), std::placeholders::_1),
+        [&main_object, worker](const Iterator &iterator,
+                               ScratchData &   scratch_data,
+                               CopyData &      copy_data) {
+          (main_object.*worker)(iterator, scratch_data, copy_data);
+        },
+        [&main_object, copier](const CopyData &copy_data) {
+          (main_object.*copier)(copy_data);
+        },
         sample_scratch_data,
         sample_copy_data,
         queue_length,
