@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2008 - 2018 by the deal.II authors
+// Copyright (C) 2008 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -40,17 +40,48 @@ namespace parallel
 {
   /**
    * This class describes the interface for all triangulation classes that
-   * work in parallel, namely parallel::distributed::Triangulation and
+   * work in parallel, namely parallel::distributed::Triangulation,
+   * parallel::fullydistributed::Triangulation, and
    * parallel::shared::Triangulation.
+   *
+   * It is, consequently, a class that can be used to test whether a
+   * pointer of reference to a triangulation object refers to a
+   * sequential triangulation, or whether the triangulation is in fact
+   * parallel. In other words, one could write a function like this:
+   * @code
+   *   template <int dim, int spacedim>
+   *   bool is_parallel (const dealii::Triangulation<dim,spacedim> &tria)
+   *   {
+   *     if (dynamic_cast<const parallel::TriangulationBase<dim,spacedim>*>
+   *                     (&tria)
+   *         != nullptr)
+   *       return true;
+   *     else
+   *       return false;
+   *   }
+   * @endcode
+   *
+   * All parallel triangulations share certain traits, such as the fact that
+   * they communicate via
+   * @ref GlossMPICommunicator "MPI communicators"
+   * or that they have
+   * @ref GlossLocallyOwnedCell "locally owned",
+   * @ref GlossGhostCell "ghost", and possibly
+   * @ref GlossArtificialCell "artificial cells".
+   * This class provides
+   * a number of member functions that allows querying some information
+   * about the triangulation that is independent of how exactly a
+   * parallel triangulation is implemented (i.e., which of the various
+   * classes derived from the current one it actually is).
    */
   template <int dim, int spacedim = dim>
-  class Triangulation : public dealii::Triangulation<dim, spacedim>
+  class TriangulationBase : public dealii::Triangulation<dim, spacedim>
   {
   public:
     /**
      * Constructor.
      */
-    Triangulation(
+    TriangulationBase(
       MPI_Comm mpi_communicator,
       const typename dealii::Triangulation<dim, spacedim>::MeshSmoothing
                  smooth_grid = (dealii::Triangulation<dim, spacedim>::none),
@@ -59,31 +90,32 @@ namespace parallel
     /**
      * Destructor.
      */
-    virtual ~Triangulation() override;
+    virtual ~TriangulationBase() override;
 
     /**
      * Return MPI communicator used by this triangulation.
      */
-    virtual MPI_Comm
+    virtual const MPI_Comm &
     get_communicator() const;
 
+    /**
+     * Return if multilevel hierarchy is supported and has been constructed.
+     */
+    virtual bool
+    is_multilevel_hierarchy_constructed() const = 0;
+
     /**
      * Implementation of the same function as in the base class.
+     *
+     * @note This function copies the cells, but not the communicator,
+     * of the source triangulation. In other words, the resulting
+     * triangulation will operate on the communicator it was constructed
+     * with.
      */
     virtual void
     copy_triangulation(
       const dealii::Triangulation<dim, spacedim> &old_tria) override;
 
-    /**
-     * Return the number of active cells owned by each of the MPI processes
-     * that contribute to this triangulation. The element of this vector
-     * indexed by locally_owned_subdomain() equals the result of
-     * n_locally_owned_active_cells().
-     */
-    const std::vector<unsigned int> &
-    n_locally_owned_active_cells_per_processor() const;
-
-
     /**
      * Return the number of active cells in the triangulation that are locally
      * owned, i.e. that have a subdomain_id equal to
@@ -110,7 +142,7 @@ namespace parallel
      * by each processor. This equals the overall number of active cells in
      * the triangulation.
      */
-    virtual types::global_dof_index
+    virtual types::global_cell_index
     n_global_active_cells() const override;
 
     /**
@@ -144,8 +176,9 @@ namespace parallel
      * ghost cell adjacent to the cells of the local processor. In other
      * words, this is the set of subdomain_id() for all ghost cells.
      *
-     * @note: If @p i is contained in the list of processor @p j, then @p j
-     * will also be contained in the list of processor @p i.
+     * The returned sets are symmetric, that is if @p i is contained in the
+     * list of processor @p j, then @p j will also be contained in the list of
+     * processor @p i.
      */
     const std::set<types::subdomain_id> &
     ghost_owners() const;
@@ -156,8 +189,14 @@ namespace parallel
      * other words, this is the set of level_subdomain_id() for all level
      * ghost cells.
      *
-     * @note: If @p i is contained in the list of processor @p j, then @p j
-     * will also be contained in the list of processor @p i.
+     * The returned sets are symmetric, that is if @p i is contained in the
+     * list of processor @p j, then @p j will also be contained in the list of
+     * processor @p i.
+     *
+     * @note The level ghost owners can only be determined if the multigrid
+     * ownership has been assigned (by setting the
+     * construct_multigrid_hierarchy flag at construction time), otherwise the
+     * returned set will be empty.
      */
     const std::set<types::subdomain_id> &
     level_ghost_owners() const;
@@ -165,10 +204,33 @@ namespace parallel
     /**
      * Return a map that, for each vertex, lists all the processors whose
      * subdomains are adjacent to that vertex.
+     *
+     * @deprecated Use GridTools::compute_vertices_with_ghost_neighbors()
+     * instead of
+     * parallel::TriangulationBase::compute_vertices_with_ghost_neighbors().
      */
-    virtual std::map<unsigned int, std::set<dealii::types::subdomain_id>>
+    DEAL_II_DEPRECATED virtual std::map<unsigned int,
+                                        std::set<dealii::types::subdomain_id>>
     compute_vertices_with_ghost_neighbors() const;
 
+    /**
+     * @copydoc dealii::Triangulation::get_boundary_ids()
+     *
+     * @note This function involves a global communication gathering all current
+     *   IDs from all processes.
+     */
+    virtual std::vector<types::boundary_id>
+    get_boundary_ids() const override;
+
+    /**
+     * @copydoc dealii::Triangulation::get_manifold_ids()
+     *
+     * @note This function involves a global communication gathering all current
+     *   IDs from all processes.
+     */
+    virtual std::vector<types::manifold_id>
+    get_manifold_ids() const override;
+
   protected:
     /**
      * MPI communicator to be used for the triangulation. We create a unique
@@ -195,15 +257,14 @@ namespace parallel
     struct NumberCache
     {
       /**
-       * This vector stores the number of locally owned active cells per MPI
-       * rank.
+       * Number of locally owned active cells of this MPI rank.
        */
-      std::vector<unsigned int> n_locally_owned_active_cells;
+      unsigned int n_locally_owned_active_cells;
       /**
        * The total number of active cells (sum of @p
        * n_locally_owned_active_cells).
        */
-      types::global_dof_index n_global_active_cells;
+      types::global_cell_index n_global_active_cells;
       /**
        * The global number of levels computed as the maximum number of levels
        * taken over all MPI ranks, so <tt>n_levels()<=n_global_levels =
@@ -231,12 +292,75 @@ namespace parallel
      */
     virtual void
     update_number_cache();
+  };
 
+  /**
+   *  Using directive for backwards-compatibility.
+   *  @deprecated Use TriangulationBase instead of Triangulation.
+   */
+  template <int dim, int spacedim = dim>
+  using Triangulation DEAL_II_DEPRECATED = TriangulationBase<dim, spacedim>;
+
+
+
+  /**
+   * A base class for distributed triangulations, i.e., triangulations that
+   * do not store all cells on all processors. This implies that not
+   * every detail of a triangulation may be known on each processor.
+   * In particular, you have to expect that triangulations of classes
+   * derived from this one only store some of the active cells (namely,
+   * the
+   * @ref GlossLocallyOwnedCell "locally owned cells"),
+   * along with
+   * @ref GlossGhostCell "ghost cells"
+   * and possibly
+   * @ref GlossArtificialCell "artificial cells".
+   * In contrast to the classes
+   * derived from parallel::TriangulationBase, it is certain that the
+   * classes derived from the current class will not store the entire
+   * triangulation as long as it has a large enough number of cells. (The
+   * difference to parallel::TriangulationBase is that the
+   * parallel::shared::Triangulation is derived from
+   * parallel::TriangulationBase, but not from the current class.) The
+   * distinction is not large in practice: Everything that is difficult for
+   * parallel distributed triangulation is generally also difficult for any
+   * other kind of parallel triangulation classes; however, this intermediate
+   * base class allows to further differentiate between the different kinds of
+   * classes providing parallel mesh functionality.
+   *
+   * This class can, then, be used to test whether a
+   * pointer or reference to a triangulation object refers to any kind of
+   * parallel triangulation, or whether the triangulation is in fact
+   * parallel distributed. In other words, one could write a function like
+   * this:
+   * @code
+   *   template <int dim, int spacedim>
+   *   bool
+   *   is_parallel_distributed(const dealii::Triangulation<dim,spacedim> &tria)
+   *   {
+   *     if(dynamic_cast<const
+   *                     parallel::DistributedTriangulationBase<dim,spacedim>*>
+   *                    (&tria)
+   *        != nullptr)
+   *       return true;
+   *     else
+   *       return false;
+   *   }
+   * @endcode
+   */
+  template <int dim, int spacedim = dim>
+  class DistributedTriangulationBase
+    : public dealii::parallel::TriangulationBase<dim, spacedim>
+  {
+  public:
     /**
-     * Store MPI ranks of level ghost owners of this processor on all levels.
+     * Constructor.
      */
-    void
-    fill_level_ghost_owners();
+    DistributedTriangulationBase(
+      MPI_Comm mpi_communicator,
+      const typename dealii::Triangulation<dim, spacedim>::MeshSmoothing
+                 smooth_grid = (dealii::Triangulation<dim, spacedim>::none),
+      const bool check_for_distorted_cells = false);
   };
 
 } // namespace parallel
