@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -18,8 +18,11 @@
 
 
 #include <deal.II/base/config.h>
+
 #include <deal.II/base/exceptions.h>
+
 #include <deal.II/dofs/dof_objects.h>
+
 #include <vector>
 
 DEAL_II_NAMESPACE_OPEN
@@ -34,7 +37,6 @@ namespace internal
    */
   namespace DoFHandlerImplementation
   {
-
     /**
      *
      * <h4>DoFFaces</h4>
@@ -69,8 +71,8 @@ namespace internal
     {
     public:
       /**
-       * Constructor. This constructor is deleted to prevent the use of this template,
-       * as only the specializations should be used
+       * Constructor. This constructor is deleted to prevent the use of this
+       * template, as only the specializations should be used
        */
       DoFFaces() = delete;
     };
@@ -89,15 +91,16 @@ namespace internal
        * Determine an estimate for the memory consumption (in bytes) of this
        * object.
        */
-      std::size_t memory_consumption () const;
+      std::size_t
+      memory_consumption() const;
 
       /**
        * Read or write the data of this object to or from a stream for the
        * purpose of serialization
        */
       template <class Archive>
-      void serialize(Archive &ar,
-                     const unsigned int version);
+      void
+      serialize(Archive &ar, const unsigned int version);
     };
 
     /**
@@ -119,15 +122,16 @@ namespace internal
        * Determine an estimate for the memory consumption (in bytes) of this
        * object.
        */
-      std::size_t memory_consumption () const;
+      std::size_t
+      memory_consumption() const;
 
       /**
        * Read or write the data of this object to or from a stream for the
        * purpose of serialization
        */
       template <class Archive>
-      void serialize(Archive &ar,
-                     const unsigned int version);
+      void
+      serialize(Archive &ar, const unsigned int version);
     };
 
     /**
@@ -154,42 +158,43 @@ namespace internal
        * Determine an estimate for the memory consumption (in bytes) of this
        * object.
        */
-      std::size_t memory_consumption () const;
+      std::size_t
+      memory_consumption() const;
 
       /**
        * Read or write the data of this object to or from a stream for the
        * purpose of serialization
        */
       template <class Archive>
-      void serialize(Archive &ar,
-                     const unsigned int version);
+      void
+      serialize(Archive &ar, const unsigned int version);
     };
 
 
 
     template <class Archive>
-    void DoFFaces<1>::serialize (Archive &,
-                                 const unsigned int)
+    void
+    DoFFaces<1>::serialize(Archive &, const unsigned int)
     {}
 
 
     template <class Archive>
-    void DoFFaces<2>::serialize (Archive &ar,
-                                 const unsigned int)
+    void
+    DoFFaces<2>::serialize(Archive &ar, const unsigned int)
     {
       ar &lines;
     }
 
 
     template <class Archive>
-    void DoFFaces<3>::serialize (Archive &ar,
-                                 const unsigned int)
+    void
+    DoFFaces<3>::serialize(Archive &ar, const unsigned int)
     {
       ar &lines &quads;
     }
 
-  }
-}
+  } // namespace DoFHandlerImplementation
+} // namespace internal
 
 DEAL_II_NAMESPACE_CLOSE
 
