@@ -25,18 +25,22 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 template <int, int>
 class DoFHandler;
+#endif
 
 namespace internal
 {
   namespace DoFHandlerImplementation
   {
+#ifndef DOXYGEN
     template <int>
     class DoFLevel;
     template <int>
     class DoFFaces;
-
+#endif
 
     /**
      * Store the indices of the degrees of freedom which are located on
@@ -149,10 +153,8 @@ namespace internal
       void
       serialize(Archive &ar, const unsigned int version);
 
-      /**
-       * Declare the classes that store levels and faces of DoFs friends so
-       * that they can resize arrays.
-       */
+      // Declare the classes that store levels and faces of DoFs friends so
+      // that they can resize arrays.
       template <int>
       friend class DoFLevel;
       template <int>
