@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2017 by the deal.II authors
+// Copyright (C) 2010 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -17,13 +17,15 @@
 #define dealii_fe_dg_vector_h
 
 #include <deal.II/base/config.h>
-#include <deal.II/base/table.h>
-#include <deal.II/base/polynomials_raviart_thomas.h>
-#include <deal.II/base/polynomials_nedelec.h>
-#include <deal.II/base/polynomials_bdm.h>
-#include <deal.II/base/polynomial.h>
-#include <deal.II/base/tensor_product_polynomials.h>
+
 #include <deal.II/base/geometry_info.h>
+#include <deal.II/base/polynomial.h>
+#include <deal.II/base/polynomials_bdm.h>
+#include <deal.II/base/polynomials_nedelec.h>
+#include <deal.II/base/polynomials_raviart_thomas.h>
+#include <deal.II/base/table.h>
+#include <deal.II/base/tensor_product_polynomials.h>
+
 #include <deal.II/fe/fe.h>
 #include <deal.II/fe/fe_poly_tensor.h>
 
@@ -49,27 +51,26 @@ DEAL_II_NAMESPACE_OPEN
  * @author Guido Kanschat
  * @date 2010
  */
-template <class PolynomialType, int dim, int spacedim=dim>
-class FE_DGVector
-  :
-  public FE_PolyTensor<PolynomialType, dim, spacedim>
+template <class PolynomialType, int dim, int spacedim = dim>
+class FE_DGVector : public FE_PolyTensor<dim, spacedim>
 {
 public:
   /**
    * Constructor for the vector element of degree @p p.
    */
-  FE_DGVector (const unsigned int p, MappingType m);
+  FE_DGVector(const unsigned int p, MappingKind m);
 
   /**
    * Return a string that uniquely identifies a finite element. This class
-   * returns <tt>FE_RaviartThomas<dim>(degree)</tt>, with @p dim and @p degree
+   * returns `FE_DGVector_` plus a piece of the name that is taken from what
+   * the polynomial object returns, plus `<dim>(degree)`, with @p dim and @p degree
    * replaced by appropriate values.
    */
-  virtual std::string get_name () const;
+  virtual std::string
+  get_name() const override;
 
-  virtual
-  std::unique_ptr<FiniteElement<dim,spacedim> >
-  clone() const;
+  virtual std::unique_ptr<FiniteElement<dim, spacedim>>
+  clone() const override;
 
   /**
    * This function returns @p true, if the shape function @p shape_index has
@@ -77,10 +78,12 @@ public:
    *
    * For this element, we always return @p true.
    */
-  virtual bool has_support_on_face (const unsigned int shape_index,
-                                    const unsigned int face_index) const;
+  virtual bool
+  has_support_on_face(const unsigned int shape_index,
+                      const unsigned int face_index) const override;
 
-  virtual std::size_t memory_consumption () const;
+  virtual std::size_t
+  memory_consumption() const override;
 
 private:
   /**
@@ -90,7 +93,7 @@ private:
    * FiniteElementData.
    */
   static std::vector<unsigned int>
-  get_dpo_vector (const unsigned int degree);
+  get_dpo_vector(const unsigned int degree);
 
   /**
    * Fields of cell-independent data.
@@ -112,7 +115,7 @@ private:
      * space cell is then simply done by multiplication with the Jacobian of
      * the mapping.
      */
-    std::vector<std::vector<Tensor<1,dim> > > shape_values;
+    std::vector<std::vector<Tensor<1, dim>>> shape_values;
 
     /**
      * Array with shape function gradients in quadrature points. There is one
@@ -123,7 +126,7 @@ private:
      * then only have to apply the transformation (which is a matrix-vector
      * multiplication) when visiting an actual cell.
      */
-    std::vector<std::vector<Tensor<2,dim> > > shape_gradients;
+    std::vector<std::vector<Tensor<2, dim>>> shape_gradients;
   };
   Table<3, double> interior_weights;
 };
@@ -132,12 +135,16 @@ private:
 
 /**
  * A vector-valued DG element based on the polynomials space of FE_Nedelec.
+ * This class implements a "broken" finite element
+ * space that is discontinuous between cells and on each cell has shape
+ * functions that equal those of the Nedelec element.
  *
+ * The related class FE_DGRT is used in step-61.
  * @ingroup fe
  * @author Guido Kanschat
  * @date 2011
  */
-template <int dim, int spacedim=dim>
+template <int dim, int spacedim = dim>
 class FE_DGNedelec : public FE_DGVector<PolynomialsNedelec<dim>, dim, spacedim>
 {
 public:
@@ -145,67 +152,80 @@ public:
    * Constructor for the discontinuous N&eacute;d&eacute;lec element of degree
    * @p p.
    */
-  FE_DGNedelec (const unsigned int p);
+  FE_DGNedelec(const unsigned int p);
 
   /**
    * Return a string that uniquely identifies a finite element. This class
    * returns <tt>FE_DGNedelec<dim>(degree)</tt>, with @p dim and @p degree
    * replaced by appropriate values.
    */
-  virtual std::string get_name () const;
+  virtual std::string
+  get_name() const override;
 };
 
 
 
 /**
  * A vector-valued DG element based on the polynomials space of
- * FE_RaviartThomas.
+ * FE_RaviartThomas. This class implements a "broken" finite element
+ * space that is discontinuous between cells and on each cell has shape
+ * functions that equal those of the Raviart-Thomas element.
+ *
+ * The class is used in step-61.
  *
  * @ingroup fe
  * @author Guido Kanschat
  * @date 2011
  */
-template <int dim, int spacedim=dim>
-class FE_DGRaviartThomas : public FE_DGVector<PolynomialsRaviartThomas<dim>, dim, spacedim>
+template <int dim, int spacedim = dim>
+class FE_DGRaviartThomas
+  : public FE_DGVector<PolynomialsRaviartThomas<dim>, dim, spacedim>
 {
 public:
   /**
    * Constructor for the Raviart-Thomas element of degree @p p.
    */
-  FE_DGRaviartThomas (const unsigned int p);
+  FE_DGRaviartThomas(const unsigned int p);
 
   /**
    * Return a string that uniquely identifies a finite element. This class
    * returns <tt>FE_DGRaviartThomas<dim>(degree)</tt>, with @p dim and @p
    * degree replaced by appropriate values.
    */
-  virtual std::string get_name () const;
+  virtual std::string
+  get_name() const override;
 };
 
 
 
 /**
  * A vector-valued DG element based on the polynomials space of FE_BDM.
+ * This class implements a "broken" finite element
+ * space that is discontinuous between cells and on each cell has shape
+ * functions that equal those of the BDM element.
+ *
+ * The related class FE_DGRT is used in step-61.
  *
  * @ingroup fe
  * @author Guido Kanschat
  * @date 2011
  */
-template <int dim, int spacedim=dim>
+template <int dim, int spacedim = dim>
 class FE_DGBDM : public FE_DGVector<PolynomialsBDM<dim>, dim, spacedim>
 {
 public:
   /**
    * Constructor for the discontinuous BDM element of degree @p p.
    */
-  FE_DGBDM (const unsigned int p);
+  FE_DGBDM(const unsigned int p);
 
   /**
    * Return a string that uniquely identifies a finite element. This class
    * returns <tt>FE_DGBDM<dim>(degree)</tt>, with @p dim and @p degree
    * replaced by appropriate values.
    */
-  virtual std::string get_name () const;
+  virtual std::string
+  get_name() const override;
 };
 
 
