@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2018 by the deal.II authors
+// Copyright (C) 2001 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -25,10 +25,13 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 template <int dim, int spacedim>
 class MappingQ;
 template <int dim>
 class Quadrature;
+#endif
 
 /*!@addtogroup fe */
 /*@{*/
@@ -317,17 +320,6 @@ public:
     const std::vector<Vector<double>> &support_point_values,
     std::vector<double> &              nodal_values) const override;
 
-  /**
-   * Determine an estimate for the memory consumption (in bytes) of this
-   * object.
-   *
-   * This function is made virtual, since finite element objects are usually
-   * accessed through pointers to their base class, rather than the class
-   * itself.
-   */
-  virtual std::size_t
-  memory_consumption() const override;
-
   virtual std::unique_ptr<FiniteElement<dim, spacedim>>
   clone() const override;
 
@@ -377,15 +369,11 @@ private:
    */
   mutable Threads::Mutex mutex;
 
-  /**
-   * Allow access from other dimensions.
-   */
+  // Allow access from other dimensions.
   template <int dim1, int spacedim1>
   friend class FE_DGQ;
 
-  /**
-   * Allow @p MappingQ class to access to build_renumbering function.
-   */
+  // Allow @p MappingQ class to access to build_renumbering function.
   template <int dim1, int spacedim1>
   friend class MappingQ;
 };
