@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2018 by the deal.II authors
+// Copyright (C) 2006 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 #define dealii_fe_poly_templates_h
 
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/polynomial_space.h>
 #include <deal.II/base/qprojector.h>
 #include <deal.II/base/tensor_product_polynomials.h>
@@ -25,6 +27,7 @@
 
 #include <deal.II/fe/fe_poly.h>
 #include <deal.II/fe/fe_values.h>
+#include <deal.II/fe/mapping_cartesian.h>
 
 
 DEAL_II_NAMESPACE_OPEN
@@ -57,7 +60,7 @@ double
 FE_Poly<PolynomialType, dim, spacedim>::shape_value(const unsigned int i,
                                                     const Point<dim> & p) const
 {
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
+  AssertIndexRange(i, this->dofs_per_cell);
   return poly_space.compute_value(i, p);
 }
 
@@ -70,8 +73,8 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_value_component(
   const unsigned int component) const
 {
   (void)component;
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
-  Assert(component == 0, ExcIndexRange(component, 0, 1));
+  AssertIndexRange(i, this->dofs_per_cell);
+  AssertIndexRange(component, 1);
   return poly_space.compute_value(i, p);
 }
 
@@ -82,7 +85,7 @@ Tensor<1, dim>
 FE_Poly<PolynomialType, dim, spacedim>::shape_grad(const unsigned int i,
                                                    const Point<dim> & p) const
 {
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
+  AssertIndexRange(i, this->dofs_per_cell);
   return poly_space.template compute_derivative<1>(i, p);
 }
 
@@ -96,8 +99,8 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_grad_component(
   const unsigned int component) const
 {
   (void)component;
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
-  Assert(component == 0, ExcIndexRange(component, 0, 1));
+  AssertIndexRange(i, this->dofs_per_cell);
+  AssertIndexRange(component, 1);
   return poly_space.template compute_derivative<1>(i, p);
 }
 
@@ -109,7 +112,7 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_grad_grad(
   const unsigned int i,
   const Point<dim> & p) const
 {
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
+  AssertIndexRange(i, this->dofs_per_cell);
   return poly_space.template compute_derivative<2>(i, p);
 }
 
@@ -123,8 +126,8 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_grad_grad_component(
   const unsigned int component) const
 {
   (void)component;
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
-  Assert(component == 0, ExcIndexRange(component, 0, 1));
+  AssertIndexRange(i, this->dofs_per_cell);
+  AssertIndexRange(component, 1);
   return poly_space.template compute_derivative<2>(i, p);
 }
 
@@ -136,7 +139,7 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_3rd_derivative(
   const unsigned int i,
   const Point<dim> & p) const
 {
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
+  AssertIndexRange(i, this->dofs_per_cell);
   return poly_space.template compute_derivative<3>(i, p);
 }
 
@@ -150,8 +153,8 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_3rd_derivative_component(
   const unsigned int component) const
 {
   (void)component;
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
-  Assert(component == 0, ExcIndexRange(component, 0, 1));
+  AssertIndexRange(i, this->dofs_per_cell);
+  AssertIndexRange(component, 1);
   return poly_space.template compute_derivative<3>(i, p);
 }
 
@@ -163,7 +166,7 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_4th_derivative(
   const unsigned int i,
   const Point<dim> & p) const
 {
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
+  AssertIndexRange(i, this->dofs_per_cell);
   return poly_space.template compute_derivative<4>(i, p);
 }
 
@@ -177,8 +180,8 @@ FE_Poly<PolynomialType, dim, spacedim>::shape_4th_derivative_component(
   const unsigned int component) const
 {
   (void)component;
-  Assert(i < this->dofs_per_cell, ExcIndexRange(i, 0, this->dofs_per_cell));
-  Assert(component == 0, ExcIndexRange(component, 0, 1));
+  AssertIndexRange(i, this->dofs_per_cell);
+  AssertIndexRange(component, 1);
   return poly_space.template compute_derivative<4>(i, p);
 }
 
@@ -221,6 +224,46 @@ FE_Poly<PolynomialType, dim, spacedim>::requires_update_flags(
 //---------------------------------------------------------------------------
 
 
+
+/**
+ * Returns whether we need to correct the Hessians and third derivatives with
+ * the derivatives of the Jacobian. This is determined by checking if
+ * the jacobian_pushed_forward are zero.
+ *
+ * Especially for the third derivatives, the correction term is very expensive,
+ * which is why we check if the derivatives are zero before computing the
+ * correction.
+ */
+template <int dim, int spacedim>
+bool
+higher_derivatives_need_correcting(
+  const Mapping<dim, spacedim> &mapping,
+  const internal::FEValuesImplementation::MappingRelatedData<dim, spacedim>
+    &                mapping_data,
+  const unsigned int n_q_points,
+  const UpdateFlags  update_flags)
+{
+  // If higher derivatives weren't requested we don't need to correct them.
+  const bool update_higher_derivatives =
+    (update_flags & update_hessians) || (update_flags & update_3rd_derivatives);
+  if (!update_higher_derivatives)
+    return false;
+
+  // If we have a Cartesian mapping, we know that jacoban_pushed_forward_grads
+  // are identically zero.
+  if (dynamic_cast<const MappingCartesian<dim> *>(&mapping))
+    return false;
+
+  // Here, we should check if jacobian_pushed_forward_grads are zero at the
+  // quadrature points. This is yet to be implemented.
+  (void)mapping_data;
+  (void)n_q_points;
+
+  return true;
+}
+
+
+
 template <class PolynomialType, int dim, int spacedim>
 void
 FE_Poly<PolynomialType, dim, spacedim>::fill_fe_values(
@@ -247,6 +290,12 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_values(
 
   const UpdateFlags flags(fe_data.update_each);
 
+  const bool need_to_correct_higher_derivatives =
+    higher_derivatives_need_correcting(mapping,
+                                       mapping_data,
+                                       quadrature.size(),
+                                       flags);
+
   // transform gradients and higher derivatives. there is nothing to do
   // for values since we already emplaced them into output_data when
   // we were in get_data()
@@ -266,12 +315,8 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_values(
                           mapping_internal,
                           make_array_view(output_data.shape_hessians, k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        for (unsigned int i = 0; i < quadrature.size(); ++i)
-          for (unsigned int j = 0; j < spacedim; ++j)
-            output_data.shape_hessians[k][i] -=
-              mapping_data.jacobian_pushed_forward_grads[i][j] *
-              output_data.shape_gradients[k][i][j];
+      if (need_to_correct_higher_derivatives)
+        correct_hessians(output_data, mapping_data, quadrature.size());
     }
 
   if (flags & update_3rd_derivatives &&
@@ -284,11 +329,8 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_values(
                           make_array_view(output_data.shape_3rd_derivatives,
                                           k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        correct_third_derivatives(output_data,
-                                  mapping_data,
-                                  quadrature.size(),
-                                  k);
+      if (need_to_correct_higher_derivatives)
+        correct_third_derivatives(output_data, mapping_data, quadrature.size());
     }
 }
 
@@ -331,6 +373,12 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_face_values(
 
   const UpdateFlags flags(fe_data.update_each);
 
+  const bool need_to_correct_higher_derivatives =
+    higher_derivatives_need_correcting(mapping,
+                                       mapping_data,
+                                       quadrature.size(),
+                                       flags);
+
   // transform gradients and higher derivatives. we also have to copy
   // the values (unlike in the case of fill_fe_values()) since
   // we need to take into account the offsets
@@ -356,12 +404,8 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_face_values(
           mapping_internal,
           make_array_view(output_data.shape_hessians, k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        for (unsigned int i = 0; i < quadrature.size(); ++i)
-          for (unsigned int j = 0; j < spacedim; ++j)
-            output_data.shape_hessians[k][i] -=
-              mapping_data.jacobian_pushed_forward_grads[i][j] *
-              output_data.shape_gradients[k][i][j];
+      if (need_to_correct_higher_derivatives)
+        correct_hessians(output_data, mapping_data, quadrature.size());
     }
 
   if (flags & update_3rd_derivatives)
@@ -376,11 +420,8 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_face_values(
                           make_array_view(output_data.shape_3rd_derivatives,
                                           k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        correct_third_derivatives(output_data,
-                                  mapping_data,
-                                  quadrature.size(),
-                                  k);
+      if (need_to_correct_higher_derivatives)
+        correct_third_derivatives(output_data, mapping_data, quadrature.size());
     }
 }
 
@@ -426,6 +467,12 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_subface_values(
 
   const UpdateFlags flags(fe_data.update_each);
 
+  const bool need_to_correct_higher_derivatives =
+    higher_derivatives_need_correcting(mapping,
+                                       mapping_data,
+                                       quadrature.size(),
+                                       flags);
+
   // transform gradients and higher derivatives. we also have to copy
   // the values (unlike in the case of fill_fe_values()) since
   // we need to take into account the offsets
@@ -451,12 +498,8 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_subface_values(
           mapping_internal,
           make_array_view(output_data.shape_hessians, k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        for (unsigned int i = 0; i < quadrature.size(); ++i)
-          for (unsigned int j = 0; j < spacedim; ++j)
-            output_data.shape_hessians[k][i] -=
-              mapping_data.jacobian_pushed_forward_grads[i][j] *
-              output_data.shape_gradients[k][i][j];
+      if (need_to_correct_higher_derivatives)
+        correct_hessians(output_data, mapping_data, quadrature.size());
     }
 
   if (flags & update_3rd_derivatives)
@@ -471,16 +514,32 @@ FE_Poly<PolynomialType, dim, spacedim>::fill_fe_subface_values(
                           make_array_view(output_data.shape_3rd_derivatives,
                                           k));
 
-      for (unsigned int k = 0; k < this->dofs_per_cell; ++k)
-        correct_third_derivatives(output_data,
-                                  mapping_data,
-                                  quadrature.size(),
-                                  k);
+      if (need_to_correct_higher_derivatives)
+        correct_third_derivatives(output_data, mapping_data, quadrature.size());
     }
 }
 
 
 
+template <class PolynomialType, int dim, int spacedim>
+inline void
+FE_Poly<PolynomialType, dim, spacedim>::correct_hessians(
+  internal::FEValuesImplementation::FiniteElementRelatedData<dim, spacedim>
+    &output_data,
+  const internal::FEValuesImplementation::MappingRelatedData<dim, spacedim>
+    &                mapping_data,
+  const unsigned int n_q_points) const
+{
+  for (unsigned int dof = 0; dof < this->dofs_per_cell; ++dof)
+    for (unsigned int i = 0; i < n_q_points; ++i)
+      for (unsigned int j = 0; j < spacedim; ++j)
+        output_data.shape_hessians[dof][i] -=
+          mapping_data.jacobian_pushed_forward_grads[i][j] *
+          output_data.shape_gradients[dof][i][j];
+}
+
+
+
 template <class PolynomialType, int dim, int spacedim>
 inline void
 FE_Poly<PolynomialType, dim, spacedim>::correct_third_derivatives(
@@ -488,26 +547,26 @@ FE_Poly<PolynomialType, dim, spacedim>::correct_third_derivatives(
     &output_data,
   const internal::FEValuesImplementation::MappingRelatedData<dim, spacedim>
     &                mapping_data,
-  const unsigned int n_q_points,
-  const unsigned int dof) const
+  const unsigned int n_q_points) const
 {
-  for (unsigned int i = 0; i < n_q_points; ++i)
-    for (unsigned int j = 0; j < spacedim; ++j)
-      for (unsigned int k = 0; k < spacedim; ++k)
-        for (unsigned int l = 0; l < spacedim; ++l)
-          for (unsigned int m = 0; m < spacedim; ++m)
-            {
-              output_data.shape_3rd_derivatives[dof][i][j][k][l] -=
-                (mapping_data.jacobian_pushed_forward_grads[i][m][j][l] *
-                 output_data.shape_hessians[dof][i][k][m]) +
-                (mapping_data.jacobian_pushed_forward_grads[i][m][k][l] *
-                 output_data.shape_hessians[dof][i][j][m]) +
-                (mapping_data.jacobian_pushed_forward_grads[i][m][j][k] *
-                 output_data.shape_hessians[dof][i][l][m]) +
-                (mapping_data
-                   .jacobian_pushed_forward_2nd_derivatives[i][m][j][k][l] *
-                 output_data.shape_gradients[dof][i][m]);
-            }
+  for (unsigned int dof = 0; dof < this->dofs_per_cell; ++dof)
+    for (unsigned int i = 0; i < n_q_points; ++i)
+      for (unsigned int j = 0; j < spacedim; ++j)
+        for (unsigned int k = 0; k < spacedim; ++k)
+          for (unsigned int l = 0; l < spacedim; ++l)
+            for (unsigned int m = 0; m < spacedim; ++m)
+              {
+                output_data.shape_3rd_derivatives[dof][i][j][k][l] -=
+                  (mapping_data.jacobian_pushed_forward_grads[i][m][j][l] *
+                   output_data.shape_hessians[dof][i][k][m]) +
+                  (mapping_data.jacobian_pushed_forward_grads[i][m][k][l] *
+                   output_data.shape_hessians[dof][i][j][m]) +
+                  (mapping_data.jacobian_pushed_forward_grads[i][m][j][k] *
+                   output_data.shape_hessians[dof][i][l][m]) +
+                  (mapping_data
+                     .jacobian_pushed_forward_2nd_derivatives[i][m][j][k][l] *
+                   output_data.shape_gradients[dof][i][m]);
+              }
 }
 
 namespace internal
@@ -580,6 +639,16 @@ FE_Poly<PolynomialType, dim, spacedim>::get_poly_space_numbering_inverse() const
 
 
 
+template <class PolynomialType, int dim, int spacedim>
+std::size_t
+FE_Poly<PolynomialType, dim, spacedim>::memory_consumption() const
+{
+  return FiniteElement<dim, spacedim>::memory_consumption() +
+         poly_space.memory_consumption();
+}
+
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
