@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2018 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -35,9 +35,11 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declaration
+#  ifndef DOXYGEN
 template <int dim, int spacedim>
 class FE_Enriched;
-
+#  endif
 
 /**
  * This class provides an interface to group several elements together into
@@ -527,7 +529,13 @@ public:
   /**
    * Move constructor.
    */
-  FESystem(FESystem<dim, spacedim> &&) = default; // NOLINT
+  FESystem(FESystem<dim, spacedim> &&other_fe_system) noexcept
+    : FiniteElement<dim, spacedim>(std::move(other_fe_system))
+  {
+    base_elements = std::move(other_fe_system.base_elements);
+    generalized_support_points_index_table =
+      std::move(other_fe_system.generalized_support_points_index_table);
+  }
 
   /**
    * Destructor.
@@ -1226,10 +1234,10 @@ private:
       base_fe_output_objects;
   };
 
-  /*
+  /**
    * Mutex for protecting initialization of restriction and embedding matrix.
    */
-  mutable Threads::Mutex mutex;
+  mutable std::mutex mutex;
 
   friend class FE_Enriched<dim, spacedim>;
 };
@@ -1263,8 +1271,7 @@ namespace internal
     std::pair<std::unique_ptr<FiniteElement<dim, spacedim>>, unsigned int>
     promote_to_fe_pair(const FiniteElement<dim, spacedim> &fe)
     {
-      return std::make_pair<std::unique_ptr<FiniteElement<dim, spacedim>>,
-                            unsigned int>(std::move(fe.clone()), 1u);
+      return std::make_pair(std::move(fe.clone()), 1u);
     }
 
 
