@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2019 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,15 +29,18 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 template <int dim, int space_dim>
 class Triangulation;
 template <int dim>
 struct CellData;
+#endif
 
 /**
  * This class implements an input mechanism for grid data. It allows to read a
  * grid structure into a triangulation object. At present, UCD (unstructured
- * cell data), DB Mesh, XDA, Gmsh, Tecplot, NetCDF, UNV, VTK, ASSIMP, and Cubit
+ * cell data), DB Mesh, XDA, %Gmsh, Tecplot, NetCDF, UNV, VTK, ASSIMP, and Cubit
  * are supported as input format for grid data. Any numerical data other than
  * geometric (vertex locations) and topological (how vertices form cells,
  * faces, and edges) information is ignored, but the readers for the various
@@ -142,14 +145,14 @@ struct CellData;
  * several example files. If the reader does not grok your files, it should be
  * fairly simple to extend it.
  *
- * <li> <tt>Gmsh 1.0 mesh</tt> format: this format is used by the @p Gmsh mesh
- * generator (see http://www.geuz.org/gmsh/). The documentation in the @p Gmsh
+ * <li> <tt>%Gmsh 1.0 mesh</tt> format: this format is used by the @p %Gmsh mesh
+ * generator (see http://gmsh.info/). The documentation in the @p %Gmsh
  * manual explains how to generate meshes compatible with the deal.II library
- * (i.e. quads rather than triangles). In order to use this format, Gmsh has
+ * (i.e. quads rather than triangles). In order to use this format, %Gmsh has
  * to output the file in the old format 1.0. This is done adding the line
  * "Mesh.MshFileVersion = 1" to the input file.
  *
- * <li> <tt>Gmsh 2.0 mesh</tt> format: this is a variant of the above format.
+ * <li> <tt>%Gmsh 2.0 mesh</tt> format: this is a variant of the above format.
  * The read_msh() function automatically determines whether an input file is
  * version 1 or version 2.
  *
@@ -330,6 +333,8 @@ public:
     tecplot,
     /// Use read_vtk()
     vtk,
+    /// Use read_vtu()
+    vtu,
     /// Use read_assimp()
     assimp,
   };
@@ -401,6 +406,26 @@ public:
   void
   read_vtk(std::istream &in);
 
+  /**
+   * Read grid data from a unstructured vtu file, saved by deal.II using
+   * GridOut::write_vtu(), with the flag
+   * GridOutFlags::Vtu::serialize_triangulation set to true.
+   *
+   * Notice that this function does not support reading in arbitrary vtu files,
+   * but only files that were written by deal.II itself, using the function
+   * GridOut::write_vtu and setting GridOutFlags::Vtu::serialize_triangulation
+   * to true.
+   *
+   * When this flag is set to true, the generated vtu file contains the
+   * triangulation in a xml section which is ignored by vtu general vtu readers.
+   * If this section is absent, an exception is thrown.
+   *
+   * @author Luca Heltai, Nicola Giuliani, 2020
+   */
+  void
+  read_vtu(std::istream &in);
+
+
   /**
    * Read grid data from an unv file as generated by the Salome mesh
    * generator. Numerical data is ignored.
@@ -481,12 +506,12 @@ public:
 
   /**
    * Read grid data from an msh file, either version 1 or version 2 of that
-   * file format. The Gmsh formats are documented at
-   * http://www.geuz.org/gmsh/.
+   * file format. The %Gmsh formats are documented at
+   * http://www.gmsh.info/.
    *
    * @note The input function of deal.II does not distinguish between newline
    * and other whitespace. Therefore, deal.II will be able to read files in a
-   * slightly more general format than Gmsh.
+   * slightly more general format than %Gmsh.
    */
   void
   read_msh(std::istream &in);
