@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2019 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -110,6 +110,14 @@ public:
   virtual DerivativeForm<1, spacedim, spacedim>
   push_forward_gradient(const Point<spacedim> &chart_point) const override;
 
+  /**
+   * @copydoc Manifold::normal_vector()
+   */
+  virtual Tensor<1, spacedim>
+  normal_vector(
+    const typename Triangulation<dim, spacedim>::face_iterator &face,
+    const Point<spacedim> &p) const override;
+
   /**
    * The center of the spherical coordinate system.
    */
@@ -213,7 +221,7 @@ private:
  * Manifold to the cell containing the center. It is advisable to combine this
  * class with TransfiniteInterpolationManifold to ensure a smooth transition
  * from a curved shape to the straight coordinate system in the center of the
- * ball.
+ * ball. (See also the extensive discussion in step-65.)
  *
  * @ingroup manifold
  *
@@ -255,7 +263,7 @@ public:
                      const Point<spacedim> &x2) const override;
 
   /**
-   * Return the (normalized) normal vector at the point @p p.
+   * @copydoc Manifold::normal_vector()
    */
   virtual Tensor<1, spacedim>
   normal_vector(
@@ -593,6 +601,10 @@ public:
    *
    * The tolerance argument is used in debug mode to actually check that the
    * two functions are one the inverse of the other.
+   *
+   * Note: the object constructed in this way stores pointers to the
+   * push_forward and  pull_back functions. Therefore, one must guarantee that
+   * the function objects are destroyed only after the constructed manifold.
    */
   FunctionManifold(
     const Function<chartdim> & push_forward_function,
@@ -600,6 +612,23 @@ public:
     const Tensor<1, chartdim> &periodicity = Tensor<1, chartdim>(),
     const double               tolerance   = 1e-10);
 
+  /**
+   * Same as previous, except this constructor takes ownership of the Function
+   * objects passed as first and second argument, and is ultimately in charge
+   * of deleting the pointers when the FunctionManifold object is destroyed.
+   *
+   * This constructor is useful because it allows creating function objects at
+   * the place of calling the constructor without having to name and later
+   * delete these objects. This allows the following idiom:
+   * FunctionManifold<dim> manifold(std_cxx14::make_unique<MyPushForward>(...),
+   *                                std_cxx14::make_unique<MyPullBack>(...));
+   */
+  FunctionManifold(
+    std::unique_ptr<Function<chartdim>> push_forward,
+    std::unique_ptr<Function<spacedim>> pull_back,
+    const Tensor<1, chartdim> &         periodicity = Tensor<1, chartdim>(),
+    const double                        tolerance   = 1e-10);
+
   /**
    * Expressions constructor. Takes the expressions of the push_forward
    * function of spacedim components, and of the pull_back function of @p
@@ -711,7 +740,7 @@ private:
    * destructor will delete the function objects pointed to be the two
    * pointers.
    */
-  const bool owns_pointers;
+  bool owns_pointers;
 
   /**
    * The expression used to construct the push_forward function.
@@ -805,6 +834,7 @@ private:
  * assumed to be given by another manifold (e.g. a polar manifold on a circle).
  * The mechanism to extend the boundary information is a so-called transfinite
  * interpolation.
+ * The use of this class is discussed extensively in step-65.
  *
  * The formula for extending such a description in 2D is, for example,
  * described on
