@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2019 by the deal.II authors
+// Copyright (C) 2005 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -26,8 +26,6 @@
 #include <deal.II/base/smartpointer.h>
 #include <deal.II/base/template_constraints.h>
 
-#include <deal.II/distributed/cell_data_transfer.templates.h>
-
 #include <deal.II/dofs/deprecated_function_map.h>
 #include <deal.II/dofs/dof_accessor.h>
 #include <deal.II/dofs/dof_iterator_selector.h>
@@ -44,9 +42,20 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 template <int dim, int spacedim>
 class Triangulation;
 
+namespace parallel
+{
+  namespace distributed
+  {
+    template <int dim, int spacedim, typename VectorType>
+    class CellDataTransfer;
+  }
+} // namespace parallel
+
 namespace internal
 {
   namespace DoFHandlerImplementation
@@ -84,7 +93,7 @@ namespace internal
     struct Implementation;
   }
 } // namespace internal
-
+#endif
 
 
 namespace hp
@@ -128,14 +137,14 @@ namespace hp
    * - When coarsening cells, the (now active) parent cell will be assigned
    *   an active FE index that is determined from its (no longer active)
    *   children, following the FiniteElementDomination logic: Out of the set of
-   *   elements previously assigned to the former children, we choose the
-   *   dominating one for the parent cell. If none was found, we pick the least
-   *   dominant element in the whole collection that dominates all former
-   *   children. See hp::FECollection::find_dominating_fe_extended() for further
-   *   information on this topic.
+   *   elements previously assigned to the former children, we choose the one
+   *   dominated by all children for the parent cell. If none was found, we pick
+   *   the most dominant element in the whole collection that is dominated by
+   *   all former children. See hp::FECollection::find_dominated_fe_extended()
+   *   for further information on this topic.
    *
-   * @note Finite elements need to be assigned to each cell by calling
-   * distribute_dofs() first to make this functionality available.
+   * @note Finite elements need to be assigned to each cell by either calling
+   * set_fe() or distribute_dofs() first to make this functionality available.
    *
    *
    * <h3>Active FE indices and parallel meshes</h3>
@@ -351,21 +360,6 @@ namespace hp
      */
     static const bool is_hp_dof_handler = true;
 
-    /**
-     * When the arrays holding the DoF indices are set up, but before they are
-     * filled with actual values, they are set to an invalid value, in order
-     * to monitor possible problems. This invalid value is the constant
-     * defined here.
-     *
-     * Please note that you should not rely on it having a certain value, but
-     * rather take its symbolic name.
-     *
-     * @deprecated Use numbers::invalid_dof_index instead.
-     */
-    DEAL_II_DEPRECATED
-    static const types::global_dof_index invalid_dof_index =
-      numbers::invalid_dof_index;
-
     /**
      * The default index of the finite element to be used on a given cell. For
      * the usual, non-hp dealii::DoFHandler class that only supports the same
@@ -422,7 +416,7 @@ namespace hp
     /**
      * Assign a hp::FECollection @p fe to this object.
      *
-     * In case a parallel::Triangulation is assigned to this object,
+     * In case a parallel::TriangulationBase is assigned to this object,
      * the active_fe_indices will be exchanged between processors so that
      * each one knows the indices on its own cells and all ghost cells.
      *
@@ -769,7 +763,8 @@ namespace hp
     n_dofs(const unsigned int level) const;
 
     /**
-     * Return the number of degrees of freedom located on the boundary.
+     * Return the number of locally owned degrees of freedom located on the
+     * boundary.
      */
     types::global_dof_index
     n_boundary_dofs() const;
@@ -791,8 +786,9 @@ namespace hp
         &boundary_ids) const;
 
     /**
-     * Return the number of degrees of freedom located on those parts of the
-     * boundary which have a boundary indicator listed in the given set. The
+     * Return the number of locally owned degrees of freedom located on those
+     * parts of the boundary which have a boundary indicator listed in the given
+     * set.
      */
     types::global_dof_index
     n_boundary_dofs(const std::set<types::boundary_id> &boundary_ids) const;
@@ -827,19 +823,19 @@ namespace hp
 
     /**
      * Return a vector that stores the locally owned DoFs of each processor.
-     * If you are only interested in the number of elements each processor
-     * owns then n_dofs_per_processor() is a better choice.
      *
-     * If this is a sequential DoFHandler, then the vector has a single element
-     * that equals the IndexSet representing the entire range [0,n_dofs()].
-     * (Here, "sequential" means that either
-     * the whole program does not use MPI, or that it uses MPI
-     * but only uses a single MPI process, or that there are multiple MPI
-     * processes but the Triangulation on which this DoFHandler builds
-     * works only on one MPI process.)
+     * @deprecated As of deal.II version 9.2, we do not populate a vector with
+     * the index sets of all processors by default any more due to a possibly
+     * large memory footprint on many processors. As a consequence, this
+     * function needs to call `Utilities::all_gather(comm,
+     * locally_owned_dofs())` upon the first invocation, including global
+     * communication. Use `Utilities::all_gather(comm,
+     * dof_handler.locally_owned_dofs())` instead if using up to a few thousands
+     * of MPI ranks or some variant involving local communication with more
+     * processors.
      */
-    const std::vector<IndexSet> &
-    locally_owned_dofs_per_processor() const;
+    DEAL_II_DEPRECATED const std::vector<IndexSet> &
+                             locally_owned_dofs_per_processor() const;
 
     /**
      * Return a vector that stores the number of degrees of freedom each
@@ -847,18 +843,18 @@ namespace hp
      * of all these numbers equals the number of degrees of freedom that exist
      * globally, i.e. what n_dofs() returns.
      *
-     * Each element of the vector returned by this function equals the number
-     * of elements of the corresponding sets returned by
-     * locally_owned_dofs_per_processor().
-     *
-     * If this is a sequential DoFHandler, then the vector has a single element
-     * equal to n_dofs(). (Here, "sequential" means that either the whole
-     * program does not use MPI, or that it uses MPI but only uses a single MPI
-     * process, or that there are multiple MPI processes but the Triangulation
-     * on which this DoFHandler builds works only on one MPI process.)
+     * @deprecated As of deal.II version 9.2, we do not populate a vector with
+     * the numbers of dofs of all processors by default any more due to a
+     * possibly large memory footprint on many processors. As a consequence,
+     * this function needs to call `Utilities::all_gather(comm,
+     * n_locally_owned_dofs()` upon the first invocation, including global
+     * communication. Use `Utilities::all_gather(comm,
+     * dof_handler.n_locally_owned_dofs()` instead if using up to a few
+     * thousands of MPI ranks or some variant involving local communication with
+     * more processors.
      */
-    const std::vector<types::global_dof_index> &
-    n_locally_owned_dofs_per_processor() const;
+    DEAL_II_DEPRECATED const std::vector<types::global_dof_index> &
+                             n_locally_owned_dofs_per_processor() const;
 
     /**
      * Return an IndexSet describing the set of locally owned DoFs used for
@@ -870,23 +866,21 @@ namespace hp
     locally_owned_mg_dofs(const unsigned int level) const;
 
     /**
-     * Return a vector that stores the locally owned level DoFs of each
-     * processor on the given level @p level. Since hp::DoFHandler does not
-     * support multilevel methods yet, this function throws an exception
-     * ExcNotImplemented() independent of its argument.
-     */
-    const std::vector<IndexSet> &
-    locally_owned_mg_dofs_per_processor(const unsigned int level) const;
-
-    /**
-     * Return a constant reference to the set of finite element objects that
-     * are used by this @p DoFHandler.
+     * Return a vector that stores the locally owned DoFs of each processor on
+     * the given level @p level.
      *
-     * @deprecated Use get_fe_collection() instead.
+     * @deprecated As of deal.II version 9.2, we do not populate a vector with
+     * the index sets of all processors by default any more due to a possibly
+     * large memory footprint on many processors. As a consequence, this
+     * function needs to call `Utilities::all_gather(comm,
+     * locally_owned_dofs_mg())` upon the first invocation, including global
+     * communication. Use `Utilities::all_gather(comm,
+     * dof_handler.locally_owned_dofs_mg())` instead if using up to a few
+     * thousands of MPI ranks or some variant involving local communication with
+     * more processors.
      */
-    DEAL_II_DEPRECATED
-    const hp::FECollection<dim, spacedim> &
-    get_fe() const;
+    DEAL_II_DEPRECATED const std::vector<IndexSet> &
+                             locally_owned_mg_dofs_per_processor(const unsigned int level) const;
 
     /**
      * Return a constant reference to the indexth finite element object that is
@@ -973,7 +967,19 @@ namespace hp
     void
     load(Archive &ar, const unsigned int version);
 
+#ifdef DOXYGEN
+    /**
+     * Write and read the data of this object from a stream for the purpose
+     * of serialization.
+     */
+    template <class Archive>
+    void
+    serialize(Archive &archive, const unsigned int version);
+#else
+    // This macro defines the serialize() method that is compatible with
+    // the templated save() and load() method that have been implemented.
     BOOST_SERIALIZATION_SPLIT_MEMBER()
+#endif
 
     /**
      * Exception
@@ -1282,9 +1288,7 @@ namespace hp
      */
     std::vector<boost::signals2::connection> tria_listeners;
 
-    /**
-     * Make accessor objects friends.
-     */
+    // Make accessor objects friends.
     template <int, class, bool>
     friend class dealii::DoFAccessor;
     template <class, bool>
@@ -1293,10 +1297,8 @@ namespace hp
     friend struct dealii::internal::DoFCellAccessorImplementation::
       Implementation;
 
-    /**
-     * Likewise for DoFLevel objects since they need to access the vertex dofs
-     * in the functions that set and retrieve vertex dof indices.
-     */
+    // Likewise for DoFLevel objects since they need to access the vertex dofs
+    // in the functions that set and retrieve vertex dof indices.
     template <int>
     friend class dealii::internal::hp::DoFIndicesOnFacesOrEdges;
     friend struct dealii::internal::hp::DoFHandlerImplementation::
@@ -1334,6 +1336,38 @@ namespace hp
     // then just hand everything over to the other function that does the work
     return n_boundary_dofs(boundary_ids_only);
   }
+
+
+
+  template <>
+  inline types::global_dof_index
+  DoFHandler<2, 3>::n_boundary_dofs() const
+  {
+    Assert(false, ExcNotImplemented());
+    return 0;
+  }
+
+
+
+  template <>
+  template <typename number>
+  inline types::global_dof_index
+  DoFHandler<2, 3>::n_boundary_dofs(
+    const std::map<types::boundary_id, const Function<3, number> *> &) const
+  {
+    Assert(false, ExcNotImplemented());
+    return 0;
+  }
+
+
+
+  template <>
+  inline types::global_dof_index
+  DoFHandler<2, 3>::n_boundary_dofs(const std::set<types::boundary_id> &) const
+  {
+    Assert(false, ExcNotImplemented());
+    return 0;
+  }
 }
 
 
@@ -1490,6 +1524,24 @@ namespace hp
   const std::vector<types::global_dof_index> &
   DoFHandler<dim, spacedim>::n_locally_owned_dofs_per_processor() const
   {
+    if (number_cache.n_locally_owned_dofs_per_processor.empty() &&
+        number_cache.n_global_dofs > 0)
+      {
+        MPI_Comm comm;
+
+        const parallel::TriangulationBase<dim, spacedim> *tr =
+          (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
+            &this->get_triangulation()));
+        if (tr != nullptr)
+          comm = tr->get_communicator();
+        else
+          comm = MPI_COMM_SELF;
+
+        const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
+          number_cache)
+          .n_locally_owned_dofs_per_processor =
+          number_cache.get_n_locally_owned_dofs_per_processor(comm);
+      }
     return number_cache.n_locally_owned_dofs_per_processor;
   }
 
@@ -1499,6 +1551,24 @@ namespace hp
   const std::vector<IndexSet> &
   DoFHandler<dim, spacedim>::locally_owned_dofs_per_processor() const
   {
+    if (number_cache.locally_owned_dofs_per_processor.empty() &&
+        number_cache.n_global_dofs > 0)
+      {
+        MPI_Comm comm;
+
+        const parallel::TriangulationBase<dim, spacedim> *tr =
+          (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
+            &this->get_triangulation()));
+        if (tr != nullptr)
+          comm = tr->get_communicator();
+        else
+          comm = MPI_COMM_SELF;
+
+        const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
+          number_cache)
+          .locally_owned_dofs_per_processor =
+          number_cache.get_locally_owned_dofs_per_processor(comm);
+      }
     return number_cache.locally_owned_dofs_per_processor;
   }
 
@@ -1512,33 +1582,44 @@ namespace hp
     Assert(false, ExcNotImplemented());
     (void)level;
     Assert(level < this->get_triangulation().n_global_levels(),
-           ExcMessage("invalid level in locally_owned_mg_dofs"));
+           ExcMessage("The given level index exceeds the number of levels "
+                      "present in the triangulation"));
     return mg_number_cache[0].locally_owned_dofs;
   }
 
 
+
   template <int dim, int spacedim>
   const std::vector<IndexSet> &
   DoFHandler<dim, spacedim>::locally_owned_mg_dofs_per_processor(
     const unsigned int level) const
   {
-    Assert(false, ExcNotImplemented());
-    (void)level;
     Assert(level < this->get_triangulation().n_global_levels(),
-           ExcMessage("invalid level in locally_owned_mg_dofs_per_processor"));
-    return mg_number_cache[0].locally_owned_dofs_per_processor;
-  }
+           ExcMessage("The given level index exceeds the number of levels "
+                      "present in the triangulation"));
+    Assert(
+      mg_number_cache.size() == this->get_triangulation().n_global_levels(),
+      ExcMessage(
+        "The level dofs are not set up properly! Did you call distribute_mg_dofs()?"));
+    if (mg_number_cache[level].locally_owned_dofs_per_processor.empty() &&
+        mg_number_cache[level].n_global_dofs > 0)
+      {
+        MPI_Comm comm;
 
+        const parallel::TriangulationBase<dim, spacedim> *tr =
+          (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
+            &this->get_triangulation()));
+        if (tr != nullptr)
+          comm = tr->get_communicator();
+        else
+          comm = MPI_COMM_SELF;
 
-
-  template <int dim, int spacedim>
-  inline const hp::FECollection<dim, spacedim> &
-  DoFHandler<dim, spacedim>::get_fe() const
-  {
-    Assert(fe_collection.size() > 0,
-           ExcMessage("No finite element collection is associated with "
-                      "this DoFHandler"));
-    return fe_collection;
+        const_cast<dealii::internal::DoFHandlerImplementation::NumberCache &>(
+          mg_number_cache[level])
+          .locally_owned_dofs_per_processor =
+          mg_number_cache[level].get_locally_owned_dofs_per_processor(comm);
+      }
+    return mg_number_cache[level].locally_owned_dofs_per_processor;
   }
 
 
