@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2018 by the deal.II authors
+// Copyright (C) 2005 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -164,7 +164,7 @@ namespace hp
   inline const Mapping<dim, spacedim> &MappingCollection<dim, spacedim>::
                                        operator[](const unsigned int index) const
   {
-    Assert(index < mappings.size(), ExcIndexRange(index, 0, mappings.size()));
+    AssertIndexRange(index, mappings.size());
     return *mappings[index];
   }
 
