@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2004 - 2019 by the deal.II authors
+// Copyright (C) 2004 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -37,9 +37,11 @@
 DEAL_II_NAMESPACE_OPEN
 
 
+// Forward declaration
+#ifndef DOXYGEN
 template <typename>
 class MatrixIterator;
-
+#endif
 
 
 /*! @addtogroup Matrix1
@@ -99,9 +101,7 @@ namespace BlockMatrixIterators
      */
     unsigned int col_block;
 
-    /**
-     * Let the iterator class be a friend.
-     */
+    // Let the iterator class be a friend.
     template <typename>
     friend class MatrixIterator;
   };
@@ -280,9 +280,7 @@ namespace BlockMatrixIterators
     bool
     operator==(const Accessor &a) const;
 
-    /**
-     * Let the iterator class be a friend.
-     */
+    // Let the iterator class be a friend.
     template <typename>
     friend class dealii::MatrixIterator;
   };
@@ -364,6 +362,7 @@ public:
    * library containers.
    */
   using value_type      = typename BlockType::value_type;
+  using real_type       = typename numbers::NumberTraits<value_type>::real_type;
   using pointer         = value_type *;
   using const_pointer   = const value_type *;
   using reference       = value_type &;
@@ -702,6 +701,13 @@ public:
   value_type
   matrix_norm_square(const BlockVectorType &v) const;
 
+  /**
+   * Return the frobenius norm of the matrix, i.e. the square root of the sum
+   * of squares of all entries in the matrix.
+   */
+  real_type
+  frobenius_norm() const;
+
   /**
    * Compute the matrix scalar product $\left(u,Mv\right)$.
    */
@@ -1034,11 +1040,11 @@ private:
      * A mutex variable used to guard access to the member variables of this
      * structure;
      */
-    Threads::Mutex mutex;
+    std::mutex mutex;
 
     /**
      * Copy operator. This is needed because the default copy operator of this
-     * class is deleted (since Threads::Mutex is not copyable) and hence the
+     * class is deleted (since std::mutex is not copyable) and hence the
      * default copy operator of the enclosing class is also deleted.
      *
      * The implementation here simply does nothing -- TemporaryData objects
@@ -1060,10 +1066,8 @@ private:
    */
   TemporaryData temporary_data;
 
-  /**
-   * Make the iterator class a friend. We have to work around a compiler bug
-   * here again.
-   */
+  // Make the iterator class a friend. We have to work around a compiler bug
+  // here again.
   template <typename, bool>
   friend class BlockMatrixIterators::Accessor;
 
@@ -1566,8 +1570,8 @@ inline typename BlockMatrixBase<MatrixType>::BlockType &
 BlockMatrixBase<MatrixType>::block(const unsigned int row,
                                    const unsigned int column)
 {
-  Assert(row < n_block_rows(), ExcIndexRange(row, 0, n_block_rows()));
-  Assert(column < n_block_cols(), ExcIndexRange(column, 0, n_block_cols()));
+  AssertIndexRange(row, n_block_rows());
+  AssertIndexRange(column, n_block_cols());
 
   return *sub_objects[row][column];
 }
@@ -1579,8 +1583,8 @@ inline const typename BlockMatrixBase<MatrixType>::BlockType &
 BlockMatrixBase<MatrixType>::block(const unsigned int row,
                                    const unsigned int column) const
 {
-  Assert(row < n_block_rows(), ExcIndexRange(row, 0, n_block_rows()));
-  Assert(column < n_block_cols(), ExcIndexRange(column, 0, n_block_cols()));
+  AssertIndexRange(row, n_block_rows());
+  AssertIndexRange(column, n_block_cols());
 
   return *sub_objects[row][column];
 }
@@ -2407,6 +2411,28 @@ BlockMatrixBase<MatrixType>::matrix_norm_square(const BlockVectorType &v) const
 
 
 
+template <class MatrixType>
+typename BlockMatrixBase<MatrixType>::real_type
+BlockMatrixBase<MatrixType>::frobenius_norm() const
+{
+  value_type norm_sqr = 0;
+
+  // For each block, get the Frobenius norm, and add the square to the
+  // accumulator for the full matrix
+  for (unsigned int row = 0; row < n_block_rows(); ++row)
+    {
+      for (unsigned int col = 0; col < n_block_cols(); ++col)
+        {
+          const value_type block_norm = block(row, col).frobenius_norm();
+          norm_sqr += block_norm * block_norm;
+        }
+    }
+
+  return std::sqrt(norm_sqr);
+}
+
+
+
 template <class MatrixType>
 template <class BlockVectorType>
 typename BlockMatrixBase<MatrixType>::value_type
@@ -2517,7 +2543,7 @@ template <class MatrixType>
 inline typename BlockMatrixBase<MatrixType>::const_iterator
 BlockMatrixBase<MatrixType>::begin(const size_type r) const
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   return const_iterator(this, r);
 }
 
@@ -2527,7 +2553,7 @@ template <class MatrixType>
 inline typename BlockMatrixBase<MatrixType>::const_iterator
 BlockMatrixBase<MatrixType>::end(const size_type r) const
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   return const_iterator(this, r + 1);
 }
 
@@ -2555,7 +2581,7 @@ template <class MatrixType>
 inline typename BlockMatrixBase<MatrixType>::iterator
 BlockMatrixBase<MatrixType>::begin(const size_type r)
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   return iterator(this, r);
 }
 
@@ -2565,7 +2591,7 @@ template <class MatrixType>
 inline typename BlockMatrixBase<MatrixType>::iterator
 BlockMatrixBase<MatrixType>::end(const size_type r)
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   return iterator(this, r + 1);
 }
 
