@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2018 by the deal.II authors
+// Copyright (C) 2016 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 #define dealii_diagonal_matrix_h
 
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/lac/vector.h>
 #include <deal.II/lac/vector_operation.h>
 
@@ -52,10 +54,18 @@ public:
   using size_type  = typename VectorType::size_type;
 
   /**
-   * Constructor.
+   * Default constructor. The object needs still to be reinitialized to be
+   * usable.
    */
   DiagonalMatrix() = default;
 
+  /**
+   * Constructor initializing this object as a diagonal matrix of size `n x n`
+   * where `n` is the size of the vector, and with diagonal entries equal to the
+   * elements of @p vec.
+   */
+  explicit DiagonalMatrix(const VectorType &vec);
+
   /**
    * Initialize with a given vector by copying the content of the vector
    * @p vec.
@@ -215,6 +225,13 @@ private:
 
 #ifndef DOXYGEN
 
+template <typename VectorType>
+DiagonalMatrix<VectorType>::DiagonalMatrix(const VectorType &vec)
+  : diagonal(vec)
+{}
+
+
+
 template <typename VectorType>
 void
 DiagonalMatrix<VectorType>::clear()
