@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2019 by the deal.II authors
+// Copyright (C) 2000 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -85,10 +85,6 @@ public:
              VectorMemory<VectorType> &mem,
              const AdditionalData &    data = AdditionalData());
 
-  /**
-   * Virtual destructor.
-   */
-  virtual ~EigenPower();
 
   /**
    * Power method. @p x is the (not necessarily normalized, but nonzero) start
@@ -176,12 +172,6 @@ public:
                VectorMemory<VectorType> &mem,
                const AdditionalData &    data = AdditionalData());
 
-
-  /**
-   * Virtual destructor.
-   */
-  virtual ~EigenInverse();
-
   /**
    * Inverse method. @p value is the start guess for the eigenvalue and @p x
    * is the (not necessarily normalized, but nonzero) start vector for the
@@ -214,12 +204,6 @@ EigenPower<VectorType>::EigenPower(SolverControl &           cn,
 
 
 
-template <class VectorType>
-EigenPower<VectorType>::~EigenPower()
-{}
-
-
-
 template <class VectorType>
 template <typename MatrixType>
 void
@@ -303,12 +287,6 @@ EigenInverse<VectorType>::EigenInverse(SolverControl &           cn,
 
 
 
-template <class VectorType>
-EigenInverse<VectorType>::~EigenInverse()
-{}
-
-
-
 template <class VectorType>
 template <typename MatrixType>
 void
