@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2018 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -22,6 +22,7 @@
 #include <deal.II/lac/la_parallel_block_vector.h>
 #include <deal.II/lac/lapack_support.h>
 #include <deal.II/lac/petsc_block_vector.h>
+#include <deal.II/lac/read_write_vector.h>
 #include <deal.II/lac/trilinos_parallel_block_vector.h>
 #include <deal.II/lac/vector.h>
 
@@ -31,8 +32,10 @@ DEAL_II_NAMESPACE_OPEN
 // Forward type declaration to have special treatment of
 // LAPACKFullMatrix<number>
 // in multivector_inner_product()
+#ifndef DOXYGEN
 template <typename Number>
 class LAPACKFullMatrix;
+#endif
 
 namespace LinearAlgebra
 {
@@ -210,6 +213,38 @@ namespace LinearAlgebra
 
 #ifdef DEAL_II_WITH_PETSC
 
+    namespace petsc_helpers
+    {
+      template <typename PETSC_Number, typename Number>
+      void
+      copy_petsc_vector(const PETSC_Number *petsc_start_ptr,
+                        const PETSC_Number *petsc_end_ptr,
+                        Number *            ptr)
+      {
+        std::copy(petsc_start_ptr, petsc_end_ptr, ptr);
+      }
+
+      template <typename PETSC_Number, typename Number>
+      void
+      copy_petsc_vector(const std::complex<PETSC_Number> *petsc_start_ptr,
+                        const std::complex<PETSC_Number> *petsc_end_ptr,
+                        std::complex<Number> *            ptr)
+      {
+        std::copy(petsc_start_ptr, petsc_end_ptr, ptr);
+      }
+
+      template <typename PETSC_Number, typename Number>
+      void
+      copy_petsc_vector(const std::complex<PETSC_Number> * /*petsc_start_ptr*/,
+                        const std::complex<PETSC_Number> * /*petsc_end_ptr*/,
+                        Number * /*ptr*/)
+      {
+        AssertThrow(false, ExcMessage("Tried to copy complex -> real"));
+      }
+    } // namespace petsc_helpers
+
+
+
     template <typename Number>
     BlockVector<Number> &
     BlockVector<Number>::
@@ -217,7 +252,38 @@ namespace LinearAlgebra
     {
       AssertDimension(this->n_blocks(), petsc_vec.n_blocks());
       for (unsigned int i = 0; i < this->n_blocks(); ++i)
-        this->block(i) = petsc_vec.block(i);
+        {
+          // We would like to use the same compact infrastructure as for the
+          // Trilinos vector below, but the interface through ReadWriteVector
+          // does not support overlapping (ghosted) PETSc vectors, which we need
+          // for backward compatibility.
+
+          Assert(petsc_vec.block(i).locally_owned_elements() ==
+                   this->block(i).locally_owned_elements(),
+                 StandardExceptions::ExcInvalidState());
+
+          // get a representation of the vector and copy it
+          PetscScalar *  start_ptr;
+          PetscErrorCode ierr =
+            VecGetArray(static_cast<const Vec &>(petsc_vec.block(i)),
+                        &start_ptr);
+          AssertThrow(ierr == 0, ExcPETScError(ierr));
+
+          const size_type vec_size = this->block(i).local_size();
+          petsc_helpers::copy_petsc_vector(start_ptr,
+                                           start_ptr + vec_size,
+                                           this->block(i).begin());
+
+          // restore the representation of the vector
+          ierr = VecRestoreArray(static_cast<const Vec &>(petsc_vec.block(i)),
+                                 &start_ptr);
+          AssertThrow(ierr == 0, ExcPETScError(ierr));
+
+          // spread ghost values between processes?
+          if (this->block(i).vector_is_ghosted ||
+              petsc_vec.block(i).has_ghost_elements())
+            this->block(i).update_ghost_values();
+        }
 
       return *this;
     }
@@ -235,7 +301,18 @@ namespace LinearAlgebra
     {
       AssertDimension(this->n_blocks(), trilinos_vec.n_blocks());
       for (unsigned int i = 0; i < this->n_blocks(); ++i)
-        this->block(i) = trilinos_vec.block(i);
+        {
+          const auto &partitioner  = this->block(i).get_partitioner();
+          IndexSet    combined_set = partitioner->locally_owned_range();
+          combined_set.add_indices(partitioner->ghost_indices());
+          ReadWriteVector<Number> rw_vector(combined_set);
+          rw_vector.import(trilinos_vec.block(i), VectorOperation::insert);
+          this->block(i).import(rw_vector, VectorOperation::insert);
+
+          if (this->block(i).has_ghost_elements() ||
+              trilinos_vec.block(i).has_ghost_elements())
+            this->block(i).update_ghost_values();
+        }
 
       return *this;
     }
@@ -260,10 +337,11 @@ namespace LinearAlgebra
           // start all requests for all blocks before finishing the transfers as
           // this saves repeated synchronizations. In order to avoid conflict
           // with possible other ongoing communication requests (from
-          // LA::distributed::Vector that supports unfinished requests), add an
-          // arbitrary number 8273 to the communication tag
+          // LA::distributed::Vector that supports unfinished requests), add
+          // 100 to the communication tag (the first 100 can be used by normal
+          // vectors).
           for (unsigned int block = start; block < end; ++block)
-            this->block(block).compress_start(block + 8273 - start, operation);
+            this->block(block).compress_start(block - start + 100, operation);
           for (unsigned int block = start; block < end; ++block)
             this->block(block).compress_finish(operation);
         }
@@ -286,10 +364,10 @@ namespace LinearAlgebra
 
           // In order to avoid conflict with possible other ongoing
           // communication requests (from LA::distributed::Vector that supports
-          // unfinished requests), add an arbitrary number 9923 to the
-          // communication tag
+          // unfinished requests), add 100 to the communication tag (the first
+          // 100 can be used by normal vectors)
           for (unsigned int block = start; block < end; ++block)
-            this->block(block).update_ghost_values_start(block - start + 9923);
+            this->block(block).update_ghost_values_start(block - start + 100);
           for (unsigned int block = start; block < end; ++block)
             this->block(block).update_ghost_values_finish();
         }
@@ -387,21 +465,6 @@ namespace LinearAlgebra
 
 
 
-    template <typename Number>
-    void
-    BlockVector<Number>::equ(const Number               a,
-                             const BlockVector<Number> &v,
-                             const Number               b,
-                             const BlockVector<Number> &w)
-    {
-      AssertDimension(this->n_blocks(), v.n_blocks());
-      AssertDimension(this->n_blocks(), w.n_blocks());
-      for (unsigned int block = 0; block < this->n_blocks(); ++block)
-        this->block(block).equ(a, v.block(block), b, w.block(block));
-    }
-
-
-
     template <typename Number>
     BlockVector<Number> &
     BlockVector<Number>::operator+=(const VectorSpaceVector<Number> &vv)
@@ -519,22 +582,6 @@ namespace LinearAlgebra
 
 
 
-    template <typename Number>
-    void
-    BlockVector<Number>::sadd(const Number               x,
-                              const Number               a,
-                              const BlockVector<Number> &v,
-                              const Number               b,
-                              const BlockVector<Number> &w)
-    {
-      AssertDimension(this->n_blocks(), v.n_blocks());
-      AssertDimension(this->n_blocks(), w.n_blocks());
-      for (unsigned int block = 0; block < this->n_blocks(); ++block)
-        this->block(block).sadd(x, a, v.block(block), b, w.block(block));
-    }
-
-
-
     template <typename Number>
     template <typename OtherNumber>
     void
@@ -577,7 +624,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return -Utilities::MPI::max(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
@@ -603,7 +650,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::sum(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
@@ -624,7 +671,8 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::sum(
-                 local_result, this->block(0).partitioner->get_communicator()) /
+                 local_result,
+                 this->block(0).partitioner->get_mpi_communicator()) /
                static_cast<real_type>(this->size());
       else
         return local_result / static_cast<real_type>(this->size());
@@ -644,7 +692,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::sum(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
@@ -663,7 +711,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::sum(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
@@ -690,10 +738,10 @@ namespace LinearAlgebra
         local_result += std::pow(this->block(i).lp_norm_local(p), p);
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
-        return std::pow(
-          Utilities::MPI::sum(local_result,
-                              this->block(0).partitioner->get_communicator()),
-          static_cast<real_type>(1.0 / p));
+        return std::pow(Utilities::MPI::sum(
+                          local_result,
+                          this->block(0).partitioner->get_mpi_communicator()),
+                        static_cast<real_type>(1.0 / p));
       else
         return std::pow(local_result, static_cast<real_type>(1.0 / p));
     }
@@ -713,7 +761,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::max(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
@@ -747,7 +795,7 @@ namespace LinearAlgebra
 
       if (this->block(0).partitioner->n_mpi_processes() > 1)
         return Utilities::MPI::sum(
-          local_result, this->block(0).partitioner->get_communicator());
+          local_result, this->block(0).partitioner->get_mpi_communicator());
       else
         return local_result;
     }
