@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2007 - 2018 by the deal.II authors
+// Copyright (C) 2007 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -34,8 +34,11 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 template <typename MatrixType>
 class MatrixBlock;
+#endif
 
 namespace internal
 {
@@ -127,7 +130,13 @@ public:
   /**
    * Copy constructor.
    */
-  MatrixBlock(const MatrixBlock<MatrixType> &M);
+  MatrixBlock(const MatrixBlock<MatrixType> &M) = default;
+
+  /**
+   * Assignment operator.
+   */
+  MatrixBlock<MatrixType> &
+  operator=(const MatrixBlock<MatrixType> &) = default;
 
   /**
    * Constructor setting block coordinates, but not initializing the matrix.
@@ -635,17 +644,6 @@ inline MatrixBlock<MatrixType>::MatrixBlock()
 {}
 
 
-template <typename MatrixType>
-inline MatrixBlock<MatrixType>::MatrixBlock(const MatrixBlock<MatrixType> &M)
-  : Subscriptor()
-  , row(M.row)
-  , column(M.column)
-  , matrix(M.matrix)
-  , row_indices(M.row_indices)
-  , column_indices(M.column_indices)
-{}
-
-
 template <typename MatrixType>
 inline MatrixBlock<MatrixType>::MatrixBlock(size_type i, size_type j)
   : row(i)
