@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2017 by the deal.II authors
+// Copyright (C) 2011 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -17,8 +17,12 @@
 #define dealii_parallel_vector_h
 
 #include <deal.II/base/config.h>
+
 #include <deal.II/lac/la_parallel_vector.h>
 
+DEAL_II_WARNING(
+  "This file is deprecated. Use <deal.II/lac/la_parallel_vector.h> and LinearAlgebra::distributed::Vector instead.")
+
 #include <cstring>
 #include <iomanip>
 
@@ -136,12 +140,16 @@ namespace parallel
      * not rely on this class to automatically detect the unsupported case.
      *
      * @author Katharina Kormann, Martin Kronbichler, 2010, 2011
+     *
+     * @deprecated Use LinearAlgebra::distributed::Vector instead.
      */
-    using LinearAlgebra::distributed::Vector;
+    template <typename Number>
+    using Vector DEAL_II_DEPRECATED =
+      LinearAlgebra::distributed::Vector<Number>;
 
     /*@}*/
-  }
-}
+  } // namespace distributed
+} // namespace parallel
 
 DEAL_II_NAMESPACE_CLOSE
 
