@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2012 - 2017 by the deal.II authors
+// Copyright (C) 2012 - 2018 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,21 +8,21 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
 #ifndef dealii_petsc_matrix_free_h
-#define dealii_petsc_matrix_free_h
+#  define dealii_petsc_matrix_free_h
 
 
-#include <deal.II/base/config.h>
+#  include <deal.II/base/config.h>
 
-#ifdef DEAL_II_WITH_PETSC
-#  include <deal.II/lac/exceptions.h>
-#  include <deal.II/lac/petsc_matrix_base.h>
-#  include <deal.II/lac/petsc_parallel_vector.h>
+#  ifdef DEAL_II_WITH_PETSC
+#    include <deal.II/lac/exceptions.h>
+#    include <deal.II/lac/petsc_matrix_base.h>
+#    include <deal.II/lac/petsc_vector.h>
 DEAL_II_NAMESPACE_OPEN
 
 
@@ -60,11 +60,10 @@ namespace PETScWrappers
   class MatrixFree : public MatrixBase
   {
   public:
-
     /**
      * Default constructor. Create an empty matrix object.
      */
-    MatrixFree ();
+    MatrixFree();
 
     /**
      * Create a matrix object of dimensions @p m times @p n with communication
@@ -80,11 +79,11 @@ namespace PETScWrappers
      * any estimation of non_zero entries and has no option
      * <tt>is_symmetric</tt>.
      */
-    MatrixFree (const MPI_Comm     &communicator,
-                const unsigned int  m,
-                const unsigned int  n,
-                const unsigned int  local_rows,
-                const unsigned int  local_columns);
+    MatrixFree(const MPI_Comm &   communicator,
+               const unsigned int m,
+               const unsigned int n,
+               const unsigned int local_rows,
+               const unsigned int local_columns);
 
     /**
      * Create a matrix object of dimensions @p m times @p n with communication
@@ -97,87 +96,93 @@ namespace PETScWrappers
      * any estimation of non_zero entries and has no option
      * <tt>is_symmetric</tt>.
      */
-    MatrixFree (const MPI_Comm     &communicator,
-                const unsigned int  m,
-                const unsigned int  n,
-                const std::vector<unsigned int> &local_rows_per_process,
-                const std::vector<unsigned int> &local_columns_per_process,
-                const unsigned int  this_process);
+    MatrixFree(const MPI_Comm &                 communicator,
+               const unsigned int               m,
+               const unsigned int               n,
+               const std::vector<unsigned int> &local_rows_per_process,
+               const std::vector<unsigned int> &local_columns_per_process,
+               const unsigned int               this_process);
 
     /**
      * Constructor for the serial case: Same function as
      * <tt>MatrixFree()</tt>, see above, with <tt>communicator =
      * MPI_COMM_WORLD</tt>.
      */
-    MatrixFree (const unsigned int  m,
-                const unsigned int  n,
-                const unsigned int  local_rows,
-                const unsigned int  local_columns);
+    MatrixFree(const unsigned int m,
+               const unsigned int n,
+               const unsigned int local_rows,
+               const unsigned int local_columns);
 
     /**
      * Constructor for the serial case: Same function as
      * <tt>MatrixFree()</tt>, see above, with <tt>communicator =
      * MPI_COMM_WORLD</tt>.
      */
-    MatrixFree (const unsigned int  m,
-                const unsigned int  n,
-                const std::vector<unsigned int> &local_rows_per_process,
-                const std::vector<unsigned int> &local_columns_per_process,
-                const unsigned int  this_process);
+    MatrixFree(const unsigned int               m,
+               const unsigned int               n,
+               const std::vector<unsigned int> &local_rows_per_process,
+               const std::vector<unsigned int> &local_columns_per_process,
+               const unsigned int               this_process);
 
     /**
      * Throw away the present matrix and generate one that has the same
      * properties as if it were created by the constructor of this class with
      * the same argument list as the present function.
      */
-    void reinit (const MPI_Comm     &communicator,
-                 const unsigned int  m,
-                 const unsigned int  n,
-                 const unsigned int  local_rows,
-                 const unsigned int  local_columns);
+    void
+    reinit(const MPI_Comm &   communicator,
+           const unsigned int m,
+           const unsigned int n,
+           const unsigned int local_rows,
+           const unsigned int local_columns);
 
     /**
      * Throw away the present matrix and generate one that has the same
      * properties as if it were created by the constructor of this class with
      * the same argument list as the present function.
      */
-    void reinit (const MPI_Comm     &communicator,
-                 const unsigned int  m,
-                 const unsigned int  n,
-                 const std::vector<unsigned int> &local_rows_per_process,
-                 const std::vector<unsigned int> &local_columns_per_process,
-                 const unsigned int  this_process);
+    void
+    reinit(const MPI_Comm &                 communicator,
+           const unsigned int               m,
+           const unsigned int               n,
+           const std::vector<unsigned int> &local_rows_per_process,
+           const std::vector<unsigned int> &local_columns_per_process,
+           const unsigned int               this_process);
 
     /**
      * Call the @p reinit() function above with <tt>communicator =
      * MPI_COMM_WORLD</tt>.
      */
-    void reinit (const unsigned int  m,
-                 const unsigned int  n,
-                 const unsigned int  local_rows,
-                 const unsigned int  local_columns);
+    void
+    reinit(const unsigned int m,
+           const unsigned int n,
+           const unsigned int local_rows,
+           const unsigned int local_columns);
 
     /**
      * Call the @p reinit() function above with <tt>communicator =
      * MPI_COMM_WORLD</tt>.
      */
-    void reinit (const unsigned int  m,
-                 const unsigned int  n,
-                 const std::vector<unsigned int> &local_rows_per_process,
-                 const std::vector<unsigned int> &local_columns_per_process,
-                 const unsigned int  this_process);
+    void
+    reinit(const unsigned int               m,
+           const unsigned int               n,
+           const std::vector<unsigned int> &local_rows_per_process,
+           const std::vector<unsigned int> &local_columns_per_process,
+           const unsigned int               this_process);
 
     /**
      * Release all memory and return to a state just like after having called
      * the default constructor.
      */
-    void clear ();
+    void
+    clear();
 
     /**
      * Return a reference to the MPI communicator object in use with this
      * matrix.
      */
-    const MPI_Comm &get_mpi_communicator () const;
+    const MPI_Comm &
+    get_mpi_communicator() const override;
 
     /**
      * Matrix-vector multiplication: let <i>dst = M*src</i> with <i>M</i>
@@ -190,9 +195,8 @@ namespace PETScWrappers
      * have to be distributed vectors as well. Conversely, if the matrix is
      * not distributed, then neither of the vectors may be.
      */
-    virtual
-    void vmult (VectorBase       &dst,
-                const VectorBase &src) const = 0;
+    virtual void
+    vmult(VectorBase &dst, const VectorBase &src) const = 0;
 
     /**
      * Matrix-vector multiplication: let <i>dst = M<sup>T</sup>*src</i> with
@@ -206,9 +210,8 @@ namespace PETScWrappers
      * Conversely, if the matrix is not distributed, then neither of the
      * vectors may be.
      */
-    virtual
-    void Tvmult (VectorBase       &dst,
-                 const VectorBase &src) const = 0;
+    virtual void
+    Tvmult(VectorBase &dst, const VectorBase &src) const = 0;
 
     /**
      * Adding Matrix-vector multiplication. Add <i>M*src</i> on <i>dst</i>
@@ -221,9 +224,8 @@ namespace PETScWrappers
      * Conversely, if the matrix is not distributed, then neither of the
      * vectors may be.
      */
-    virtual
-    void vmult_add (VectorBase       &dst,
-                    const VectorBase &src) const = 0;
+    virtual void
+    vmult_add(VectorBase &dst, const VectorBase &src) const = 0;
 
     /**
      * Adding Matrix-vector multiplication. Add <i>M<sup>T</sup>*src</i> to
@@ -237,9 +239,8 @@ namespace PETScWrappers
      * Conversely, if the matrix is not distributed, then neither of the
      * vectors may be.
      */
-    virtual
-    void Tvmult_add (VectorBase       &dst,
-                     const VectorBase &src) const = 0;
+    virtual void
+    Tvmult_add(VectorBase &dst, const VectorBase &src) const = 0;
 
     /**
      * The matrix-vector multiplication called by @p matrix_free_mult(). This
@@ -249,16 +250,15 @@ namespace PETScWrappers
      * VectorBase &src)</tt> which is purely virtual and must be reimplemented
      * in derived classes.
      */
-    virtual
-    void vmult (Vec  &dst, const Vec  &src) const;
+    virtual void
+    vmult(Vec &dst, const Vec &src) const;
 
   private:
-
     /**
      * Copy of the communicator object to be used for this parallel matrix-
      * free object.
      */
-    MPI_Comm  communicator;
+    MPI_Comm communicator;
 
     /**
      * Callback-function registered as the matrix-vector multiplication of
@@ -271,37 +271,35 @@ namespace PETScWrappers
      * This function calls <tt>vmult(Vec &dst, const Vec &src)</tt> which
      * should be reimplemented in derived classes.
      */
-    static int matrix_free_mult (Mat  A, Vec  src, Vec  dst);
+    static int
+    matrix_free_mult(Mat A, Vec src, Vec dst);
 
     /**
      * Do the actual work for the respective @p reinit() function and the
      * matching constructor, i.e. create a matrix object. Getting rid of the
      * previous matrix is left to the caller.
      */
-    void do_reinit (const unsigned int  m,
-                    const unsigned int  n,
-                    const unsigned int  local_rows,
-                    const unsigned int  local_columns);
+    void
+    do_reinit(const unsigned int m,
+              const unsigned int n,
+              const unsigned int local_rows,
+              const unsigned int local_columns);
   };
 
 
 
-// -------- template and inline functions ----------
+  // -------- template and inline functions ----------
 
-  inline
-  const MPI_Comm &
-  MatrixFree::get_mpi_communicator () const
+  inline const MPI_Comm &
+  MatrixFree::get_mpi_communicator() const
   {
     return communicator;
   }
-}
+} // namespace PETScWrappers
 
 DEAL_II_NAMESPACE_CLOSE
 
-#endif // DEAL_II_WITH_PETSC
-
-
-/*----------------------------   petsc_matrix_free.h     ---------------------------*/
+#  endif // DEAL_II_WITH_PETSC
 
 #endif
-/*----------------------------   petsc_matrix_free.h     ---------------------------*/
+/*---------------------------- petsc_matrix_free.h --------------------------*/
