@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2018 - 2019 by the deal.II authors
+// Copyright (C) 2018 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -16,6 +16,8 @@
 #ifndef dealii_qr_h
 #define dealii_qr_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/std_cxx14/memory.h>
 
@@ -44,7 +46,7 @@ class BaseQR
   /**
    * Number type for R matrix.
    */
-  typedef typename VectorType::value_type Number;
+  using Number = typename VectorType::value_type;
 
 protected:
   /**
@@ -241,7 +243,7 @@ public:
   /**
    * Number type for R matrix.
    */
-  typedef typename VectorType::value_type Number;
+  using Number = typename VectorType::value_type;
 
   /**
    * Default constructor.
@@ -350,7 +352,7 @@ public:
   /**
    * Number type for R matrix.
    */
-  typedef typename VectorType::value_type Number;
+  using Number = typename VectorType::value_type;
 
   /**
    * Default constructor.
@@ -613,8 +615,8 @@ void
 ImplicitQR<VectorType>::apply_givens_rotation(const unsigned int i,
                                               const unsigned int k)
 {
-  Assert(i < k, ExcIndexRange(i, 0, k));
-  Assert(k < this->current_size, ExcIndexRange(k, 0, this->current_size));
+  AssertIndexRange(i, k);
+  AssertIndexRange(k, this->current_size);
   const std::array<Number, 3> csr =
     dealii::Utilities::LinearAlgebra::givens_rotation<Number>(this->R(i, k),
                                                               this->R(k, k));
@@ -749,8 +751,8 @@ void
 QR<VectorType>::apply_givens_rotation(const unsigned int i,
                                       const unsigned int k)
 {
-  Assert(i < k, ExcIndexRange(i, 0, k));
-  Assert(k < this->current_size, ExcIndexRange(k, 0, this->current_size));
+  AssertIndexRange(i, k);
+  AssertIndexRange(k, this->current_size);
   const std::array<Number, 3> csr =
     dealii::Utilities::LinearAlgebra::givens_rotation<Number>(this->R(i, k),
                                                               this->R(k, k));
@@ -786,7 +788,7 @@ template <typename VectorType>
 void
 QR<VectorType>::remove_column(const unsigned int k)
 {
-  Assert(k < this->current_size, ExcIndexRange(k, 0, this->current_size));
+  AssertIndexRange(k, this->current_size);
   Assert(this->current_size > 0,
          ExcMessage("Can not remove a column if QR is empty"));
   // apply a sequence of Givens rotations
