@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2009 - 2018 by the deal.II authors
+// Copyright (C) 2009 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -33,11 +33,14 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declaration
+#    ifndef DOXYGEN
 namespace PETScWrappers
 {
   // forward declarations
   class SolverBase;
 } // namespace PETScWrappers
+#    endif
 
 namespace SLEPcWrappers
 {
@@ -106,10 +109,8 @@ namespace SLEPcWrappers
      */
     ST st;
 
-    /**
-     * Make the solver class a friend, since it needs to set spectral
-     * transformation object.
-     */
+    // Make the solver class a friend, since it needs to set spectral
+    // transformation object.
     friend class SolverBase;
   };
 
@@ -192,10 +193,8 @@ namespace SLEPcWrappers
      */
     const AdditionalData additional_data;
 
-    /**
-     * Make the solver class a friend, since it may need to set target
-     * equal the provided shift value.
-     */
+    // Make the solver class a friend, since it may need to set target
+    // equal the provided shift value.
     friend class SolverBase;
   };
 
