@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2018 by the deal.II authors
+// Copyright (C) 1998 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -8,8 +8,8 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
@@ -18,12 +18,15 @@
 
 
 #include <deal.II/base/config.h>
+
 #include <deal.II/base/logstream.h>
+#include <deal.II/base/signaling_nan.h>
+#include <deal.II/base/subscriptor.h>
+
 #include <deal.II/lac/solver.h>
 #include <deal.II/lac/solver_control.h>
+
 #include <cmath>
-#include <deal.II/base/subscriptor.h>
-#include <deal.II/base/signaling_nan.h>
 
 DEAL_II_NAMESPACE_OPEN
 
@@ -76,7 +79,7 @@ namespace internal
      */
     SolverBicgstabData();
   };
-}
+} // namespace internal
 
 /**
  * Bicgstab algorithm by van der Vorst.
@@ -117,9 +120,9 @@ namespace internal
  * to observe the progress of the iteration.
  *
  */
-template <typename VectorType = Vector<double> >
-class SolverBicgstab : public Solver<VectorType>,
-  protected internal::SolverBicgstabData
+template <typename VectorType = Vector<double>>
+class SolverBicgstab : public SolverBase<VectorType>,
+                       protected internal::SolverBicgstabData
 {
 public:
   /**
@@ -140,11 +143,10 @@ public:
      * The default is to perform an exact residual computation and breakdown
      * parameter 1e-10.
      */
-    explicit
-    AdditionalData(const bool   exact_residual = true,
-                   const double breakdown      = 1.e-10)
-      : exact_residual(exact_residual),
-        breakdown(breakdown)
+    explicit AdditionalData(const bool   exact_residual = true,
+                            const double breakdown      = 1.e-10)
+      : exact_residual(exact_residual)
+      , breakdown(breakdown)
     {}
     /**
      * Flag for exact computation of residual.
@@ -159,31 +161,31 @@ public:
   /**
    * Constructor.
    */
-  SolverBicgstab (SolverControl            &cn,
-                  VectorMemory<VectorType> &mem,
-                  const AdditionalData     &data=AdditionalData());
+  SolverBicgstab(SolverControl &           cn,
+                 VectorMemory<VectorType> &mem,
+                 const AdditionalData &    data = AdditionalData());
 
   /**
    * Constructor. Use an object of type GrowingVectorMemory as a default to
    * allocate memory.
    */
-  SolverBicgstab (SolverControl        &cn,
-                  const AdditionalData &data=AdditionalData());
+  SolverBicgstab(SolverControl &       cn,
+                 const AdditionalData &data = AdditionalData());
 
   /**
    * Virtual destructor.
    */
-  virtual ~SolverBicgstab ();
+  virtual ~SolverBicgstab() override = default;
 
   /**
    * Solve primal problem only.
    */
   template <typename MatrixType, typename PreconditionerType>
   void
-  solve (const MatrixType         &A,
-         VectorType               &x,
-         const VectorType         &b,
-         const PreconditionerType &preconditioner);
+  solve(const MatrixType &        A,
+        VectorType &              x,
+        const VectorType &        b,
+        const PreconditionerType &preconditioner);
 
 protected:
   /**
@@ -235,17 +237,19 @@ protected:
    * Computation of the stopping criterion.
    */
   template <typename MatrixType>
-  double criterion (const MatrixType &A, const VectorType &x, const VectorType &b);
+  double
+  criterion(const MatrixType &A, const VectorType &x, const VectorType &b);
 
   /**
    * Interface for derived class.  This function gets the current iteration
    * vector, the residual and the update vector in each step. It can be used
    * for graphical output of the convergence history.
    */
-  virtual void print_vectors(const unsigned int step,
-                             const VectorType   &x,
-                             const VectorType   &r,
-                             const VectorType   &d) const;
+  virtual void
+  print_vectors(const unsigned int step,
+                const VectorType & x,
+                const VectorType & r,
+                const VectorType & d) const;
 
   /**
    * Additional parameters.
@@ -257,7 +261,8 @@ private:
    * Everything before the iteration loop.
    */
   template <typename MatrixType>
-  SolverControl::State start(const MatrixType &A);
+  SolverControl::State
+  start(const MatrixType &A);
 
   /**
    * A structure returned by the iterate() function representing what it found
@@ -270,10 +275,10 @@ private:
     unsigned int         last_step;
     double               last_residual;
 
-    IterationResult (const bool breakdown,
-                     const SolverControl::State state,
-                     const unsigned int         last_step,
-                     const double               last_residual);
+    IterationResult(const bool                 breakdown,
+                    const SolverControl::State state,
+                    const unsigned int         last_step,
+                    const double               last_residual);
   };
 
   /**
@@ -282,8 +287,7 @@ private:
    */
   template <typename MatrixType, typename PreconditionerType>
   IterationResult
-  iterate(const MatrixType         &A,
-          const PreconditionerType &preconditioner);
+  iterate(const MatrixType &A, const PreconditionerType &preconditioner);
 };
 
 /*@}*/
@@ -293,47 +297,38 @@ private:
 
 
 template <typename VectorType>
-SolverBicgstab<VectorType>::IterationResult::IterationResult
-(const bool                 breakdown,
- const SolverControl::State state,
- const unsigned int         last_step,
- const double               last_residual)
-  :
-  breakdown (breakdown),
-  state (state),
-  last_step (last_step),
-  last_residual (last_residual)
+SolverBicgstab<VectorType>::IterationResult::IterationResult(
+  const bool                 breakdown,
+  const SolverControl::State state,
+  const unsigned int         last_step,
+  const double               last_residual)
+  : breakdown(breakdown)
+  , state(state)
+  , last_step(last_step)
+  , last_residual(last_residual)
 {}
 
 
 
 template <typename VectorType>
-SolverBicgstab<VectorType>::SolverBicgstab (SolverControl            &cn,
-                                            VectorMemory<VectorType> &mem,
-                                            const AdditionalData     &data)
-  :
-  Solver<VectorType>(cn,mem),
-  Vx (nullptr),
-  Vb (nullptr),
-  additional_data(data)
+SolverBicgstab<VectorType>::SolverBicgstab(SolverControl &           cn,
+                                           VectorMemory<VectorType> &mem,
+                                           const AdditionalData &    data)
+  : SolverBase<VectorType>(cn, mem)
+  , Vx(nullptr)
+  , Vb(nullptr)
+  , additional_data(data)
 {}
 
 
 
 template <typename VectorType>
-SolverBicgstab<VectorType>::SolverBicgstab (SolverControl        &cn,
-                                            const AdditionalData &data)
-  :
-  Solver<VectorType>(cn),
-  Vx (nullptr),
-  Vb (nullptr),
-  additional_data(data)
-{}
-
-
-
-template <typename VectorType>
-SolverBicgstab<VectorType>::~SolverBicgstab ()
+SolverBicgstab<VectorType>::SolverBicgstab(SolverControl &       cn,
+                                           const AdditionalData &data)
+  : SolverBase<VectorType>(cn)
+  , Vx(nullptr)
+  , Vb(nullptr)
+  , additional_data(data)
 {}
 
 
@@ -341,12 +336,12 @@ SolverBicgstab<VectorType>::~SolverBicgstab ()
 template <typename VectorType>
 template <typename MatrixType>
 double
-SolverBicgstab<VectorType>::criterion (const MatrixType &A,
-                                       const VectorType &x,
-                                       const VectorType &b)
+SolverBicgstab<VectorType>::criterion(const MatrixType &A,
+                                      const VectorType &x,
+                                      const VectorType &b)
 {
   A.vmult(*Vt, x);
-  Vt->add(-1.,b);
+  Vt->add(-1., b);
   res = Vt->l2_norm();
 
   return res;
@@ -354,13 +349,13 @@ SolverBicgstab<VectorType>::criterion (const MatrixType &A,
 
 
 
-template <typename VectorType >
+template <typename VectorType>
 template <typename MatrixType>
 SolverControl::State
-SolverBicgstab<VectorType>::start (const MatrixType &A)
+SolverBicgstab<VectorType>::start(const MatrixType &A)
 {
   A.vmult(*Vr, *Vx);
-  Vr->sadd(-1.,1.,*Vb);
+  Vr->sadd(-1., 1., *Vb);
   res = Vr->l2_norm();
 
   return this->iteration_status(step, res, *Vx);
@@ -381,49 +376,50 @@ SolverBicgstab<VectorType>::print_vectors(const unsigned int,
 template <typename VectorType>
 template <typename MatrixType, typename PreconditionerType>
 typename SolverBicgstab<VectorType>::IterationResult
-SolverBicgstab<VectorType>::iterate(const MatrixType         &A,
+SolverBicgstab<VectorType>::iterate(const MatrixType &        A,
                                     const PreconditionerType &preconditioner)
 {
-//TODO:[GK] Implement "use the length of the computed orthogonal residual" in the BiCGStab method.
+  // TODO:[GK] Implement "use the length of the computed orthogonal residual" in
+  // the BiCGStab method.
   SolverControl::State state = SolverControl::iterate;
   alpha = omega = rho = 1.;
 
-  VectorType &r = *Vr;
+  VectorType &r    = *Vr;
   VectorType &rbar = *Vrbar;
-  VectorType &p = *Vp;
-  VectorType &y = *Vy;
-  VectorType &z = *Vz;
-  VectorType &t = *Vt;
-  VectorType &v = *Vv;
+  VectorType &p    = *Vp;
+  VectorType &y    = *Vy;
+  VectorType &z    = *Vz;
+  VectorType &t    = *Vt;
+  VectorType &v    = *Vv;
 
-  rbar = r;
+  rbar         = r;
   bool startup = true;
 
   do
     {
       ++step;
 
-      rhobar = r*rbar;
+      rhobar = r * rbar;
       beta   = rhobar * alpha / (rho * omega);
       rho    = rhobar;
       if (startup == true)
         {
-          p = r;
+          p       = r;
           startup = false;
         }
       else
         {
           p.sadd(beta, 1., r);
-          p.add(-beta*omega, v);
+          p.add(-beta * omega, v);
         }
 
-      preconditioner.vmult(y,p);
-      A.vmult(v,y);
+      preconditioner.vmult(y, p);
+      A.vmult(v, y);
       rhobar = rbar * v;
 
-      alpha = rho/rhobar;
+      alpha = rho / rhobar;
 
-//TODO:[?] Find better breakdown criterion
+      // TODO:[?] Find better breakdown criterion
 
       if (std::fabs(alpha) > 1.e10)
         return IterationResult(true, state, step, res);
@@ -433,9 +429,9 @@ SolverBicgstab<VectorType>::iterate(const MatrixType         &A,
       // check for early success, see the lac/bicgstab_early testcase as to
       // why this is necessary
       //
-      // note: the vector *Vx we pass to the iteration_status signal here is only
-      // the current approximation, not the one we will return with,
-      // which will be x=*Vx + alpha*y
+      // note: the vector *Vx we pass to the iteration_status signal here is
+      // only the current approximation, not the one we will return with, which
+      // will be x=*Vx + alpha*y
       if (this->iteration_status(step, res, *Vx) == SolverControl::success)
         {
           Vx->add(alpha, y);
@@ -443,10 +439,10 @@ SolverBicgstab<VectorType>::iterate(const MatrixType         &A,
           return IterationResult(false, SolverControl::success, step, res);
         }
 
-      preconditioner.vmult(z,r);
-      A.vmult(t,z);
-      rhobar = t*r;
-      omega = rhobar/(t*t);
+      preconditioner.vmult(z, r);
+      A.vmult(t, z);
+      rhobar = t * r;
+      omega  = rhobar / (t * t);
       Vx->add(alpha, y, omega, z);
 
       if (additional_data.exact_residual)
@@ -468,9 +464,9 @@ SolverBicgstab<VectorType>::iterate(const MatrixType         &A,
 template <typename VectorType>
 template <typename MatrixType, typename PreconditionerType>
 void
-SolverBicgstab<VectorType>::solve(const MatrixType         &A,
-                                  VectorType               &x,
-                                  const VectorType         &b,
+SolverBicgstab<VectorType>::solve(const MatrixType &        A,
+                                  VectorType &              x,
+                                  const VectorType &        b,
                                   const PreconditionerType &preconditioner)
 {
   LogStream::Prefix prefix("Bicgstab");
@@ -495,7 +491,7 @@ SolverBicgstab<VectorType>::solve(const MatrixType         &A,
 
   step = 0;
 
-  IterationResult state(false,SolverControl::failure,0,0);
+  IterationResult state(false, SolverControl::failure, 0, 0);
 
   // iterate while the inner iteration returns a breakdown
   do
@@ -514,8 +510,8 @@ SolverBicgstab<VectorType>::solve(const MatrixType         &A,
 
   // in case of failure: throw exception
   AssertThrow(state.state == SolverControl::success,
-              SolverControl::NoConvergence (state.last_step,
-                                            state.last_residual));
+              SolverControl::NoConvergence(state.last_step,
+                                           state.last_residual));
   // otherwise exit as normal
 }
 
