@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1998 - 2019 by the deal.II authors
+// Copyright (C) 1998 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -135,11 +135,6 @@ public:
   SolverFIRE(SolverControl &       solver_control,
              const AdditionalData &data = AdditionalData());
 
-  /**
-   * Virtual destructor.
-   */
-  virtual ~SolverFIRE();
-
   /**
    * Obtain a set of variables @p x that minimize an objective function
    * described by the polymorphic function wrapper @p compute, with a given
@@ -229,12 +224,6 @@ SolverFIRE<VectorType>::SolverFIRE(SolverControl &       solver_control,
 
 
 
-template <typename VectorType>
-SolverFIRE<VectorType>::~SolverFIRE()
-{}
-
-
-
 template <typename VectorType>
 template <typename PreconditionerType>
 void
