@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2002 - 2018 by the deal.II authors
+// Copyright (C) 2002 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -28,10 +28,13 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#  ifndef DOXYGEN
 template <typename number>
 class Vector;
 template <typename number>
 class FullMatrix;
+#  endif
 
 /**
  * @addtogroup Matrix1
@@ -232,9 +235,7 @@ public:
        */
       unsigned short a_index;
 
-      /**
-       * Make enclosing class a friend.
-       */
+      // Make enclosing class a friend.
       friend class const_iterator;
     };
 
@@ -285,14 +286,6 @@ public:
      * Store an object of the accessor class.
      */
     Accessor accessor;
-
-    /**
-     * Make the enclosing class a friend. This is only necessary since icc7
-     * otherwise wouldn't allow us to make const_iterator::Accessor a friend,
-     * stating that it can't access this class -- this is of course bogus,
-     * since granting friendship doesn't need access to the class being
-     * granted friendship...
-     */
   };
 
   /**
@@ -1111,8 +1104,8 @@ template <typename number>
 inline typename SparseMatrixEZ<number>::Entry *
 SparseMatrixEZ<number>::locate(const size_type row, const size_type col)
 {
-  Assert(row < m(), ExcIndexRange(row, 0, m()));
-  Assert(col < n(), ExcIndexRange(col, 0, n()));
+  AssertIndexRange(row, m());
+  AssertIndexRange(col, n());
 
   const RowInfo & r   = row_info[row];
   const size_type end = r.start + r.length;
@@ -1142,8 +1135,8 @@ template <typename number>
 inline typename SparseMatrixEZ<number>::Entry *
 SparseMatrixEZ<number>::allocate(const size_type row, const size_type col)
 {
-  Assert(row < m(), ExcIndexRange(row, 0, m()));
-  Assert(col < n(), ExcIndexRange(col, 0, n()));
+  AssertIndexRange(row, m());
+  AssertIndexRange(col, n());
 
   RowInfo &       r   = row_info[row];
   const size_type end = r.start + r.length;
@@ -1245,8 +1238,8 @@ SparseMatrixEZ<number>::set(const size_type i,
 {
   AssertIsFinite(value);
 
-  Assert(i < m(), ExcIndexRange(i, 0, m()));
-  Assert(j < n(), ExcIndexRange(j, 0, n()));
+  AssertIndexRange(i, m());
+  AssertIndexRange(j, n());
 
   if (elide_zero_values && value == 0.)
     {
@@ -1271,8 +1264,8 @@ SparseMatrixEZ<number>::add(const size_type i,
 {
   AssertIsFinite(value);
 
-  Assert(i < m(), ExcIndexRange(i, 0, m()));
-  Assert(j < n(), ExcIndexRange(j, 0, n()));
+  AssertIndexRange(i, m());
+  AssertIndexRange(j, n());
 
   // ignore zero additions
   if (std::abs(value) == 0.)
@@ -1393,7 +1386,7 @@ template <typename number>
 inline typename SparseMatrixEZ<number>::const_iterator
 SparseMatrixEZ<number>::begin(const size_type r) const
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   const_iterator result(this, r, 0);
   return result;
 }
@@ -1402,7 +1395,7 @@ template <typename number>
 inline typename SparseMatrixEZ<number>::const_iterator
 SparseMatrixEZ<number>::end(const size_type r) const
 {
-  Assert(r < m(), ExcIndexRange(r, 0, m()));
+  AssertIndexRange(r, m());
   const_iterator result(this, r + 1, 0);
   return result;
 }
