@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2000 - 2019 by the deal.II authors
+// Copyright (C) 2000 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -41,6 +41,8 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declarations
+#ifndef DOXYGEN
 class SparsityPattern;
 class SparsityPatternBase;
 class DynamicSparsityPattern;
@@ -58,7 +60,7 @@ namespace ChunkSparsityPatternIterators
 {
   class Accessor;
 }
-
+#endif
 
 /*! @addtogroup Sparsity
  *@{
@@ -243,14 +245,10 @@ namespace SparsityPatternIterators
     void
     advance();
 
-    /**
-     * Grant access to iterator class.
-     */
+    // Grant access to iterator class.
     friend class LinearIndexIterator<Iterator, Accessor>;
 
-    /**
-     * Grant access to accessor class of ChunkSparsityPattern.
-     */
+    // Grant access to accessor class of ChunkSparsityPattern.
     friend class ChunkSparsityPatternIterators::Accessor;
   };
 
@@ -682,7 +680,19 @@ public:
   void
   load(Archive &ar, const unsigned int version);
 
+#ifdef DOXYGEN
+  /**
+   * Write and read the data of this object from a stream for the purpose
+   * of serialization.
+   */
+  template <class Archive>
+  void
+  serialize(Archive &archive, const unsigned int version);
+#else
+  // This macro defines the serialize() method that is compatible with
+  // the templated save() and load() method that have been implemented.
   BOOST_SERIALIZATION_SPLIT_MEMBER()
+#endif
 
   // @}
 
@@ -803,9 +813,7 @@ protected:
    */
   bool compressed;
 
-  /**
-   * Make all sparse matrices friends of this class.
-   */
+  // Make all sparse matrices friends of this class.
   template <typename number>
   friend class SparseMatrix;
   template <typename number>
@@ -818,9 +826,7 @@ protected:
   friend class ChunkSparsityPattern;
   friend class DynamicSparsityPattern;
 
-  /**
-   * Also give access to internal details to the iterator/accessor classes.
-   */
+  // Also give access to internal details to the iterator/accessor classes.
   friend class SparsityPatternIterators::Iterator;
   friend class SparsityPatternIterators::Accessor;
   friend class ChunkSparsityPatternIterators::Accessor;
@@ -1263,7 +1269,19 @@ public:
   void
   load(Archive &ar, const unsigned int version);
 
+#ifdef DOXYGEN
+  /**
+   * Write and read the data of this object from a stream for the purpose
+   * of serialization.
+   */
+  template <class Archive>
+  void
+  serialize(Archive &archive, const unsigned int version);
+#else
+  // This macro defines the serialize() method that is compatible with
+  // the templated save() and load() method that have been implemented.
   BOOST_SERIALIZATION_SPLIT_MEMBER()
+#endif
 
   // @}
 
@@ -1293,9 +1311,7 @@ private:
    */
   bool store_diagonal_first_in_row;
 
-  /**
-   * Make all sparse matrices friends of this class.
-   */
+  // Make all sparse matrices friends of this class.
   template <typename number>
   friend class SparseMatrix;
   template <typename number>
@@ -1308,9 +1324,7 @@ private:
   friend class ChunkSparsityPattern;
   friend class DynamicSparsityPattern;
 
-  /**
-   * Also give access to internal details to the iterator/accessor classes.
-   */
+  // Also give access to internal details to the iterator/accessor classes.
   friend class SparsityPatternIterators::Iterator;
   friend class SparsityPatternIterators::Accessor;
   friend class ChunkSparsityPatternIterators::Accessor;
@@ -1473,7 +1487,7 @@ SparsityPatternBase::end() const
 inline SparsityPatternBase::iterator
 SparsityPatternBase::begin(const size_type r) const
 {
-  Assert(r < n_rows(), ExcIndexRangeType<size_type>(r, 0, n_rows()));
+  AssertIndexRange(r, n_rows());
 
   return {this, rowstart[r]};
 }
@@ -1483,7 +1497,7 @@ SparsityPatternBase::begin(const size_type r) const
 inline SparsityPatternBase::iterator
 SparsityPatternBase::end(const size_type r) const
 {
-  Assert(r < n_rows(), ExcIndexRangeType<size_type>(r, 0, n_rows()));
+  AssertIndexRange(r, n_rows());
 
   return {this, rowstart[r + 1]};
 }
@@ -1525,7 +1539,7 @@ SparsityPattern::stores_only_added_elements() const
 inline unsigned int
 SparsityPatternBase::row_length(const size_type row) const
 {
-  Assert(row < rows, ExcIndexRangeType<size_type>(row, 0, rows));
+  AssertIndexRange(row, rows);
   return rowstart[row + 1] - rowstart[row];
 }
 
@@ -1535,8 +1549,8 @@ inline SparsityPattern::size_type
 SparsityPatternBase::column_number(const size_type    row,
                                    const unsigned int index) const
 {
-  Assert(row < rows, ExcIndexRangeType<size_type>(row, 0, rows));
-  Assert(index < row_length(row), ExcIndexRange(index, 0, row_length(row)));
+  AssertIndexRange(row, rows);
+  AssertIndexRange(index, row_length(row));
 
   return colnums[rowstart[row] + index];
 }
@@ -1719,7 +1733,7 @@ SparsityPattern::copy_from(const size_type       n_rows,
         {
           const size_type col =
             internal::SparsityPatternTools::get_column_index_from_iterator(*j);
-          Assert(col < n_cols, ExcIndexRange(col, 0, n_cols));
+          AssertIndexRange(col, n_cols);
 
           if ((col != row) || !is_square)
             *cols++ = col;
