@@ -28,6 +28,7 @@ DEAL_II_NAMESPACE_OPEN
 namespace TrilinosWrappers
 {
   // Forward declarations:
+#  ifndef DOXYGEN
   class SparseMatrix;
   class PreconditionBase;
   class BlockSparseMatrix;
@@ -45,7 +46,7 @@ namespace TrilinosWrappers
       class TrilinosBlockPayload;
     }
   } // namespace internal
-
+#  endif
 
   /**
    * @name Creation of a LinearOperator
