@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2019 by the deal.II authors
+// Copyright (C) 2016 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,15 +17,17 @@
 #ifndef dealii_vector_operations_internal_h
 #define dealii_vector_operations_internal_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/memory_space.h>
 #include <deal.II/base/multithread_info.h>
 #include <deal.II/base/parallel.h>
-#include <deal.II/base/thread_management.h>
 #include <deal.II/base/types.h>
 #include <deal.II/base/vectorization.h>
 
 #include <deal.II/lac/cuda_kernels.h>
 #include <deal.II/lac/cuda_kernels.templates.h>
+#include <deal.II/lac/vector_operation.h>
 
 #include <cstdio>
 #include <cstring>
@@ -56,28 +58,6 @@ namespace internal
     }
 
 
-    template <typename T>
-    void
-    print(const T &t, const char *format)
-    {
-      if (format != nullptr)
-        std::printf(format, t);
-      else
-        std::printf(" %5.2f", double(t));
-    }
-
-
-
-    template <typename T>
-    void
-    print(const std::complex<T> &t, const char *format)
-    {
-      if (format != nullptr)
-        std::printf(format, t.real(), t.imag());
-      else
-        std::printf(" %5.2f+%5.2fi", double(t.real()), double(t.imag()));
-    }
-
     // call std::copy, except for in
     // the case where we want to copy
     // from std::complex to a
@@ -167,10 +147,11 @@ namespace internal
     template <typename Functor>
     void
     parallel_for(
-      Functor &                                                  functor,
-      const size_type                                            start,
-      const size_type                                            end,
-      const std::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
+      Functor &       functor,
+      const size_type start,
+      const size_type end,
+      const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+        &partitioner)
     {
 #ifdef DEAL_II_WITH_THREADS
       const size_type vec_size = end - start;
@@ -195,12 +176,12 @@ namespace internal
           // vector entries). The number of chunks here is calculated inside
           // TBBForFunctor. See also GitHub issue #2496 for further discussion
           // of this strategy.
-          parallel::internal::parallel_for(static_cast<size_type>(0),
-                                           static_cast<size_type>(
-                                             generic_functor.n_chunks),
-                                           generic_functor,
-                                           1,
-                                           tbb_partitioner);
+          ::dealii::parallel::internal::parallel_for(
+            static_cast<size_type>(0),
+            static_cast<size_type>(generic_functor.n_chunks),
+            generic_functor,
+            1,
+            tbb_partitioner);
           partitioner->release_one_partitioner(tbb_partitioner);
         }
       else if (vec_size > 0)
@@ -301,7 +282,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -332,7 +313,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -366,7 +347,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -396,7 +377,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -424,7 +405,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -452,7 +433,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -487,7 +468,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -521,7 +502,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -559,7 +540,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -591,7 +572,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -622,7 +603,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -658,7 +639,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -700,7 +681,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -734,7 +715,7 @@ namespace internal
       void
       operator()(const size_type begin, const size_type end) const
       {
-        if (parallel::internal::EnableOpenMPSimdFor<Number>::value)
+        if (::dealii::parallel::internal::EnableOpenMPSimdFor<Number>::value)
           {
             DEAL_II_OPENMP_SIMD_PRAGMA
             for (size_type i = begin; i < end; ++i)
@@ -762,9 +743,8 @@ namespace internal
     template <typename Number, typename Number2>
     struct Dot
     {
-      static const bool vectorizes =
-        std::is_same<Number, Number2>::value &&
-        (VectorizedArray<Number>::n_array_elements > 1);
+      static constexpr bool vectorizes = std::is_same<Number, Number2>::value &&
+                                         (VectorizedArray<Number>::size() > 1);
 
       Dot(const Number *const X, const Number2 *const Y)
         : X(X)
@@ -804,8 +784,7 @@ namespace internal
     template <typename Number, typename RealType>
     struct Norm2
     {
-      static const bool vectorizes =
-        VectorizedArray<Number>::n_array_elements > 1;
+      static const bool vectorizes = VectorizedArray<Number>::size() > 1;
 
       Norm2(const Number *const X)
         : X(X)
@@ -831,8 +810,7 @@ namespace internal
     template <typename Number, typename RealType>
     struct Norm1
     {
-      static const bool vectorizes =
-        VectorizedArray<Number>::n_array_elements > 1;
+      static const bool vectorizes = VectorizedArray<Number>::size() > 1;
 
       Norm1(const Number *X)
         : X(X)
@@ -858,8 +836,7 @@ namespace internal
     template <typename Number, typename RealType>
     struct NormP
     {
-      static const bool vectorizes =
-        VectorizedArray<Number>::n_array_elements > 1;
+      static const bool vectorizes = VectorizedArray<Number>::size() > 1;
 
       NormP(const Number *X, RealType p)
         : X(X)
@@ -887,8 +864,7 @@ namespace internal
     template <typename Number>
     struct MeanValue
     {
-      static const bool vectorizes =
-        VectorizedArray<Number>::n_array_elements > 1;
+      static const bool vectorizes = VectorizedArray<Number>::size() > 1;
 
       MeanValue(const Number *X)
         : X(X)
@@ -914,8 +890,7 @@ namespace internal
     template <typename Number>
     struct AddAndDot
     {
-      static const bool vectorizes =
-        VectorizedArray<Number>::n_array_elements > 1;
+      static const bool vectorizes = VectorizedArray<Number>::size() > 1;
 
       AddAndDot(Number *const       X,
                 const Number *const V,
@@ -1200,8 +1175,8 @@ namespace internal
       // First we work on (n_chunks/nvecs) chunks, where each chunk processes
       // nvecs*(4*8) elements.
 
-      const unsigned int nvecs = VectorizedArray<Number>::n_array_elements;
-      const size_type    regular_chunks = n_chunks / nvecs;
+      constexpr unsigned int nvecs          = VectorizedArray<Number>::size();
+      const size_type        regular_chunks = n_chunks / nvecs;
       for (size_type i = 0; i < regular_chunks; ++i)
         {
           VectorizedArray<Number> r0 = op.do_vectorized(index);
@@ -1219,8 +1194,7 @@ namespace internal
           r0 += r1;
           r2 += r3;
           r0 += r2;
-          r0.store(
-            &outer_results[i * VectorizedArray<Number>::n_array_elements]);
+          r0.store(&outer_results[i * nvecs]);
         }
 
       // If we are treating a case where the vector length is not divisible by
@@ -1229,9 +1203,12 @@ namespace internal
       // regular_chunks * nvecs; We do as much as possible with 2 SIMD
       // operations within each chunk. Here we assume that nvecs < 32/2 = 16 as
       // well as 16%nvecs==0.
-      AssertIndexRange(VectorizedArray<Number>::n_array_elements, 17);
+      static_assert(
+        VectorizedArray<Number>::size() <= 16 &&
+          16 % VectorizedArray<Number>::size() == 0,
+        "VectorizedArray::size() must be a power of 2 and not more than 16");
       Assert(16 % nvecs == 0, ExcInternalError());
-      if (n_chunks % VectorizedArray<Number>::n_array_elements != 0)
+      if (n_chunks % nvecs != 0)
         {
           VectorizedArray<Number> r0  = VectorizedArray<Number>(),
                                   r1  = VectorizedArray<Number>();
@@ -1246,7 +1223,7 @@ namespace internal
           r0.store(&outer_results[start_irreg]);
           // update n_chunks to denote unused element in outer_results[] from
           // which we can keep writing.
-          n_chunks = start_irreg + VectorizedArray<Number>::n_array_elements;
+          n_chunks = start_irreg + VectorizedArray<Number>::size();
         }
     }
 
@@ -1374,11 +1351,12 @@ namespace internal
     template <typename Operation, typename ResultType>
     void
     parallel_reduce(
-      const Operation &                                          op,
-      const size_type                                            start,
-      const size_type                                            end,
-      ResultType &                                               result,
-      const std::shared_ptr<parallel::internal::TBBPartitioner> &partitioner)
+      const Operation &op,
+      const size_type  start,
+      const size_type  end,
+      ResultType &     result,
+      const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+        &partitioner)
     {
 #ifdef DEAL_II_WITH_THREADS
       const size_type vec_size = end - start;
@@ -1405,12 +1383,12 @@ namespace internal
           // vector entries). The number of chunks here is calculated inside
           // TBBForFunctor. See also GitHub issue #2496 for further discussion
           // of this strategy.
-          parallel::internal::parallel_for(static_cast<size_type>(0),
-                                           static_cast<size_type>(
-                                             generic_functor.n_chunks),
-                                           generic_functor,
-                                           1,
-                                           tbb_partitioner);
+          ::dealii::parallel::internal::parallel_for(
+            static_cast<size_type>(0),
+            static_cast<size_type>(generic_functor.n_chunks),
+            generic_functor,
+            1,
+            tbb_partitioner);
           partitioner->release_one_partitioner(tbb_partitioner);
           result = generic_functor.do_sum();
         }
@@ -1657,6 +1635,18 @@ namespace internal
       {
         return Number();
       }
+
+      template <typename MemorySpace2>
+      static void
+      import(
+        const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner> &
+        /*thread_loop_partitioner*/,
+        const size_type /*size*/,
+        VectorOperation::values /*operation*/,
+        const ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace2>
+          & /*v_data*/,
+        ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace> & /*data*/)
+      {}
     };
 
 
@@ -2002,6 +1992,66 @@ namespace internal
 
         return sum;
       }
+
+      template <typename MemorySpace2>
+      static void
+      import(const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+               &                     thread_loop_partitioner,
+             const size_type         size,
+             VectorOperation::values operation,
+             const ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace2>
+               &v_data,
+             ::dealii::MemorySpace::MemorySpaceData<Number,
+                                                    ::dealii::MemorySpace::Host>
+               &data,
+             typename std::enable_if<
+               std::is_same<MemorySpace2, dealii::MemorySpace::Host>::value,
+               int>::type = 0)
+      {
+        if (operation == VectorOperation::insert)
+          {
+            copy(thread_loop_partitioner, size, v_data, data);
+          }
+        else if (operation == VectorOperation::add)
+          {
+            add_vector(thread_loop_partitioner, size, v_data, data);
+          }
+        else
+          {
+            AssertThrow(false, ExcNotImplemented());
+          }
+      }
+
+#ifdef DEAL_II_COMPILER_CUDA_AWARE
+      template <typename MemorySpace2>
+      static void
+      import(const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+               & /*thread_loop_partitioner*/,
+             const size_type         size,
+             VectorOperation::values operation,
+             const ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace2>
+               &v_data,
+             ::dealii::MemorySpace::MemorySpaceData<Number,
+                                                    ::dealii::MemorySpace::Host>
+               &data,
+             typename std::enable_if<
+               std::is_same<MemorySpace2, ::dealii::MemorySpace::CUDA>::value,
+               int>::type = 0)
+      {
+        if (operation == VectorOperation::insert)
+          {
+            cudaError_t cuda_error_code = cudaMemcpy(data.values.get(),
+                                                     v_data.values_dev.get(),
+                                                     size * sizeof(Number),
+                                                     cudaMemcpyDeviceToHost);
+            AssertCuda(cuda_error_code);
+          }
+        else
+          {
+            AssertThrow(false, ExcNotImplemented());
+          }
+      }
+#endif
     };
 
 
@@ -2043,13 +2093,7 @@ namespace internal
         const int n_blocks = 1 + size / (chunk_size * block_size);
         ::dealii::LinearAlgebra::CUDAWrappers::kernel::set<Number>
           <<<n_blocks, block_size>>>(data.values_dev.get(), s, size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2068,13 +2112,7 @@ namespace internal
                                      1.,
                                      v_data.values_dev.get(),
                                      size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2093,13 +2131,7 @@ namespace internal
                                      -1.,
                                      v_data.values_dev.get(),
                                      size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2114,13 +2146,7 @@ namespace internal
         const int n_blocks = 1 + size / (chunk_size * block_size);
         ::dealii::LinearAlgebra::CUDAWrappers::kernel::vec_add<Number>
           <<<n_blocks, block_size>>>(data.values_dev.get(), a, size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2140,13 +2166,7 @@ namespace internal
                                      a,
                                      v_data.values_dev.get(),
                                      size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2171,13 +2191,7 @@ namespace internal
                                                     b,
                                                     w_data.values_dev.get(),
                                                     size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2195,13 +2209,7 @@ namespace internal
         ::dealii::LinearAlgebra::CUDAWrappers::kernel::sadd<Number>
           <<<dim3(n_blocks, 1), dim3(block_size)>>>(
             x, data.values_dev.get(), 1., v_data.values_dev.get(), size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2220,13 +2228,7 @@ namespace internal
         ::dealii::LinearAlgebra::CUDAWrappers::kernel::sadd<Number>
           <<<dim3(n_blocks, 1), dim3(block_size)>>>(
             x, data.values_dev.get(), a, v_data.values_dev.get(), size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2253,13 +2255,7 @@ namespace internal
                                                     b,
                                                     w_data.values_dev.get(),
                                                     size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2274,13 +2270,7 @@ namespace internal
         const int n_blocks = 1 + size / (chunk_size * block_size);
         ::dealii::LinearAlgebra::CUDAWrappers::kernel::vec_scale<Number>
           <<<n_blocks, block_size>>>(data.values_dev.get(), factor, size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2298,13 +2288,7 @@ namespace internal
           <<<dim3(n_blocks, 1), dim3(block_size)>>>(data.values_dev.get(),
                                                     v_data.values_dev.get(),
                                                     size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2324,13 +2308,7 @@ namespace internal
                                                     a,
                                                     v_data.values_dev.get(),
                                                     size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static void
@@ -2355,13 +2333,7 @@ namespace internal
                                                     b,
                                                     w_data.values_dev.get(),
                                                     size);
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
       }
 
       static Number
@@ -2388,13 +2360,7 @@ namespace internal
                                                     v_data.values_dev.get(),
                                                     static_cast<unsigned int>(
                                                       size));
-
-#  ifdef DEBUG
-        // Check that the kernel was launched correctly
-        AssertCuda(cudaGetLastError());
-        // Check that there was no problem during the execution of the kernel
-        AssertCuda(cudaDeviceSynchronize());
-#  endif
+        AssertCudaKernel();
 
         // Copy the result back to the host
         Number result;
@@ -2542,6 +2508,64 @@ namespace internal
 
         return res;
       }
+
+      template <typename MemorySpace2>
+      static void
+      import(const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+               &                     thread_loop_partitioner,
+             const size_type         size,
+             VectorOperation::values operation,
+             const ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace2>
+               &v_data,
+             ::dealii::MemorySpace::MemorySpaceData<Number,
+                                                    ::dealii::MemorySpace::CUDA>
+               &data,
+             typename std::enable_if<
+               std::is_same<MemorySpace2, ::dealii::MemorySpace::CUDA>::value,
+               int>::type = 0)
+      {
+        if (operation == VectorOperation::insert)
+          {
+            copy(thread_loop_partitioner, size, v_data, data);
+          }
+        else if (operation == VectorOperation::add)
+          {
+            add_vector(thread_loop_partitioner, size, v_data, data);
+          }
+        else
+          {
+            AssertThrow(false, ExcNotImplemented());
+          }
+      }
+
+      template <typename MemorySpace2>
+      static void
+      import(const std::shared_ptr<::dealii::parallel::internal::TBBPartitioner>
+               & /*thread_loop_partitioner*/,
+             const size_type         size,
+             VectorOperation::values operation,
+             const ::dealii::MemorySpace::MemorySpaceData<Number, MemorySpace2>
+               &v_data,
+             ::dealii::MemorySpace::MemorySpaceData<Number,
+                                                    ::dealii::MemorySpace::CUDA>
+               &data,
+             typename std::enable_if<
+               std::is_same<MemorySpace2, ::dealii::MemorySpace::Host>::value,
+               int>::type = 0)
+      {
+        if (operation == VectorOperation::insert)
+          {
+            cudaError_t cuda_error_code = cudaMemcpy(data.values_dev.get(),
+                                                     v_data.values.get(),
+                                                     size * sizeof(Number),
+                                                     cudaMemcpyHostToDevice);
+            AssertCuda(cuda_error_code);
+          }
+        else
+          {
+            AssertThrow(false, ExcNotImplemented());
+          }
+      }
     };
 #endif
   } // namespace VectorOperations
