@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2018 by the deal.II authors
+// Copyright (C) 2016 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -23,6 +23,7 @@
 #  include <deal.II/base/tensor.h>
 #  include <deal.II/base/utilities.h>
 
+#  include <deal.II/lac/cuda_atomic.h>
 #  include <deal.II/lac/cuda_vector.h>
 
 #  include <deal.II/matrix_free/cuda_hanging_nodes_internal.h>
@@ -37,6 +38,25 @@ DEAL_II_NAMESPACE_OPEN
  */
 namespace CUDAWrappers
 {
+  namespace internal
+  {
+    /**
+     * Compute the dof/quad index for a given thread id, dimension, and
+     * number of points in each space dimensions.
+     */
+    template <int dim, int n_points_1d>
+    __device__ inline unsigned int
+    compute_index()
+    {
+      return (dim == 1 ?
+                threadIdx.x % n_points_1d :
+                dim == 2 ?
+                threadIdx.x % n_points_1d + n_points_1d * threadIdx.y :
+                threadIdx.x % n_points_1d +
+                    n_points_1d * (threadIdx.y + n_points_1d * threadIdx.z));
+    }
+  } // namespace internal
+
   /**
    * This class provides all the functions necessary to evaluate functions at
    * quadrature points and cell integrations. In functionality, this class is
@@ -72,13 +92,40 @@ namespace CUDAWrappers
   class FEEvaluation
   {
   public:
-    using value_type    = Number;
+    /**
+     * An alias for scalar quantities.
+     */
+    using value_type = Number;
+
+    /**
+     * An alias for vectorial quantities.
+     */
     using gradient_type = Tensor<1, dim, Number>;
-    using data_type     = typename MatrixFree<dim, Number>::Data;
-    static constexpr unsigned int dimension    = dim;
+
+    /**
+     * An alias to kernel specific information.
+     */
+    using data_type = typename MatrixFree<dim, Number>::Data;
+
+    /**
+     * Dimension.
+     */
+    static constexpr unsigned int dimension = dim;
+
+    /**
+     * Number of components.
+     */
     static constexpr unsigned int n_components = n_components_;
+
+    /**
+     * Number of quadrature points per cell.
+     */
     static constexpr unsigned int n_q_points =
       Utilities::pow(n_q_points_1d, dim);
+
+    /**
+     * Number of tensor degrees of freedoms per cell.
+     */
     static constexpr unsigned int tensor_dofs_per_cell =
       Utilities::pow(fe_degree + 1, dim);
 
@@ -132,33 +179,104 @@ namespace CUDAWrappers
 
     /**
      * Return the value of a finite element function at quadrature point
-     * number @p q_point after a call to @p evalue(true,...).
+     * number @p q_point after a call to @p evaluate(true,...).
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ value_type
+                                  get_value(const unsigned int q_point) const;
+
+    /**
+     * Same as above, except that the quadrature point is computed from thread
+     * id.
      */
     __device__ value_type
-               get_value(const unsigned int q_point) const;
+               get_value() const;
+
+    /**
+     * Return the value of a finite element function at degree of freedom
+     * @p dof after a call to integrate() or before a call to evaluate().
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ value_type
+                                  get_dof_value(const unsigned int dof) const;
+
+    /**
+     * Same as above, except that the local dof index is computed from the
+     * thread id.
+     */
+    __device__ value_type
+               get_dof_value() const;
 
     /**
      * Write a value to the field containing the values on quadrature points
      * with component @p q_point. Access to the same fields as through @p
-     * get_value(), This specifies the value which is tested by all basis
+     * get_value(). This specifies the value which is tested by all basis
      * function on the current cell and integrated over.
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ void
+                       submit_value(const value_type &val_in, const unsigned int q_point);
+
+    /**
+     * Same as above, except that the quadrature point is computed from the
+     * thread id.
      */
     __device__ void
-    submit_value(const value_type &val_in, const unsigned int q_point);
+    submit_value(const value_type &val_in);
+
+    /**
+     * Write a value to the field containing the values for the degree of
+     * freedom with index @p dof after a call to integrate() or before
+     * calling evaluate(). Access through the same fields as through
+     * get_dof_value().
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ void
+                       submit_dof_value(const value_type &val_in, const unsigned int dof);
+
+    /**
+     * Same as above, except that the local dof index is computed from the
+     * thread id.
+     */
+    __device__ void
+    submit_dof_value(const value_type &val_in);
 
     /**
      * Return the gradient of a finite element function at quadrature point
      * number @p q_point after a call to @p evaluate(...,true).
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ gradient_type
+                                  get_gradient(const unsigned int q_point) const;
+
+    /**
+     * Same as above, except that the quadrature point is computed from the
+     * thread id.
      */
     __device__ gradient_type
-               get_gradient(const unsigned int q_point) const;
+               get_gradient() const;
 
     /**
      * Write a contribution that is tested by the gradient to the field
-     * containing the values on quadrature points with component @p q_point
+     * containing the values on quadrature points with component @p q_point.
+     *
+     * @deprecated Use the version without parameters instead.
+     */
+    DEAL_II_DEPRECATED __device__ void
+                       submit_gradient(const gradient_type &grad_in, const unsigned int q_point);
+
+
+    /**
+     * Same as above, except that the quadrature point is computed from the
+     * thread id.
      */
     __device__ void
-    submit_gradient(const gradient_type &grad_in, const unsigned int q_point);
+    submit_gradient(const gradient_type &grad_in);
 
     // clang-format off
     /**
@@ -170,11 +288,29 @@ namespace CUDAWrappers
      *   CUDAWrappers::FEEvaluation<dim, fe_degree, n_q_points_1d, n_components, Number> *fe_eval,
      *   const unsigned int                                                               q_point) const;
      * \endcode
+     *
+     * @deprecated Use apply_for_each_quad_point() instead.
+     */
+    // clang-format on
+    template <typename Functor>
+    DEAL_II_DEPRECATED __device__ void
+                       apply_quad_point_operations(const Functor &func);
+
+    // clang-format off
+    /**
+     * Same as above, except that the functor @p func only takes a single input
+     * argument (fe_eval) and computes the quadrature point from the thread id.
+     *
+     * @p func needs to define
+     * \code
+     * __device__ void operator()(
+     *   CUDAWrappers::FEEvaluation<dim, fe_degree, n_q_points_1d, n_components, Number> *fe_eval) const;
+     * \endcode
      */
     // clang-format on
     template <typename Functor>
     __device__ void
-    apply_quad_point_operations(const Functor &func);
+    apply_for_each_quad_point(const Functor &func);
 
   private:
     types::global_dof_index *local_to_global;
@@ -183,6 +319,8 @@ namespace CUDAWrappers
 
     const unsigned int constraint_mask;
 
+    const bool use_coloring;
+
     Number *inv_jac;
     Number *JxW;
 
@@ -206,6 +344,7 @@ namespace CUDAWrappers
     : n_cells(data->n_cells)
     , padding_length(data->padding_length)
     , constraint_mask(data->constraint_mask[cell_id])
+    , use_coloring(data->use_coloring)
     , values(shdata->values)
   {
     local_to_global = data->local_to_global + padding_length * cell_id;
@@ -229,20 +368,16 @@ namespace CUDAWrappers
   {
     static_assert(n_components_ == 1, "This function only supports FE with one \
                   components");
-    const unsigned int idx =
-      (threadIdx.x % n_q_points_1d) +
-      (dim > 1 ? threadIdx.y : 0) * n_q_points_1d +
-      (dim > 2 ? threadIdx.z : 0) * n_q_points_1d * n_q_points_1d;
+    const unsigned int idx = internal::compute_index<dim, n_q_points_1d>();
 
     const types::global_dof_index src_idx = local_to_global[idx];
     // Use the read-only data cache.
     values[idx] = __ldg(&src[src_idx]);
 
-    if (constraint_mask)
-      internal::resolve_hanging_nodes<dim, fe_degree, false>(constraint_mask,
-                                                             values);
-
     __syncthreads();
+
+    internal::resolve_hanging_nodes<dim, fe_degree, false>(constraint_mask,
+                                                           values);
   }
 
 
@@ -258,17 +393,18 @@ namespace CUDAWrappers
   {
     static_assert(n_components_ == 1, "This function only supports FE with one \
                   components");
-    if (constraint_mask)
-      internal::resolve_hanging_nodes<dim, fe_degree, true>(constraint_mask,
-                                                            values);
+    internal::resolve_hanging_nodes<dim, fe_degree, true>(constraint_mask,
+                                                          values);
+
+    const unsigned int idx = internal::compute_index<dim, n_q_points_1d>();
 
-    const unsigned int idx =
-      (threadIdx.x % n_q_points_1d) +
-      (dim > 1 ? threadIdx.y : 0) * n_q_points_1d +
-      (dim > 2 ? threadIdx.z : 0) * n_q_points_1d * n_q_points_1d;
     const types::global_dof_index destination_idx = local_to_global[idx];
 
-    dst[destination_idx] += values[idx];
+    if (use_coloring)
+      dst[destination_idx] += values[idx];
+    else
+      LinearAlgebra::CUDAWrappers::atomicAdd_wrapper(&dst[destination_idx],
+                                                     values[idx]);
   }
 
 
@@ -292,13 +428,18 @@ namespace CUDAWrappers
       n_q_points_1d,
       Number>
       evaluator_tensor_product;
-    if (evaluate_grad == true)
+    if (evaluate_val == true && evaluate_grad == true)
+      {
+        evaluator_tensor_product.value_and_gradient_at_quad_pts(values,
+                                                                gradients);
+        __syncthreads();
+      }
+    else if (evaluate_grad == true)
       {
         evaluator_tensor_product.gradient_at_quad_pts(values, gradients);
         __syncthreads();
       }
-
-    if (evaluate_val == true)
+    else if (evaluate_val == true)
       {
         evaluator_tensor_product.value_at_quad_pts(values);
         __syncthreads();
@@ -324,16 +465,15 @@ namespace CUDAWrappers
       n_q_points_1d,
       Number>
       evaluator_tensor_product;
-    if (integrate_val == true)
+    if (integrate_val == true && integrate_grad == true)
+      {
+        evaluator_tensor_product.integrate_value_and_gradient(values,
+                                                              gradients);
+      }
+    else if (integrate_val == true)
       {
         evaluator_tensor_product.integrate_value(values);
         __syncthreads();
-        if (integrate_grad == true)
-          {
-            evaluator_tensor_product.integrate_gradient<true>(values,
-                                                              gradients);
-            __syncthreads();
-          }
       }
     else if (integrate_grad == true)
       {
@@ -362,6 +502,62 @@ namespace CUDAWrappers
 
 
 
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ typename FEEvaluation<dim,
+                                   fe_degree,
+                                   n_q_points_1d,
+                                   n_components_,
+                                   Number>::value_type
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    get_value() const
+  {
+    const unsigned int q_point = internal::compute_index<dim, n_q_points_1d>();
+    return values[q_point];
+  }
+
+
+
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ typename FEEvaluation<dim,
+                                   fe_degree,
+                                   n_q_points_1d,
+                                   n_components_,
+                                   Number>::value_type
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    get_dof_value(const unsigned int dof) const
+  {
+    return values[dof];
+  }
+
+
+
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ typename FEEvaluation<dim,
+                                   fe_degree,
+                                   n_q_points_1d,
+                                   n_components_,
+                                   Number>::value_type
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    get_dof_value() const
+  {
+    const unsigned int dof = internal::compute_index<dim, fe_degree + 1>();
+    return values[dof];
+  }
+
+
+
   template <int dim,
             int fe_degree,
             int n_q_points_1d,
@@ -376,6 +572,50 @@ namespace CUDAWrappers
 
 
 
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ void
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    submit_value(const value_type &val_in)
+  {
+    const unsigned int q_point = internal::compute_index<dim, n_q_points_1d>();
+    values[q_point]            = val_in * JxW[q_point];
+  }
+
+
+
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ void
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    submit_dof_value(const value_type &val_in, const unsigned int dof)
+  {
+    values[dof] = val_in;
+  }
+
+
+
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ void
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    submit_dof_value(const value_type &val_in)
+  {
+    const unsigned int dof = internal::compute_index<dim, fe_degree + 1>();
+    values[dof]            = val_in;
+  }
+
+
+
   template <int dim,
             int fe_degree,
             int n_q_points_1d,
@@ -408,6 +648,40 @@ namespace CUDAWrappers
 
 
 
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ typename FEEvaluation<dim,
+                                   fe_degree,
+                                   n_q_points_1d,
+                                   n_components_,
+                                   Number>::gradient_type
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    get_gradient() const
+  {
+    static_assert(n_components_ == 1, "This function only supports FE with one \
+                  components");
+
+    // TODO optimize if the mesh is uniform
+    const unsigned int q_point = internal::compute_index<dim, n_q_points_1d>();
+    const Number *     inv_jacobian = &inv_jac[q_point];
+    gradient_type      grad;
+    for (int d_1 = 0; d_1 < dim; ++d_1)
+      {
+        Number tmp = 0.;
+        for (int d_2 = 0; d_2 < dim; ++d_2)
+          tmp += inv_jacobian[padding_length * n_cells * (dim * d_2 + d_1)] *
+                 gradients[d_2][q_point];
+        grad[d_1] = tmp;
+      }
+
+    return grad;
+  }
+
+
+
   template <int dim,
             int fe_degree,
             int n_q_points_1d,
@@ -431,6 +705,30 @@ namespace CUDAWrappers
 
 
 
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  __device__ void
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    submit_gradient(const gradient_type &grad_in)
+  {
+    // TODO optimize if the mesh is uniform
+    const unsigned int q_point = internal::compute_index<dim, n_q_points_1d>();
+    const Number *     inv_jacobian = &inv_jac[q_point];
+    for (int d_1 = 0; d_1 < dim; ++d_1)
+      {
+        Number tmp = 0.;
+        for (int d_2 = 0; d_2 < dim; ++d_2)
+          tmp += inv_jacobian[n_cells * padding_length * (dim * d_1 + d_2)] *
+                 grad_in[d_2];
+        gradients[d_1][q_point] = tmp * JxW[q_point];
+      }
+  }
+
+
+
   template <int dim,
             int fe_degree,
             int n_q_points_1d,
@@ -441,14 +739,24 @@ namespace CUDAWrappers
   FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
     apply_quad_point_operations(const Functor &func)
   {
-    const unsigned int q_point =
-      (dim == 1 ?
-         threadIdx.x % n_q_points_1d :
-         dim == 2 ?
-         threadIdx.x % n_q_points_1d + n_q_points_1d * threadIdx.y :
-         threadIdx.x % n_q_points_1d +
-             n_q_points_1d * (threadIdx.y + n_q_points_1d * threadIdx.z));
-    func(this, q_point);
+    func(this, internal::compute_index<dim, n_q_points_1d>());
+
+    __syncthreads();
+  }
+
+
+
+  template <int dim,
+            int fe_degree,
+            int n_q_points_1d,
+            int n_components_,
+            typename Number>
+  template <typename Functor>
+  __device__ void
+  FEEvaluation<dim, fe_degree, n_q_points_1d, n_components_, Number>::
+    apply_for_each_quad_point(const Functor &func)
+  {
+    func(this);
 
     __syncthreads();
   }
