@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2019 by the deal.II authors
+// Copyright (C) 2016 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 #ifndef dealii_cuda_matrix_free_templates_h
 #define dealii_cuda_matrix_free_templates_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/matrix_free/cuda_matrix_free.h>
 
 #ifdef DEAL_II_COMPILER_CUDA_AWARE
@@ -26,6 +28,7 @@
 
 #  include <deal.II/dofs/dof_tools.h>
 
+#  include <deal.II/fe/fe_dgq.h>
 #  include <deal.II/fe/fe_values.h>
 
 #  include <deal.II/grid/filtered_iterator.h>
@@ -35,6 +38,7 @@
 
 #  include <cuda_runtime_api.h>
 
+#  include <cmath>
 #  include <functional>
 
 
@@ -45,12 +49,80 @@ namespace CUDAWrappers
   namespace internal
   {
     // These variables are stored in the device constant memory.
-    // TODO: use a template parameter
     constexpr unsigned int max_elem_degree = 10;
+
+    template <typename NumberType>
+    using DataArray = NumberType[(max_elem_degree + 1) * (max_elem_degree + 1)];
+
     __constant__ double
-      global_shape_values[(max_elem_degree + 1) * (max_elem_degree + 1)];
+      global_shape_values_d[(max_elem_degree + 1) * (max_elem_degree + 1)];
+    __constant__ float
+      global_shape_values_f[(max_elem_degree + 1) * (max_elem_degree + 1)];
+
+    template <typename Number>
+    __host__ __device__ inline DataArray<Number> &
+             get_global_shape_values();
+
+    template <>
+    __host__ __device__ inline DataArray<double> &
+             get_global_shape_values<double>()
+    {
+      return global_shape_values_d;
+    }
+
+    template <>
+    __host__ __device__ inline DataArray<float> &
+             get_global_shape_values<float>()
+    {
+      return global_shape_values_f;
+    }
+
     __constant__ double
-      global_shape_gradients[(max_elem_degree + 1) * (max_elem_degree + 1)];
+      global_shape_gradients_d[(max_elem_degree + 1) * (max_elem_degree + 1)];
+    __constant__ float
+      global_shape_gradients_f[(max_elem_degree + 1) * (max_elem_degree + 1)];
+
+    template <typename Number>
+    __host__ __device__ inline DataArray<Number> &
+             get_global_shape_gradients();
+
+    template <>
+    __host__ __device__ inline DataArray<double> &
+             get_global_shape_gradients<double>()
+    {
+      return global_shape_gradients_d;
+    }
+
+    template <>
+    __host__ __device__ inline DataArray<float> &
+             get_global_shape_gradients<float>()
+    {
+      return global_shape_gradients_f;
+    }
+
+    // for collocation methods
+    __constant__ double global_co_shape_gradients_d[(max_elem_degree + 1) *
+                                                    (max_elem_degree + 1)];
+    __constant__ float  global_co_shape_gradients_f[(max_elem_degree + 1) *
+                                                   (max_elem_degree + 1)];
+
+    template <typename Number>
+    __host__ __device__ inline DataArray<Number> &
+             get_global_co_shape_gradients();
+
+    template <>
+    __host__ __device__ inline DataArray<double> &
+             get_global_co_shape_gradients<double>()
+    {
+      return global_co_shape_gradients_d;
+    }
+
+    template <>
+    __host__ __device__ inline DataArray<float> &
+             get_global_co_shape_gradients<float>()
+    {
+      return global_co_shape_gradients_f;
+    }
 
     template <typename Number>
     using CUDAVector = ::dealii::LinearAlgebra::CUDAWrappers::Vector<Number>;
@@ -206,7 +278,9 @@ namespace CUDAWrappers
     void
     ReinitHelper<dim, Number>::setup_color_arrays(const unsigned int n_colors)
     {
-      data->n_cells.resize(n_colors);
+      // We need at least three colors when we are using CUDA-aware MPI and
+      // overlapping the communication
+      data->n_cells.resize(std::max(n_colors, 3U), 0);
       data->grid_dim.resize(n_colors);
       data->block_dim.resize(n_colors);
       data->local_to_global.resize(n_colors);
@@ -312,9 +386,9 @@ namespace CUDAWrappers
         {
           const std::vector<Point<dim>> &q_points =
             fe_values.get_quadrature_points();
-          memcpy(&q_points_host[cell_id * padding_length],
-                 q_points.data(),
-                 q_points_per_cell * sizeof(Point<dim>));
+          std::copy(q_points.begin(),
+                    q_points.end(),
+                    &q_points_host[cell_id * padding_length]);
         }
 
       if (update_flags & update_JxW_values)
@@ -329,9 +403,11 @@ namespace CUDAWrappers
         {
           const std::vector<DerivativeForm<1, dim, dim>> &inv_jacobians =
             fe_values.get_inverse_jacobians();
-          memcpy(&inv_jacobian_host[cell_id * padding_length * dim * dim],
-                 inv_jacobians.data(),
-                 q_points_per_cell * sizeof(DerivativeForm<1, dim, dim>));
+          std::copy(&inv_jacobians[0][0][0],
+                    &inv_jacobians[0][0][0] +
+                      q_points_per_cell * sizeof(DerivativeForm<1, dim, dim>) /
+                        sizeof(double),
+                    &inv_jacobian_host[cell_id * padding_length * dim * dim]);
         }
     }
 
@@ -527,7 +603,8 @@ namespace CUDAWrappers
 
   template <int dim, typename Number>
   MatrixFree<dim, Number>::MatrixFree()
-    : constrained_dofs(nullptr)
+    : n_dofs(0)
+    , constrained_dofs(nullptr)
     , padding_length(0)
   {}
 
@@ -539,11 +616,12 @@ namespace CUDAWrappers
                                   const DoFHandler<dim> &          dof_handler,
                                   const AffineConstraints<Number> &constraints,
                                   const Quadrature<1> &            quad,
-                                  const AdditionalData additional_data)
+                                  const AdditionalData &additional_data)
   {
     const auto &triangulation = dof_handler.get_triangulation();
     if (const auto parallel_triangulation =
-          dynamic_cast<const parallel::Triangulation<dim> *>(&triangulation))
+          dynamic_cast<const parallel::TriangulationBase<dim> *>(
+            &triangulation))
       internal_reinit(mapping,
                       dof_handler,
                       constraints,
@@ -563,7 +641,7 @@ namespace CUDAWrappers
   MatrixFree<dim, Number>::reinit(const DoFHandler<dim> &          dof_handler,
                                   const AffineConstraints<Number> &constraints,
                                   const Quadrature<1> &            quad,
-                                  const AdditionalData additional_data)
+                                  const AdditionalData &additional_data)
   {
     reinit(StaticMappingQ1<dim>::mapping,
            dof_handler,
@@ -587,6 +665,7 @@ namespace CUDAWrappers
     data_copy.n_cells         = n_cells[color];
     data_copy.padding_length  = padding_length;
     data_copy.row_start       = row_start[color];
+    data_copy.use_coloring    = use_coloring;
 
     return data_copy;
   }
@@ -597,69 +676,27 @@ namespace CUDAWrappers
   void
   MatrixFree<dim, Number>::free()
   {
-    for (unsigned int i = 0; i < q_points.size(); ++i)
-      {
-        if (q_points[i] != nullptr)
-          {
-            cudaError_t cuda_error = cudaFree(q_points[i]);
-            AssertCuda(cuda_error);
-            q_points[i] = nullptr;
-          }
-      }
-
-    for (unsigned int i = 0; i < local_to_global.size(); ++i)
-      {
-        if (local_to_global[i] != nullptr)
-          {
-            cudaError_t cuda_error = cudaFree(local_to_global[i]);
-            AssertCuda(cuda_error);
-            local_to_global[i] = nullptr;
-          }
-      }
-
-    for (unsigned int i = 0; i < inv_jacobian.size(); ++i)
-      {
-        if (inv_jacobian[i] != nullptr)
-          {
-            cudaError_t cuda_error = cudaFree(inv_jacobian[i]);
-            AssertCuda(cuda_error);
-            inv_jacobian[i] = nullptr;
-          }
-      }
-
-    for (unsigned int i = 0; i < JxW.size(); ++i)
-      {
-        if (JxW[i] != nullptr)
-          {
-            cudaError_t cuda_error = cudaFree(JxW[i]);
-            AssertCuda(cuda_error);
-            JxW[i] = nullptr;
-          }
-      }
-
-    for (unsigned int i = 0; i < constraint_mask.size(); ++i)
-      {
-        if (constraint_mask[i] != nullptr)
-          {
-            cudaError_t cuda_error = cudaFree(constraint_mask[i]);
-            AssertCuda(cuda_error);
-            constraint_mask[i] = nullptr;
-          }
-      }
-
-
+    for (auto &q_points_color_ptr : q_points)
+      Utilities::CUDA::free(q_points_color_ptr);
     q_points.clear();
+
+    for (auto &local_to_global_color_ptr : local_to_global)
+      Utilities::CUDA::free(local_to_global_color_ptr);
     local_to_global.clear();
+
+    for (auto &inv_jacobian_color_ptr : inv_jacobian)
+      Utilities::CUDA::free(inv_jacobian_color_ptr);
     inv_jacobian.clear();
+
+    for (auto &JxW_color_ptr : JxW)
+      Utilities::CUDA::free(JxW_color_ptr);
     JxW.clear();
+
+    for (auto &constraint_mask_color_ptr : constraint_mask)
+      Utilities::CUDA::free(constraint_mask_color_ptr);
     constraint_mask.clear();
 
-    if (constrained_dofs != nullptr)
-      {
-        cudaError_t cuda_error = cudaFree(constrained_dofs);
-        AssertCuda(cuda_error);
-        constrained_dofs = nullptr;
-      }
+    Utilities::CUDA::free(constrained_dofs);
   }
 
 
@@ -698,6 +735,29 @@ namespace CUDAWrappers
 
 
 
+  template <int dim, typename Number>
+  void
+  MatrixFree<dim, Number>::initialize_dof_vector(
+    LinearAlgebra::CUDAWrappers::Vector<Number> &vec) const
+  {
+    vec.reinit(n_dofs);
+  }
+
+
+
+  template <int dim, typename Number>
+  void
+  MatrixFree<dim, Number>::initialize_dof_vector(
+    LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA> &vec) const
+  {
+    if (partitioner)
+      vec.reinit(partitioner);
+    else
+      vec.reinit(n_dofs);
+  }
+
+
+
   template <int dim, typename Number>
   unsigned int
   MatrixFree<dim, Number>::get_padding_length() const
@@ -728,8 +788,12 @@ namespace CUDAWrappers
   MatrixFree<dim, Number>::evaluate_coefficients(Functor func) const
   {
     for (unsigned int i = 0; i < n_colors; ++i)
-      internal::evaluate_coeff<dim, Number, Functor>
-        <<<grid_dim[i], block_dim[i]>>>(func, get_data(i));
+      if (n_cells[i] > 0)
+        {
+          internal::evaluate_coeff<dim, Number, Functor>
+            <<<grid_dim[i], block_dim[i]>>>(func, get_data(i));
+          AssertCudaKernel();
+        }
   }
 
 
@@ -779,10 +843,15 @@ namespace CUDAWrappers
       AssertThrow(false, ExcMessage("Invalid parallelization scheme."));
 
     this->parallelization_scheme = additional_data.parallelization_scheme;
+    this->use_coloring           = additional_data.use_coloring;
+    this->overlap_communication_computation =
+      additional_data.overlap_communication_computation;
 
     // TODO: only free if we actually need arrays of different length
     free();
 
+    n_dofs = dof_handler.n_dofs();
+
     const FiniteElement<dim> &fe = dof_handler.get_fe();
 
     fe_degree = fe.degree;
@@ -806,20 +875,37 @@ namespace CUDAWrappers
 
     unsigned int size_shape_values = n_dofs_1d * n_q_points_1d * sizeof(Number);
 
-    cudaError_t cuda_error = cudaMemcpyToSymbol(internal::global_shape_values,
-                                                shape_info.shape_values.data(),
-                                                size_shape_values,
-                                                0,
-                                                cudaMemcpyHostToDevice);
+    FE_DGQArbitraryNodes<1> fe_quad_co(quad);
+    const ::dealii::internal::MatrixFreeFunctions::ShapeInfo<Number>
+      shape_info_co(quad, fe_quad_co);
+
+    unsigned int size_co_shape_values =
+      n_q_points_1d * n_q_points_1d * sizeof(Number);
+
+    cudaError_t cuda_error =
+      cudaMemcpyToSymbol(internal::get_global_shape_values<Number>(),
+                         shape_info.data.front().shape_values.data(),
+                         size_shape_values,
+                         0,
+                         cudaMemcpyHostToDevice);
     AssertCuda(cuda_error);
 
     if (update_flags & update_gradients)
       {
-        cuda_error = cudaMemcpyToSymbol(internal::global_shape_gradients,
-                                        shape_info.shape_gradients.data(),
-                                        size_shape_values,
-                                        0,
-                                        cudaMemcpyHostToDevice);
+        cuda_error =
+          cudaMemcpyToSymbol(internal::get_global_shape_gradients<Number>(),
+                             shape_info.data.front().shape_gradients.data(),
+                             size_shape_values,
+                             0,
+                             cudaMemcpyHostToDevice);
+        AssertCuda(cuda_error);
+
+        cuda_error =
+          cudaMemcpyToSymbol(internal::get_global_co_shape_gradients<Number>(),
+                             shape_info_co.data.front().shape_gradients.data(),
+                             size_co_shape_values,
+                             0,
+                             cudaMemcpyHostToDevice);
         AssertCuda(cuda_error);
       }
 
@@ -835,13 +921,77 @@ namespace CUDAWrappers
     CellFilter begin(IteratorFilters::LocallyOwnedCell(),
                      dof_handler.begin_active());
     CellFilter end(IteratorFilters::LocallyOwnedCell(), dof_handler.end());
-    const auto fun = [&](const CellFilter &filter) {
-      return internal::get_conflict_indices<dim, Number>(filter, constraints);
-    };
-
     std::vector<std::vector<CellFilter>> graph;
+
     if (begin != end)
-      graph = GraphColoring::make_graph_coloring(begin, end, fun);
+      {
+        if (additional_data.use_coloring)
+          {
+            const auto fun = [&](const CellFilter &filter) {
+              return internal::get_conflict_indices<dim, Number>(filter,
+                                                                 constraints);
+            };
+            graph = GraphColoring::make_graph_coloring(begin, end, fun);
+          }
+        else
+          {
+            if (additional_data.overlap_communication_computation)
+              {
+                // We create one color (1) with the cells on the boundary of the
+                // local domain and two colors (0 and 2) with the interior
+                // cells.
+                graph.resize(3, std::vector<CellFilter>());
+
+                std::vector<bool> ghost_vertices(
+                  dof_handler.get_triangulation().n_vertices(), false);
+
+                for (const auto cell :
+                     dof_handler.get_triangulation().active_cell_iterators())
+                  if (cell->is_ghost())
+                    for (unsigned int i = 0;
+                         i < GeometryInfo<dim>::vertices_per_cell;
+                         i++)
+                      ghost_vertices[cell->vertex_index(i)] = true;
+
+                std::vector<dealii::FilteredIterator<dealii::TriaActiveIterator<
+                  dealii::DoFCellAccessor<dealii::DoFHandler<dim, dim>,
+                                          false>>>>
+                  inner_cells;
+
+                for (auto cell = begin; cell != end; ++cell)
+                  {
+                    bool ghost_vertex = false;
+
+                    for (unsigned int i = 0;
+                         i < GeometryInfo<dim>::vertices_per_cell;
+                         i++)
+                      if (ghost_vertices[cell->vertex_index(i)])
+                        {
+                          ghost_vertex = true;
+                          break;
+                        }
+
+                    if (ghost_vertex)
+                      graph[1].emplace_back(cell);
+                    else
+                      inner_cells.emplace_back(cell);
+                  }
+                for (unsigned i = 0; i < inner_cells.size(); i++)
+                  if (i < inner_cells.size() / 2)
+                    graph[0].emplace_back(inner_cells[i]);
+                  else
+                    graph[2].emplace_back(inner_cells[i]);
+              }
+            else
+              {
+                // If we are not using coloring, all the cells belong to the
+                // same color.
+                graph.resize(1, std::vector<CellFilter>());
+                for (auto cell = begin; cell != end; ++cell)
+                  graph[0].emplace_back(cell);
+              }
+          }
+      }
     n_colors = graph.size();
 
     helper.setup_color_arrays(n_colors);
@@ -900,8 +1050,8 @@ namespace CUDAWrappers
             unsigned int i_constraint = 0;
             for (unsigned int i = 0; i < n_local_dofs; ++i)
               {
-                // is_constrained uses a global dof id but constrained_dofs_host
-                // works on the local id
+                // is_constrained uses a global dof id but
+                // constrained_dofs_host works on the local id
                 if (constraints.is_constrained(partitioner->local_to_global(i)))
                   {
                     constrained_dofs_host[i_constraint] = i;
@@ -948,11 +1098,15 @@ namespace CUDAWrappers
   {
     // Execute the loop on the cells
     for (unsigned int i = 0; i < n_colors; ++i)
-      internal::apply_kernel_shmem<dim, Number, Functor>
-        <<<grid_dim[i], block_dim[i]>>>(func,
-                                        get_data(i),
-                                        src.get_values(),
-                                        dst.get_values());
+      if (n_cells[i] > 0)
+        {
+          internal::apply_kernel_shmem<dim, Number, Functor>
+            <<<grid_dim[i], block_dim[i]>>>(func,
+                                            get_data(i),
+                                            src.get_values(),
+                                            dst.get_values());
+          AssertCudaKernel();
+        }
   }
 
 
@@ -965,24 +1119,101 @@ namespace CUDAWrappers
     const LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA> &src,
     LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA> &dst) const
   {
-    // Create the ghosted source and the ghosted destination
-    LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA> ghosted_src(
-      partitioner);
-    LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA> ghosted_dst(
-      ghosted_src);
-    ghosted_src = src;
+    // in case we have compatible partitioners, we can simply use the provided
+    // vectors
+    if (src.get_partitioner().get() == partitioner.get() &&
+        dst.get_partitioner().get() == partitioner.get())
+      {
+        // This code is inspired to the code in TaskInfo::loop.
+        if (overlap_communication_computation)
+          {
+            src.update_ghost_values_start(0);
+            // In parallel, it's possible that some processors do not own any
+            // cells.
+            if (n_cells[0] > 0)
+              {
+                internal::apply_kernel_shmem<dim, Number, Functor>
+                  <<<grid_dim[0], block_dim[0]>>>(func,
+                                                  get_data(0),
+                                                  src.get_values(),
+                                                  dst.get_values());
+                AssertCudaKernel();
+              }
+            src.update_ghost_values_finish();
 
-    // Execute the loop on the cells
-    for (unsigned int i = 0; i < n_colors; ++i)
-      internal::apply_kernel_shmem<dim, Number, Functor>
-        <<<grid_dim[i], block_dim[i]>>>(func,
-                                        get_data(i),
-                                        ghosted_src.get_values(),
-                                        ghosted_dst.get_values());
+            // In serial this color does not exist because there are no ghost
+            // cells
+            if (n_cells[1] > 0)
+              {
+                internal::apply_kernel_shmem<dim, Number, Functor>
+                  <<<grid_dim[1], block_dim[1]>>>(func,
+                                                  get_data(1),
+                                                  src.get_values(),
+                                                  dst.get_values());
+                AssertCudaKernel();
+                // We need a synchronization point because we don't want
+                // CUDA-aware MPI to start the MPI communication until the
+                // kernel is done.
+                cudaDeviceSynchronize();
+              }
 
-    // Add the ghosted values
-    ghosted_dst.compress(VectorOperation::add);
-    dst = ghosted_dst;
+            dst.compress_start(0, VectorOperation::add);
+            // When the mesh is coarse it is possible that some processors do
+            // not own any cells
+            if (n_cells[2] > 0)
+              {
+                internal::apply_kernel_shmem<dim, Number, Functor>
+                  <<<grid_dim[2], block_dim[2]>>>(func,
+                                                  get_data(2),
+                                                  src.get_values(),
+                                                  dst.get_values());
+                AssertCudaKernel();
+              }
+            dst.compress_finish(VectorOperation::add);
+          }
+        else
+          {
+            src.update_ghost_values();
+
+            // Execute the loop on the cells
+            for (unsigned int i = 0; i < n_colors; ++i)
+              if (n_cells[i] > 0)
+                {
+                  internal::apply_kernel_shmem<dim, Number, Functor>
+                    <<<grid_dim[i], block_dim[i]>>>(func,
+                                                    get_data(i),
+                                                    src.get_values(),
+                                                    dst.get_values());
+                }
+            dst.compress(VectorOperation::add);
+          }
+        src.zero_out_ghosts();
+      }
+    else
+      {
+        // Create the ghosted source and the ghosted destination
+        LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA>
+          ghosted_src(partitioner);
+        LinearAlgebra::distributed::Vector<Number, MemorySpace::CUDA>
+          ghosted_dst(ghosted_src);
+        ghosted_src = src;
+
+        // Execute the loop on the cells
+        for (unsigned int i = 0; i < n_colors; ++i)
+          if (n_cells[i] > 0)
+            {
+              internal::apply_kernel_shmem<dim, Number, Functor>
+                <<<grid_dim[i], block_dim[i]>>>(func,
+                                                get_data(i),
+                                                ghosted_src.get_values(),
+                                                ghosted_dst.get_values());
+              AssertCudaKernel();
+            }
+
+        // Add the ghosted values
+        ghosted_dst.compress(VectorOperation::add);
+        dst = ghosted_dst;
+      }
   }
 
 
@@ -1014,6 +1245,7 @@ namespace CUDAWrappers
                                                       src.size(),
                                                       src.get_values(),
                                                       dst.get_values());
+    AssertCudaKernel();
   }
 
 
@@ -1032,6 +1264,7 @@ namespace CUDAWrappers
                                                       src.local_size(),
                                                       src.get_values(),
                                                       dst.get_values());
+    AssertCudaKernel();
   }
 
 
@@ -1059,6 +1292,7 @@ namespace CUDAWrappers
                                                       dst.size(),
                                                       val,
                                                       dst.get_values());
+    AssertCudaKernel();
   }
 
 
@@ -1075,6 +1309,7 @@ namespace CUDAWrappers
                                                       dst.local_size(),
                                                       val,
                                                       dst.get_values());
+    AssertCudaKernel();
   }
 
 
