@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2019 by the deal.II authors
+// Copyright (C) 2011 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,8 @@
 #define dealii_matrix_free_shape_info_h
 
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/aligned_vector.h>
 #include <deal.II/base/exceptions.h>
 #include <deal.II/base/quadrature_lib.h>
@@ -92,19 +94,208 @@ namespace internal
       tensor_symmetric_plus_dg0 = 5
     };
 
+
+
     /**
-     * The class that stores the shape functions, gradients and Hessians
-     * evaluated for a tensor product finite element and tensor product
-     * quadrature formula on the unit cell. Because of this structure, only
-     * one-dimensional data is stored.
+     * This struct stores the shape functions, their gradients and Hessians
+     * evaluated for a one-dimensional section of a tensor product finite
+     * element and tensor product quadrature formula in reference
+     * coordinates. This data structure also includes the evaluation of
+     * quantities at the cell boundary and on the sub-interval $(0, 0.5)$ and
+     * $(0.5, 1)$ for face integrals.
+     *
+     * @author Katharina Kormann, Martin Kronbichler, Julius Witte, 2010-2020
+     */
+    template <typename Number>
+    struct UnivariateShapeData
+    {
+      /**
+       * Empty constructor. Sets default configuration.
+       */
+      UnivariateShapeData();
+
+      /**
+       * Return the memory consumption of this class in bytes.
+       */
+      std::size_t
+      memory_consumption() const;
+
+      /**
+       * Encodes the type of element detected at construction. FEEvaluation
+       * will select the most efficient algorithm based on the given element
+       * type.
+       */
+      ElementType element_type;
+
+      /**
+       * Stores the shape values of the 1D finite element evaluated on all 1D
+       * quadrature points in vectorized format, i.e., as an array of
+       * VectorizedArray<dim>::size equal elements. The length of
+       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
+       * points are the index running fastest.
+       */
+      AlignedVector<Number> shape_values;
+
+      /**
+       * Stores the shape gradients of the 1D finite element evaluated on all
+       * 1D quadrature points in vectorized format, i.e., as an array of
+       * VectorizedArray<dim>::size equal elements. The length of
+       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
+       * points are the index running fastest.
+       */
+      AlignedVector<Number> shape_gradients;
+
+      /**
+       * Stores the shape Hessians of the 1D finite element evaluated on all
+       * 1D quadrature points in vectorized format, i.e., as an array of
+       * VectorizedArray<dim>::size equal elements. The length of
+       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
+       * points are the index running fastest.
+       */
+      AlignedVector<Number> shape_hessians;
+
+      /**
+       * Stores the shape gradients of the shape function space associated to
+       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>).
+       */
+      AlignedVector<Number> shape_gradients_collocation;
+
+      /**
+       * Stores the shape hessians of the shape function space associated to
+       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>).
+       */
+      AlignedVector<Number> shape_hessians_collocation;
+
+      /**
+       * Stores the shape values in a different format, namely the so-called
+       * even-odd scheme where the symmetries in shape_values are used for
+       * faster evaluation.
+       */
+      AlignedVector<Number> shape_values_eo;
+
+      /**
+       * Stores the shape gradients in a different format, namely the so-
+       * called even-odd scheme where the symmetries in shape_gradients are
+       * used for faster evaluation.
+       */
+      AlignedVector<Number> shape_gradients_eo;
+
+      /**
+       * Stores the shape second derivatives in a different format, namely the
+       * so-called even-odd scheme where the symmetries in shape_hessians are
+       * used for faster evaluation.
+       */
+      AlignedVector<Number> shape_hessians_eo;
+
+      /**
+       * Stores the shape gradients of the shape function space associated to
+       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>). This
+       * array provides an alternative representation of the
+       * shape_gradients_collocation field in the even-odd format.
+       */
+      AlignedVector<Number> shape_gradients_collocation_eo;
+
+      /**
+       * Stores the shape hessians of the shape function space associated to
+       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>). This
+       * array provides an alternative representation of the
+       * shape_hessians_collocation field in the even-odd format.
+       */
+      AlignedVector<Number> shape_hessians_collocation_eo;
+
+      /**
+       * Stores the inverse transformation from the data at quadrature points
+       * to the basis defined by the shape_values fields. The data at
+       * quadrature points is interpreted either implicitly by its polynomial
+       * interpolation, or explicitly in terms of separate polynomials such as
+       * with the `_collocation` fields. The size of the array equals the
+       * layout of the `shape_values` array, and it is combined with the shape
+       * values array such that this matrix is the pseudo inverse of
+       * shape_values. In case the number of 1D quadrature points equals the
+       * size of the basis, this array is exactly the inverse of the
+       * shape_values array. The length of this array is <tt>n_dofs_1d *
+       * n_q_points_1d</tt> and quadrature points are the index running
+       * fastest.
+       */
+      AlignedVector<Number> inverse_shape_values;
+
+      /**
+       * Stores the even-odd variant of the `inverse_shape_values` field.
+       */
+      AlignedVector<Number> inverse_shape_values_eo;
+
+      /**
+       * Collects all data of 1D shape values evaluated at the point 0 and 1
+       * (the vertices) in one data structure. Sorting is first the values,
+       * then gradients, then second derivatives.
+       */
+      AlignedVector<Number> shape_data_on_face[2];
+
+      /**
+       * Stores one-dimensional values of shape functions on subface. Since
+       * there are two subfaces, store two variants.
+       */
+      AlignedVector<Number> values_within_subface[2];
+
+      /**
+       * Stores one-dimensional gradients of shape functions on subface. Since
+       * there are two subfaces, store two variants.
+       */
+      AlignedVector<Number> gradients_within_subface[2];
+
+      /**
+       * Stores one-dimensional gradients of shape functions on subface. Since
+       * there are two subfaces, store two variants.
+       */
+      AlignedVector<Number> hessians_within_subface[2];
+
+      /**
+       * We store a copy of the one-dimensional quadrature formula
+       * used for initialization.
+       */
+      Quadrature<1> quadrature;
+
+      /**
+       * Stores the degree of the element.
+       */
+      unsigned int fe_degree;
+
+      /**
+       * Stores the number of quadrature points per dimension.
+       */
+      unsigned int n_q_points_1d;
+
+      /**
+       * Indicates whether the basis functions are nodal in 0 and 1, i.e., the
+       * end points of the unit cell.
+       */
+      bool nodal_at_cell_boundaries;
+    };
+
+
+
+    /**
+     * This struct stores a tensor (Kronecker) product view of the finite
+     * element and quadrature formula used for evaluation. It is based on a
+     * single or a collection of UnivariateShapeData object(s) that describe
+     * one-dimensional ingredients, plus some additional information about how
+     * these are combined and how indices are laid out in the multi-dimensional
+     * case such as the hierarchical -> lexicographic ordering of FE_Q.
      *
      * @ingroup matrixfree
      *
-     * @author Katharina Kormann and Martin Kronbichler, 2010, 2011
+     * @author Katharina Kormann, Martin Kronbichler, Julius Witte, 2010-2020
      */
     template <typename Number>
     struct ShapeInfo
     {
+      /**
+       * Encodes the type of element detected at construction. FEEvaluation
+       * will select the most efficient algorithm based on the given element
+       * type.
+       */
+      ElementType element_type;
+
       /**
        * Empty constructor. Does nothing.
        */
@@ -132,106 +323,22 @@ namespace internal
              const FiniteElement<dim> &fe_dim,
              const unsigned int        base_element = 0);
 
+      /**
+       * Return data of univariate shape functions which defines the
+       * dimension @p dimension of tensor product shape functions
+       * regarding vector component @p component of the underlying
+       * finite element.
+       */
+      const UnivariateShapeData<Number> &
+      get_shape_data(const unsigned int dimension = 0,
+                     const unsigned int component = 0) const;
+
       /**
        * Return the memory consumption of this class in bytes.
        */
       std::size_t
       memory_consumption() const;
 
-      /**
-       * Encodes the type of element detected at construction. FEEvaluation
-       * will select the most efficient algorithm based on the given element
-       * type.
-       */
-      ElementType element_type;
-
-      /**
-       * Stores the shape values of the 1D finite element evaluated on all 1D
-       * quadrature points in vectorized format, i.e., as an array of
-       * VectorizedArray<dim>::n_array_elements equal elements. The length of
-       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
-       * points are the index running fastest.
-       */
-      AlignedVector<Number> shape_values;
-
-      /**
-       * Stores the shape gradients of the 1D finite element evaluated on all
-       * 1D quadrature points in vectorized format, i.e., as an array of
-       * VectorizedArray<dim>::n_array_elements equal elements. The length of
-       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
-       * points are the index running fastest.
-       */
-      AlignedVector<Number> shape_gradients;
-
-      /**
-       * Stores the shape Hessians of the 1D finite element evaluated on all
-       * 1D quadrature points in vectorized format, i.e., as an array of
-       * VectorizedArray<dim>::n_array_elements equal elements. The length of
-       * this array is <tt>n_dofs_1d * n_q_points_1d</tt> and quadrature
-       * points are the index running fastest.
-       */
-      AlignedVector<Number> shape_hessians;
-
-      /**
-       * Stores the shape values in a different format, namely the so-called
-       * even-odd scheme where the symmetries in shape_values are used for
-       * faster evaluation.
-       */
-      AlignedVector<Number> shape_values_eo;
-
-      /**
-       * Stores the shape gradients in a different format, namely the so-
-       * called even-odd scheme where the symmetries in shape_gradients are
-       * used for faster evaluation.
-       */
-      AlignedVector<Number> shape_gradients_eo;
-
-      /**
-       * Stores the shape second derivatives in a different format, namely the
-       * so-called even-odd scheme where the symmetries in shape_hessians are
-       * used for faster evaluation.
-       */
-      AlignedVector<Number> shape_hessians_eo;
-
-      /**
-       * Stores the shape gradients of the shape function space associated to
-       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>). For
-       * faster evaluation only the even-odd format is necessary.
-       */
-      AlignedVector<Number> shape_gradients_collocation_eo;
-
-      /**
-       * Stores the shape hessians of the shape function space associated to
-       * the quadrature (collocation), given by FE_DGQ<1>(Quadrature<1>). For
-       * faster evaluation only the even-odd format is necessary.
-       */
-      AlignedVector<Number> shape_hessians_collocation_eo;
-
-      /**
-       * Collects all data of 1D shape values evaluated at the point 0 and 1
-       * (the vertices) in one data structure. Sorting is first the values,
-       * then gradients, then second derivatives.
-       */
-      AlignedVector<Number> shape_data_on_face[2];
-
-      /**
-       * Stores one-dimensional values of shape functions on subface. Since
-       * there are two subfaces, store two variants.
-       */
-      AlignedVector<Number> values_within_subface[2];
-
-      /**
-       * Stores one-dimensional gradients of shape functions on subface. Since
-       * there are two subfaces, store two variants.
-       */
-      AlignedVector<Number> gradients_within_subface[2];
-
-      /**
-       * Stores one-dimensional gradients of shape functions on subface. Since
-       * there are two subfaces, store two variants.
-       */
-      AlignedVector<Number> hessians_within_subface[2];
-
       /**
        * Renumbering from deal.II's numbering of cell degrees of freedom to
        * lexicographic numbering used inside the FEEvaluation schemes of the
@@ -242,14 +349,28 @@ namespace internal
       std::vector<unsigned int> lexicographic_numbering;
 
       /**
-       * Stores the degree of the element.
+       * Stores data of univariate shape functions defining the
+       * underlying tensor product finite element.
        */
-      unsigned int fe_degree;
+      std::vector<UnivariateShapeData<Number>> data;
 
       /**
-       * Stores the number of quadrature points per dimension.
+       * Grants access to univariate shape function data of given
+       * dimension and vector component. Rows identify dimensions and
+       * columns identify vector components.
        */
-      unsigned int n_q_points_1d;
+      dealii::Table<2, UnivariateShapeData<Number> *> data_access;
+
+      /**
+       * Stores the number of space dimensions.
+       */
+      unsigned int n_dimensions;
+
+      /**
+       * Stores the number of vector components of the underlying
+       * vector-valued finite element.
+       */
+      unsigned int n_components;
 
       /**
        * Stores the number of quadrature points in @p dim dimensions for a
@@ -273,12 +394,6 @@ namespace internal
        */
       unsigned int dofs_per_component_on_face;
 
-      /**
-       * Indicates whether the basis functions are nodal in 0 and 1, i.e., the
-       * end points of the unit cell.
-       */
-      bool nodal_at_cell_boundaries;
-
       /**
        * For nodal basis functions with nodes located at the boundary of the
        * unit cell, face integrals that involve only the values of the shape
@@ -356,12 +471,14 @@ namespace internal
        */
       dealii::Table<2, unsigned int> face_to_cell_index_hermite;
 
+    private:
       /**
        * Check whether we have symmetries in the shape values. In that case,
        * also fill the shape_???_eo fields.
        */
       bool
-      check_1d_shapes_symmetric(const unsigned int n_q_points_1d);
+      check_1d_shapes_symmetric(
+        UnivariateShapeData<Number> &univariate_shape_data);
 
       /**
        * Check whether symmetric 1D basis functions are such that the shape
@@ -370,7 +487,8 @@ namespace internal
        * that save some operations in the evaluation.
        */
       bool
-      check_1d_shapes_collocation();
+      check_1d_shapes_collocation(
+        const UnivariateShapeData<Number> &univariate_shape_data) const;
     };
 
 
@@ -383,17 +501,28 @@ namespace internal
                                         const FiniteElement<dim> &fe_in,
                                         const unsigned int base_element_number)
       : element_type(tensor_general)
-      , fe_degree(0)
-      , n_q_points_1d(0)
+      , n_dimensions(0)
+      , n_components(0)
       , n_q_points(0)
       , dofs_per_component_on_cell(0)
       , n_q_points_face(0)
       , dofs_per_component_on_face(0)
-      , nodal_at_cell_boundaries(false)
     {
       reinit(quad, fe_in, base_element_number);
     }
 
+    template <typename Number>
+    inline const UnivariateShapeData<Number> &
+    ShapeInfo<Number>::get_shape_data(const unsigned int dimension,
+                                      const unsigned int component) const
+    {
+      AssertDimension(n_dimensions, data_access.size(0));
+      AssertDimension(n_components, data_access.size(1));
+      AssertIndexRange(dimension, n_dimensions);
+      AssertIndexRange(component, n_components);
+      return *(data_access(dimension, component));
+    }
+
   } // end of namespace MatrixFreeFunctions
 
 } // end of namespace internal
