@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2011 - 2019 by the deal.II authors
+// Copyright (C) 2011 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 #define dealii_matrix_free_shape_info_templates_h
 
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/base/memory_consumption.h>
 #include <deal.II/base/polynomial.h>
 #include <deal.II/base/polynomials_piecewise.h>
@@ -28,6 +30,8 @@
 #include <deal.II/fe/fe_poly.h>
 #include <deal.II/fe/fe_q_dg0.h>
 
+#include <deal.II/lac/householder.h>
+
 #include <deal.II/matrix_free/shape_info.h>
 
 
@@ -38,6 +42,16 @@ namespace internal
 {
   namespace MatrixFreeFunctions
   {
+    template <typename Number>
+    UnivariateShapeData<Number>::UnivariateShapeData()
+      : element_type(tensor_general)
+      , fe_degree(0)
+      , n_q_points_1d(0)
+      , nodal_at_cell_boundaries(false)
+    {}
+
+
+
     // ----------------- actual ShapeInfo functions --------------------
 
     template <typename Number>
@@ -47,9 +61,9 @@ namespace internal
       return a;
     }
 
-    template <typename Number>
+    template <typename Number, std::size_t width>
     Number
-    get_first_array_element(const VectorizedArray<Number> a)
+    get_first_array_element(const VectorizedArray<Number, width> a)
     {
       return a[0];
     }
@@ -57,13 +71,12 @@ namespace internal
     template <typename Number>
     ShapeInfo<Number>::ShapeInfo()
       : element_type(tensor_general)
-      , fe_degree(numbers::invalid_unsigned_int)
-      , n_q_points_1d(0)
+      , n_dimensions(0)
+      , n_components(0)
       , n_q_points(0)
       , dofs_per_component_on_cell(0)
       , n_q_points_face(0)
       , dofs_per_component_on_face(0)
-      , nodal_at_cell_boundaries(false)
     {}
 
 
@@ -76,13 +89,41 @@ namespace internal
                               const unsigned int        base_element_number)
     {
       const FiniteElement<dim> *fe = &fe_in.base_element(base_element_number);
+      n_dimensions                 = dim;
+      n_components                 = fe_in.n_components();
 
       Assert(fe->n_components() == 1,
              ExcMessage("FEEvaluation only works for scalar finite elements."));
 
-      fe_degree     = fe->degree;
-      n_q_points_1d = quad.size();
+      // assuming isotropy of dimensions and components
+      data.resize(1);
+      UnivariateShapeData<Number> &univariate_shape_data = data.front();
+      data_access.reinit(n_dimensions, n_components);
+      data_access.fill(&univariate_shape_data);
+      univariate_shape_data.quadrature    = quad;
+      univariate_shape_data.fe_degree     = fe->degree;
+      univariate_shape_data.n_q_points_1d = quad.size();
 
+      // grant write access to common univariate shape data
+      auto &shape_values    = univariate_shape_data.shape_values;
+      auto &shape_gradients = univariate_shape_data.shape_gradients;
+      auto &shape_hessians  = univariate_shape_data.shape_hessians;
+      auto &shape_gradients_collocation =
+        univariate_shape_data.shape_gradients_collocation;
+      auto &shape_hessians_collocation =
+        univariate_shape_data.shape_hessians_collocation;
+      auto &inverse_shape_values  = univariate_shape_data.inverse_shape_values;
+      auto &shape_data_on_face    = univariate_shape_data.shape_data_on_face;
+      auto &values_within_subface = univariate_shape_data.values_within_subface;
+      auto &gradients_within_subface =
+        univariate_shape_data.gradients_within_subface;
+      auto &hessians_within_subface =
+        univariate_shape_data.hessians_within_subface;
+      auto &nodal_at_cell_boundaries =
+        univariate_shape_data.nodal_at_cell_boundaries;
+
+      const unsigned int fe_degree     = fe->degree;
+      const unsigned int n_q_points_1d = quad.size();
       const unsigned int n_dofs_1d = std::min(fe->dofs_per_cell, fe_degree + 1);
 
       // renumber (this is necessary for FE_Q, for example, since there the
@@ -200,18 +241,18 @@ namespace internal
         dim > 1 ? Utilities::fixed_power<dim - 1>(fe_degree + 1) : 1;
 
       const unsigned int array_size = n_dofs_1d * n_q_points_1d;
-      this->shape_gradients.resize_fast(array_size);
-      this->shape_values.resize_fast(array_size);
-      this->shape_hessians.resize_fast(array_size);
+      shape_gradients.resize_fast(array_size);
+      shape_values.resize_fast(array_size);
+      shape_hessians.resize_fast(array_size);
 
-      this->shape_data_on_face[0].resize(3 * n_dofs_1d);
-      this->shape_data_on_face[1].resize(3 * n_dofs_1d);
-      this->values_within_subface[0].resize(array_size);
-      this->values_within_subface[1].resize(array_size);
-      this->gradients_within_subface[0].resize(array_size);
-      this->gradients_within_subface[1].resize(array_size);
-      this->hessians_within_subface[0].resize(array_size);
-      this->hessians_within_subface[1].resize(array_size);
+      shape_data_on_face[0].resize(3 * n_dofs_1d);
+      shape_data_on_face[1].resize(3 * n_dofs_1d);
+      values_within_subface[0].resize(array_size);
+      values_within_subface[1].resize(array_size);
+      gradients_within_subface[0].resize(array_size);
+      gradients_within_subface[1].resize(array_size);
+      hessians_within_subface[0].resize(array_size);
+      hessians_within_subface[1].resize(array_size);
 
       for (unsigned int i = 0; i < n_dofs_1d; ++i)
         {
@@ -250,18 +291,18 @@ namespace internal
             }
 
           // evaluate basis functions on the 1D faces, i.e., in zero and one
-          Point<dim> q_point             = unit_point;
-          q_point[0]                     = 0;
-          this->shape_data_on_face[0][i] = fe->shape_value(my_i, q_point);
-          this->shape_data_on_face[0][i + n_dofs_1d] =
+          Point<dim> q_point       = unit_point;
+          q_point[0]               = 0;
+          shape_data_on_face[0][i] = fe->shape_value(my_i, q_point);
+          shape_data_on_face[0][i + n_dofs_1d] =
             fe->shape_grad(my_i, q_point)[0];
-          this->shape_data_on_face[0][i + 2 * n_dofs_1d] =
+          shape_data_on_face[0][i + 2 * n_dofs_1d] =
             fe->shape_grad_grad(my_i, q_point)[0][0];
-          q_point[0]                     = 1;
-          this->shape_data_on_face[1][i] = fe->shape_value(my_i, q_point);
-          this->shape_data_on_face[1][i + n_dofs_1d] =
+          q_point[0]               = 1;
+          shape_data_on_face[1][i] = fe->shape_value(my_i, q_point);
+          shape_data_on_face[1][i + n_dofs_1d] =
             fe->shape_grad(my_i, q_point)[0];
-          this->shape_data_on_face[1][i + 2 * n_dofs_1d] =
+          shape_data_on_face[1][i + 2 * n_dofs_1d] =
             fe->shape_grad_grad(my_i, q_point)[0][0];
         }
 
@@ -272,50 +313,123 @@ namespace internal
       // FE_DGQArbitraryNodes underflow.
       if (n_q_points_1d < 200)
         {
-          const unsigned int stride = (n_q_points_1d + 1) / 2;
-          shape_gradients_collocation_eo.resize(n_q_points_1d * stride);
-          shape_hessians_collocation_eo.resize(n_q_points_1d * stride);
-          FE_DGQArbitraryNodes<1> fe(quad.get_points());
-          for (unsigned int i = 0; i < n_q_points_1d / 2; ++i)
-            for (unsigned int q = 0; q < stride; ++q)
+          shape_gradients_collocation.resize(n_q_points_1d * n_q_points_1d);
+          shape_hessians_collocation.resize(n_q_points_1d * n_q_points_1d);
+          FE_DGQArbitraryNodes<1> fe_coll(quad.get_points());
+          for (unsigned int i = 0; i < n_q_points_1d; ++i)
+            for (unsigned int q = 0; q < n_q_points_1d; ++q)
               {
-                shape_gradients_collocation_eo[i * stride + q] =
-                  0.5 *
-                  (fe.shape_grad(i, quad.get_points()[q])[0] +
-                   fe.shape_grad(i,
-                                 quad.get_points()[n_q_points_1d - 1 - q])[0]);
-                shape_gradients_collocation_eo[(n_q_points_1d - 1 - i) *
-                                                 stride +
-                                               q] =
-                  0.5 *
-                  (fe.shape_grad(i, quad.get_points()[q])[0] -
-                   fe.shape_grad(i,
-                                 quad.get_points()[n_q_points_1d - 1 - q])[0]);
-                shape_hessians_collocation_eo[i * stride + q] =
-                  0.5 * (fe.shape_grad_grad(i, quad.get_points()[q])[0][0] +
-                         fe.shape_grad_grad(
-                           i, quad.get_points()[n_q_points_1d - 1 - q])[0][0]);
-                shape_hessians_collocation_eo[(n_q_points_1d - 1 - i) * stride +
-                                              q] =
-                  0.5 * (fe.shape_grad_grad(i, quad.get_points()[q])[0][0] -
-                         fe.shape_grad_grad(
-                           i, quad.get_points()[n_q_points_1d - 1 - q])[0][0]);
-              }
-          if (n_q_points_1d % 2 == 1)
-            for (unsigned int q = 0; q < stride; ++q)
-              {
-                shape_gradients_collocation_eo[n_q_points_1d / 2 * stride + q] =
-                  fe.shape_grad(n_q_points_1d / 2, quad.get_points()[q])[0];
-                shape_hessians_collocation_eo[n_q_points_1d / 2 * stride + q] =
-                  fe.shape_grad_grad(n_q_points_1d / 2,
-                                     quad.get_points()[q])[0][0];
+                shape_gradients_collocation[i * n_q_points_1d + q] =
+                  fe_coll.shape_grad(i, quad.get_points()[q])[0];
+                shape_hessians_collocation[i * n_q_points_1d + q] =
+                  fe_coll.shape_grad_grad(i, quad.get_points()[q])[0][0];
               }
+
+          // compute the inverse shape functions in three steps: we first
+          // change from the given quadrature formula and the associated
+          // Lagrange polynomials to the Lagrange polynomials at quadrature
+          // points. in this basis, we can then perform the second step, which
+          // is the computation of a projection matrix from the potentially
+          // higher polynomial degree associated to the quadrature points to a
+          // polynomial space of degree equal to the degree of the given
+          // elements. in the third step, we change from the Lagrange
+          // polynomials in the Gauss quadrature points to the polynomial
+          // space of the given element
+
+          // step 1: change basis from the Lagrange polynomials at the given
+          // quadrature points to the Lagrange basis at Gauss quadrature
+          // points. this is often the identity operation as we often compute
+          // with Gaussian quadrature, but not necessarily so
+          QGauss<1>          quad_gauss(n_q_points_1d);
+          FullMatrix<double> transform_to_gauss(n_q_points_1d, n_q_points_1d);
+          for (unsigned int i = 0; i < n_q_points_1d; ++i)
+            for (unsigned int j = 0; j < n_q_points_1d; ++j)
+              transform_to_gauss(i, j) =
+                fe_coll.shape_value(j, quad_gauss.point(i));
+
+          // step 2: computation for the projection (in reference coordinates)
+          // from higher to lower polynomial degree
+          //
+          // loop over quadrature points, multiply by q-weight on high degree
+          // integrate loop going from high degree to low degree loop over new
+          // points, multiply by inverse q-weight on low degree
+          //
+          // This projection step is for the special case of Lagrange
+          // polynomials where most of the interpolation matrices are unit
+          // matrices when applying the inverse mass matrix, so we do not need
+          // to compute much.
+          QGauss<1>               quad_project(n_dofs_1d);
+          FE_DGQArbitraryNodes<1> fe_project(quad_project.get_points());
+
+          FullMatrix<double> project_gauss(n_dofs_1d, n_q_points_1d);
+
+          for (unsigned int i = 0; i < n_dofs_1d; ++i)
+            for (unsigned int q = 0; q < n_q_points_1d; ++q)
+              project_gauss(i, q) =
+                fe_project.shape_value(i, quad_gauss.get_points()[q]) *
+                (quad_gauss.weight(q) / quad_project.weight(i));
+          FullMatrix<double> project_to_dof_space(n_dofs_1d, n_q_points_1d);
+          project_gauss.mmult(project_to_dof_space, transform_to_gauss);
+
+          // step 3: change the basis back to the given finite element
+          // space. we can use a shortcut for elements that define support
+          // points, in which case we can evaluate the Lagrange polynomials of
+          // the Gauss quadrature in those points. this will give more
+          // accurate results than the inversion of a matrix. for more general
+          // polynomial spaces, we must invert a matrix of a Vandermonde type,
+          // which we do by a Householder transformation to keep roundoff
+          // errors low.
+          inverse_shape_values.resize_fast(array_size);
+          FullMatrix<double> transform_from_gauss(n_dofs_1d, n_dofs_1d);
+          if (fe->has_support_points())
+            {
+              for (unsigned int i = 0; i < n_dofs_1d; ++i)
+                for (unsigned int j = 0; j < n_dofs_1d; ++j)
+                  transform_from_gauss(i, j) = fe_project.shape_value(
+                    j,
+                    Point<1>(
+                      fe->get_unit_support_points()[scalar_lexicographic[i]]
+                                                   [0]));
+              FullMatrix<double> result(n_dofs_1d, n_q_points_1d);
+              transform_from_gauss.mmult(result, project_to_dof_space);
+
+              // set very small entries to zero - we are in reference space
+              // with normalized numbers, so this is straight-forward to check
+              // here
+              for (unsigned int i = 0; i < n_dofs_1d; ++i)
+                for (unsigned int q = 0; q < n_q_points_1d; ++q)
+                  inverse_shape_values[i * n_q_points_1d + q] =
+                    std::abs(result(i, q)) < 1e-15 ? 0 : result(i, q);
+            }
+          else
+            {
+              for (unsigned int i = 0; i < n_dofs_1d; ++i)
+                for (unsigned int j = 0; j < n_dofs_1d; ++j)
+                  {
+                    Point<dim> q_point = unit_point;
+                    q_point[0]         = quad_project.point(i)[0];
+
+                    transform_from_gauss(i, j) =
+                      fe->shape_value(scalar_lexicographic[j], q_point);
+                  }
+              Householder<double> H(transform_from_gauss);
+              Vector<double>      in(n_dofs_1d), out(n_dofs_1d);
+              for (unsigned int q = 0; q < n_q_points_1d; ++q)
+                {
+                  for (unsigned int i = 0; i < n_dofs_1d; ++i)
+                    in(i) = project_to_dof_space(i, q);
+                  H.least_squares(out, in);
+                  for (unsigned int i = 0; i < n_dofs_1d; ++i)
+                    inverse_shape_values[i * n_q_points_1d + q] =
+                      std::abs(out(i)) < 1e-15 ? 0. : out(i);
+                }
+            }
         }
 
       if (element_type == tensor_general &&
-          check_1d_shapes_symmetric(n_q_points_1d))
+          check_1d_shapes_symmetric(univariate_shape_data))
         {
-          if (check_1d_shapes_collocation())
+          if (check_1d_shapes_collocation(univariate_shape_data))
             element_type = tensor_symmetric_collocation;
           else
             element_type = tensor_symmetric;
@@ -334,7 +448,7 @@ namespace internal
             }
         }
       else if (element_type == tensor_symmetric_plus_dg0)
-        check_1d_shapes_symmetric(n_q_points_1d);
+        check_1d_shapes_symmetric(univariate_shape_data);
 
       nodal_at_cell_boundaries = true;
       for (unsigned int i = 1; i < n_dofs_1d; ++i)
@@ -348,7 +462,7 @@ namespace internal
         {
           face_to_cell_index_nodal.reinit(GeometryInfo<dim>::faces_per_cell,
                                           dofs_per_component_on_face);
-          for (unsigned int f = 0; f < GeometryInfo<dim>::faces_per_cell; ++f)
+          for (auto f : GeometryInfo<dim>::face_indices())
             {
               const unsigned int direction = f / 2;
               const unsigned int stride =
@@ -381,7 +495,7 @@ namespace internal
         {
           face_to_cell_index_hermite.reinit(GeometryInfo<dim>::faces_per_cell,
                                             2 * dofs_per_component_on_face);
-          for (unsigned int f = 0; f < GeometryInfo<dim>::faces_per_cell; ++f)
+          for (auto f : GeometryInfo<dim>::face_indices())
             {
               const unsigned int direction = f / 2;
               const unsigned int stride =
@@ -416,6 +530,8 @@ namespace internal
                     }
             }
         }
+
+      univariate_shape_data.element_type = this->element_type;
     }
 
 
@@ -423,11 +539,31 @@ namespace internal
     template <typename Number>
     bool
     ShapeInfo<Number>::check_1d_shapes_symmetric(
-      const unsigned int n_q_points_1d)
+      UnivariateShapeData<Number> &univariate_shape_data)
     {
       if (dofs_per_component_on_cell == 0)
         return false;
 
+      const auto n_q_points_1d   = univariate_shape_data.n_q_points_1d;
+      const auto fe_degree       = univariate_shape_data.fe_degree;
+      auto &     shape_values    = univariate_shape_data.shape_values;
+      auto &     shape_gradients = univariate_shape_data.shape_gradients;
+      auto &     shape_hessians  = univariate_shape_data.shape_hessians;
+      auto &     shape_gradients_collocation =
+        univariate_shape_data.shape_gradients_collocation;
+      auto &shape_hessians_collocation =
+        univariate_shape_data.shape_hessians_collocation;
+      auto &shape_values_eo    = univariate_shape_data.shape_values_eo;
+      auto &shape_gradients_eo = univariate_shape_data.shape_gradients_eo;
+      auto &shape_hessians_eo  = univariate_shape_data.shape_hessians_eo;
+      auto &shape_gradients_collocation_eo =
+        univariate_shape_data.shape_gradients_collocation_eo;
+      auto &shape_hessians_collocation_eo =
+        univariate_shape_data.shape_hessians_collocation_eo;
+      auto &inverse_shape_values = univariate_shape_data.inverse_shape_values;
+      auto &inverse_shape_values_eo =
+        univariate_shape_data.inverse_shape_values_eo;
+
       const double zero_tol =
         std::is_same<Number, double>::value == true ? 1e-12 : 1e-7;
       // symmetry for values
@@ -443,8 +579,8 @@ namespace internal
             return false;
 
       // shape values should be zero at x=0.5 for all basis functions except
-      // for the middle one
-      if (n_q_points_1d % 2 == 1 && n_dofs_1d % 2 == 1)
+      // for the middle one for degrees of 4 and higher
+      if (n_dofs_1d > 3 && n_q_points_1d % 2 == 1 && n_dofs_1d % 2 == 1)
         {
           for (unsigned int i = 0; i < n_dofs_1d / 2; ++i)
             if (std::abs(get_first_array_element(
@@ -483,46 +619,53 @@ namespace internal
               zero_tol_hessian)
             return false;
 
-      const unsigned int stride = (n_q_points_1d + 1) / 2;
-      shape_values_eo.resize((fe_degree + 1) * stride);
-      shape_gradients_eo.resize((fe_degree + 1) * stride);
-      shape_hessians_eo.resize((fe_degree + 1) * stride);
-      for (unsigned int i = 0; i < (fe_degree + 1) / 2; ++i)
-        for (unsigned int q = 0; q < stride; ++q)
-          {
-            shape_values_eo[i * stride + q] =
-              0.5 * (shape_values[i * n_q_points_1d + q] +
-                     shape_values[i * n_q_points_1d + n_q_points_1d - 1 - q]);
-            shape_values_eo[(fe_degree - i) * stride + q] =
-              0.5 * (shape_values[i * n_q_points_1d + q] -
-                     shape_values[i * n_q_points_1d + n_q_points_1d - 1 - q]);
+      auto convert_to_eo = [](const AlignedVector<Number> &array,
+                              const unsigned               n_rows,
+                              const unsigned               n_cols) {
+        const unsigned int    stride = (n_cols + 1) / 2;
+        AlignedVector<Number> array_eo(n_rows * stride);
+        for (unsigned int i = 0; i < n_rows / 2; ++i)
+          for (unsigned int q = 0; q < stride; ++q)
+            {
+              array_eo[i * stride + q] =
+                0.5 *
+                (array[i * n_cols + q] + array[i * n_cols + n_cols - 1 - q]);
+              array_eo[(n_rows - 1 - i) * stride + q] =
+                0.5 *
+                (array[i * n_cols + q] - array[i * n_cols + n_cols - 1 - q]);
+            }
+        if ((n_rows - 1) % 2 == 0)
+          for (unsigned int q = 0; q < stride; ++q)
+            {
+              array_eo[(n_rows - 1) / 2 * stride + q] =
+                array[((n_rows - 1) / 2) * n_cols + q];
+            }
 
-            shape_gradients_eo[i * stride + q] =
-              0.5 *
-              (shape_gradients[i * n_q_points_1d + q] +
-               shape_gradients[i * n_q_points_1d + n_q_points_1d - 1 - q]);
-            shape_gradients_eo[(fe_degree - i) * stride + q] =
-              0.5 *
-              (shape_gradients[i * n_q_points_1d + q] -
-               shape_gradients[i * n_q_points_1d + n_q_points_1d - 1 - q]);
+        return array_eo;
+      };
 
-            shape_hessians_eo[i * stride + q] =
-              0.5 * (shape_hessians[i * n_q_points_1d + q] +
-                     shape_hessians[i * n_q_points_1d + n_q_points_1d - 1 - q]);
-            shape_hessians_eo[(fe_degree - i) * stride + q] =
-              0.5 * (shape_hessians[i * n_q_points_1d + q] -
-                     shape_hessians[i * n_q_points_1d + n_q_points_1d - 1 - q]);
-          }
-      if (fe_degree % 2 == 0)
-        for (unsigned int q = 0; q < stride; ++q)
-          {
-            shape_values_eo[fe_degree / 2 * stride + q] =
-              shape_values[(fe_degree / 2) * n_q_points_1d + q];
-            shape_gradients_eo[fe_degree / 2 * stride + q] =
-              shape_gradients[(fe_degree / 2) * n_q_points_1d + q];
-            shape_hessians_eo[fe_degree / 2 * stride + q] =
-              shape_hessians[(fe_degree / 2) * n_q_points_1d + q];
-          }
+      shape_values_eo =
+        convert_to_eo(shape_values, fe_degree + 1, n_q_points_1d);
+      shape_gradients_eo =
+        convert_to_eo(shape_gradients, fe_degree + 1, n_q_points_1d);
+      shape_hessians_eo =
+        convert_to_eo(shape_hessians, fe_degree + 1, n_q_points_1d);
+
+      // FE_DGQArbitraryNodes underflow (see also above where
+      // shape_gradients_collocation and shape_hessians_collocation is set up).
+      if (n_q_points_1d < 200)
+        {
+          shape_gradients_collocation_eo =
+            convert_to_eo(shape_gradients_collocation,
+                          n_q_points_1d,
+                          n_q_points_1d);
+          shape_hessians_collocation_eo =
+            convert_to_eo(shape_hessians_collocation,
+                          n_q_points_1d,
+                          n_q_points_1d);
+          inverse_shape_values_eo =
+            convert_to_eo(inverse_shape_values, fe_degree + 1, n_q_points_1d);
+        }
 
       return true;
     }
@@ -531,11 +674,15 @@ namespace internal
 
     template <typename Number>
     bool
-    ShapeInfo<Number>::check_1d_shapes_collocation()
+    ShapeInfo<Number>::check_1d_shapes_collocation(
+      const UnivariateShapeData<Number> &univariate_shape_data) const
     {
       if (dofs_per_component_on_cell != n_q_points)
         return false;
 
+      const auto fe_degree    = univariate_shape_data.fe_degree;
+      auto &     shape_values = univariate_shape_data.shape_values;
+
       const double zero_tol =
         std::is_same<Number, double>::value == true ? 1e-12 : 1e-7;
       // check: identity operation for shape values
@@ -563,11 +710,25 @@ namespace internal
     template <typename Number>
     std::size_t
     ShapeInfo<Number>::memory_consumption() const
+    {
+      std::size_t memory = sizeof(*this);
+      for (const auto &univariate_shape_data : data)
+        memory += univariate_shape_data.memory_consumption();
+      return memory;
+    }
+
+    template <typename Number>
+    std::size_t
+    UnivariateShapeData<Number>::memory_consumption() const
     {
       std::size_t memory = sizeof(*this);
       memory += MemoryConsumption::memory_consumption(shape_values);
       memory += MemoryConsumption::memory_consumption(shape_gradients);
       memory += MemoryConsumption::memory_consumption(shape_hessians);
+      memory +=
+        MemoryConsumption::memory_consumption(shape_gradients_collocation);
+      memory +=
+        MemoryConsumption::memory_consumption(shape_hessians_collocation);
       memory += MemoryConsumption::memory_consumption(shape_values_eo);
       memory += MemoryConsumption::memory_consumption(shape_gradients_eo);
       memory += MemoryConsumption::memory_consumption(shape_hessians_eo);
@@ -587,10 +748,9 @@ namespace internal
       return memory;
     }
 
-    // end of functions for ShapeInfo
+  } // namespace MatrixFreeFunctions
 
-  } // end of namespace MatrixFreeFunctions
-} // end of namespace internal
+} // namespace internal
 
 
 DEAL_II_NAMESPACE_CLOSE
