@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2006 - 2018 by the deal.II authors
+// Copyright (C) 2006 - 2019 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,10 +29,13 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace MeshWorker
 {
+  // Forward declarations
+#ifndef DOXYGEN
   template <int dim, int spacedim, typename number>
   class DoFInfo;
   template <int dim, int spacedim>
   class IntegrationInfo;
+#endif
 
   /**
    * A local integrator object, which can be used to simplify the call of
