@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2010 - 2019 by the deal.II authors
+// Copyright (C) 2010 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -29,8 +29,11 @@
 
 DEAL_II_NAMESPACE_OPEN
 
+// Forward declaration
+#ifndef DOXYGEN
 template <int dim, int spacedim>
 class DoFHandler;
+#endif
 
 
 /**
@@ -100,6 +103,24 @@ public:
     const std::set<types::boundary_id> &boundary_ids,
     const ComponentMask &               component_mask = ComponentMask());
 
+  /**
+   * Add user defined constraints to be used on level @p level.
+   *
+   * The user can call this function multiple times and any new,
+   * conflicting constraints will overwrite the previous constraints
+   * for that DoF.
+   *
+   * Before the transfer, the user defined constraints will be distributed
+   * to the source vector, and then any DoF index set using
+   * make_zero_boundary_constraints() will be overwritten with
+   * value zero.
+   *
+   * @note This is currently only implemented for MGTransferMatrixFree.
+   */
+  void
+  add_user_constraints(const unsigned int               level,
+                       const AffineConstraints<double> &constraints_on_level);
+
   /**
    * Fill the internal data structures with information
    * about no normal flux boundary dofs.
@@ -193,6 +214,16 @@ public:
   const AffineConstraints<double> &
   get_level_constraint_matrix(const unsigned int level) const;
 
+  /**
+   * Return the user defined constraint matrix for a given level. These
+   * constraints are set using the function add_user_constraints() and
+   * should not contain constraints for DoF indices set in
+   * make_zero_boundary_constraints() as they will be overwritten during
+   * the transfer.
+   */
+  const AffineConstraints<double> &
+  get_user_constraint_matrix(const unsigned int level) const;
+
 private:
   /**
    * The indices of boundary dofs for each level.
@@ -210,6 +241,11 @@ private:
    * periodic boundary conditions for each level .
    */
   std::vector<AffineConstraints<double>> level_constraints;
+
+  /**
+   * Constraint matrices defined by user.
+   */
+  std::vector<AffineConstraints<double>> user_constraints;
 };
 
 
@@ -220,12 +256,14 @@ MGConstrainedDoFs::initialize(const DoFHandler<dim, spacedim> &dof)
   boundary_indices.clear();
   refinement_edge_indices.clear();
   level_constraints.clear();
+  user_constraints.clear();
 
   const unsigned int nlevels = dof.get_triangulation().n_global_levels();
 
   // At this point level_constraint and refinement_edge_indices are empty.
-  level_constraints.resize(nlevels);
   refinement_edge_indices.resize(nlevels);
+  level_constraints.resize(nlevels);
+  user_constraints.resize(nlevels);
   for (unsigned int l = 0; l < nlevels; ++l)
     {
       IndexSet relevant_dofs;
@@ -239,7 +277,7 @@ MGConstrainedDoFs::initialize(const DoFHandler<dim, spacedim> &dof)
       for (; cell != endc; ++cell)
         if (cell->level_subdomain_id() != numbers::artificial_subdomain_id)
           {
-            for (unsigned int f = 0; f < GeometryInfo<dim>::faces_per_cell; ++f)
+            for (auto f : GeometryInfo<dim>::face_indices())
               if (cell->has_periodic_neighbor(f) &&
                   cell->periodic_neighbor(f)->level() == cell->level())
                 {
@@ -351,8 +389,7 @@ MGConstrainedDoFs::make_no_normal_flux_constraints(
   // For a given boundary id, find which vector component is on the boundary
   // and set a zero boundary constraint for those degrees of freedom.
   const unsigned int n_components = DoFTools::n_components(dof);
-  Assert(first_vector_component + dim <= n_components,
-         ExcIndexRange(first_vector_component, 0, n_components - dim + 1));
+  AssertIndexRange(first_vector_component + dim - 1, n_components);
 
   ComponentMask comp_mask(n_components, false);
 
@@ -396,11 +433,31 @@ MGConstrainedDoFs::make_no_normal_flux_constraints(
 }
 
 
+inline void
+MGConstrainedDoFs::add_user_constraints(
+  const unsigned int               level,
+  const AffineConstraints<double> &constraints_on_level)
+{
+  AssertIndexRange(level, user_constraints.size());
+
+  // Get the relevant DoFs from level_constraints if
+  // the user constraint matrix has not been initialized
+  if (user_constraints[level].get_local_lines().size() == 0)
+    user_constraints[level].reinit(level_constraints[level].get_local_lines());
+
+  user_constraints[level].merge(
+    constraints_on_level,
+    AffineConstraints<double>::MergeConflictBehavior::right_object_wins);
+  user_constraints[level].close();
+}
+
+
 inline void
 MGConstrainedDoFs::clear()
 {
   boundary_indices.clear();
   refinement_edge_indices.clear();
+  user_constraints.clear();
 }
 
 
@@ -484,6 +541,15 @@ MGConstrainedDoFs::get_level_constraint_matrix(const unsigned int level) const
 
 
 
+inline const AffineConstraints<double> &
+MGConstrainedDoFs::get_user_constraint_matrix(const unsigned int level) const
+{
+  AssertIndexRange(level, user_constraints.size());
+  return user_constraints[level];
+}
+
+
+
 DEAL_II_NAMESPACE_CLOSE
 
 #endif
