@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2001 - 2019 by the deal.II authors
+// Copyright (C) 2001 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -20,6 +20,8 @@
 
 #include <deal.II/base/mg_level_object.h>
 
+#include <deal.II/distributed/tria_base.h>
+
 #include <deal.II/dofs/dof_handler.h>
 
 #include <deal.II/lac/affine_constraints.h>
@@ -82,12 +84,11 @@ namespace internal
            const SparsityPatternType &sp,
            DoFHandlerType &           dh)
     {
-      const parallel::Triangulation<DoFHandlerType::dimension,
-                                    DoFHandlerType::space_dimension>
-        *dist_tria = dynamic_cast<
-          const parallel::Triangulation<DoFHandlerType::dimension,
-                                        DoFHandlerType::space_dimension> *>(
-          &(dh.get_triangulation()));
+      const dealii::parallel::TriangulationBase<DoFHandlerType::dimension,
+                                                DoFHandlerType::space_dimension>
+        *dist_tria = dynamic_cast<const dealii::parallel::TriangulationBase<
+          DoFHandlerType::dimension,
+          DoFHandlerType::space_dimension> *>(&(dh.get_triangulation()));
       MPI_Comm communicator =
         dist_tria != nullptr ? dist_tria->get_communicator() : MPI_COMM_SELF;
 
@@ -115,12 +116,11 @@ namespace internal
            const SparsityPatternType &sp,
            DoFHandlerType &           dh)
     {
-      const parallel::Triangulation<DoFHandlerType::dimension,
-                                    DoFHandlerType::space_dimension>
-        *dist_tria = dynamic_cast<
-          const parallel::Triangulation<DoFHandlerType::dimension,
-                                        DoFHandlerType::space_dimension> *>(
-          &(dh.get_triangulation()));
+      const dealii::parallel::TriangulationBase<DoFHandlerType::dimension,
+                                                DoFHandlerType::space_dimension>
+        *dist_tria = dynamic_cast<const dealii::parallel::TriangulationBase<
+          DoFHandlerType::dimension,
+          DoFHandlerType::space_dimension> *>(&(dh.get_triangulation()));
       MPI_Comm communicator =
         dist_tria != nullptr ? dist_tria->get_communicator() : MPI_COMM_SELF;
       matrix.reinit(dh.locally_owned_mg_dofs(level + 1),
@@ -149,12 +149,11 @@ namespace internal
            const SparsityPatternType &sp,
            DoFHandlerType &           dh)
     {
-      const parallel::Triangulation<DoFHandlerType::dimension,
-                                    DoFHandlerType::space_dimension>
-        *dist_tria = dynamic_cast<
-          const parallel::Triangulation<DoFHandlerType::dimension,
-                                        DoFHandlerType::space_dimension> *>(
-          &(dh.get_triangulation()));
+      const dealii::parallel::TriangulationBase<DoFHandlerType::dimension,
+                                                DoFHandlerType::space_dimension>
+        *dist_tria = dynamic_cast<const dealii::parallel::TriangulationBase<
+          DoFHandlerType::dimension,
+          DoFHandlerType::space_dimension> *>(&(dh.get_triangulation()));
       MPI_Comm communicator =
         dist_tria != nullptr ? dist_tria->get_communicator() : MPI_COMM_SELF;
       matrix.reinit(dh.locally_owned_mg_dofs(level + 1),
@@ -182,12 +181,11 @@ namespace internal
            const SparsityPatternType &sp,
            DoFHandlerType &           dh)
     {
-      const parallel::Triangulation<DoFHandlerType::dimension,
-                                    DoFHandlerType::space_dimension>
-        *dist_tria = dynamic_cast<
-          const parallel::Triangulation<DoFHandlerType::dimension,
-                                        DoFHandlerType::space_dimension> *>(
-          &(dh.get_triangulation()));
+      const dealii::parallel::TriangulationBase<DoFHandlerType::dimension,
+                                                DoFHandlerType::space_dimension>
+        *dist_tria = dynamic_cast<const dealii::parallel::TriangulationBase<
+          DoFHandlerType::dimension,
+          DoFHandlerType::space_dimension> *>(&(dh.get_triangulation()));
       MPI_Comm communicator =
         dist_tria != nullptr ? dist_tria->get_communicator() : MPI_COMM_SELF;
       matrix.reinit(dh.locally_owned_mg_dofs(level + 1),
@@ -244,12 +242,11 @@ namespace internal
            const SparsityPatternType &sp,
            const DoFHandlerType &     dh)
     {
-      const parallel::Triangulation<DoFHandlerType::dimension,
-                                    DoFHandlerType::space_dimension>
-        *dist_tria = dynamic_cast<
-          const parallel::Triangulation<DoFHandlerType::dimension,
-                                        DoFHandlerType::space_dimension> *>(
-          &(dh.get_triangulation()));
+      const dealii::parallel::TriangulationBase<DoFHandlerType::dimension,
+                                                DoFHandlerType::space_dimension>
+        *dist_tria = dynamic_cast<const dealii::parallel::TriangulationBase<
+          DoFHandlerType::dimension,
+          DoFHandlerType::space_dimension> *>(&(dh.get_triangulation()));
       MPI_Comm communicator =
         dist_tria != nullptr ? dist_tria->get_communicator() : MPI_COMM_SELF;
       // Reinit PETSc matrix
@@ -296,7 +293,7 @@ public:
    */
   template <int dim, class InVector, int spacedim>
   void
-  copy_to_mg(const DoFHandler<dim, spacedim> &mg_dof,
+  copy_to_mg(const DoFHandler<dim, spacedim> &dof_handler,
              MGLevelObject<VectorType> &      dst,
              const InVector &                 src) const;
 
@@ -309,7 +306,7 @@ public:
    */
   template <int dim, class OutVector, int spacedim>
   void
-  copy_from_mg(const DoFHandler<dim, spacedim> &mg_dof,
+  copy_from_mg(const DoFHandler<dim, spacedim> &dof_handler,
                OutVector &                      dst,
                const MGLevelObject<VectorType> &src) const;
 
@@ -320,7 +317,7 @@ public:
    */
   template <int dim, class OutVector, int spacedim>
   void
-  copy_from_mg_add(const DoFHandler<dim, spacedim> &mg_dof,
+  copy_from_mg_add(const DoFHandler<dim, spacedim> &dof_handler,
                    OutVector &                      dst,
                    const MGLevelObject<VectorType> &src) const;
 
@@ -360,7 +357,8 @@ protected:
    */
   template <int dim, int spacedim>
   void
-  fill_and_communicate_copy_indices(const DoFHandler<dim, spacedim> &mg_dof);
+  fill_and_communicate_copy_indices(
+    const DoFHandler<dim, spacedim> &dof_handler);
 
   /**
    * Sizes of the multi-level vectors.
@@ -419,6 +417,14 @@ protected:
    */
   SmartPointer<const MGConstrainedDoFs, MGLevelGlobalTransfer<VectorType>>
     mg_constrained_dofs;
+
+private:
+  /**
+   * This function is called to make sure that build() has been invoked.
+   */
+  template <int dim, int spacedim>
+  void
+  assert_built(const DoFHandler<dim, spacedim> &dof_handler) const;
 };
 
 
@@ -453,7 +459,7 @@ public:
    */
   template <int dim, typename Number2, int spacedim>
   void
-  copy_to_mg(const DoFHandler<dim, spacedim> &                          mg_dof,
+  copy_to_mg(const DoFHandler<dim, spacedim> &dof_handler,
              MGLevelObject<LinearAlgebra::distributed::Vector<Number>> &dst,
              const LinearAlgebra::distributed::Vector<Number2> &src) const;
 
@@ -467,8 +473,8 @@ public:
   template <int dim, typename Number2, int spacedim>
   void
   copy_from_mg(
-    const DoFHandler<dim, spacedim> &                                mg_dof,
-    LinearAlgebra::distributed::Vector<Number2> &                    dst,
+    const DoFHandler<dim, spacedim> &            dof_handler,
+    LinearAlgebra::distributed::Vector<Number2> &dst,
     const MGLevelObject<LinearAlgebra::distributed::Vector<Number>> &src) const;
 
   /**
@@ -479,8 +485,8 @@ public:
   template <int dim, typename Number2, int spacedim>
   void
   copy_from_mg_add(
-    const DoFHandler<dim, spacedim> &                                mg_dof,
-    LinearAlgebra::distributed::Vector<Number2> &                    dst,
+    const DoFHandler<dim, spacedim> &            dof_handler,
+    LinearAlgebra::distributed::Vector<Number2> &dst,
     const MGLevelObject<LinearAlgebra::distributed::Vector<Number>> &src) const;
 
   /**
@@ -520,7 +526,7 @@ protected:
    */
   template <int dim, typename Number2, int spacedim>
   void
-  copy_to_mg(const DoFHandler<dim, spacedim> &                          mg_dof,
+  copy_to_mg(const DoFHandler<dim, spacedim> &dof_handler,
              MGLevelObject<LinearAlgebra::distributed::Vector<Number>> &dst,
              const LinearAlgebra::distributed::Vector<Number2> &        src,
              const bool solution_transfer) const;
@@ -530,7 +536,8 @@ protected:
    */
   template <int dim, int spacedim>
   void
-  fill_and_communicate_copy_indices(const DoFHandler<dim, spacedim> &mg_dof);
+  fill_and_communicate_copy_indices(
+    const DoFHandler<dim, spacedim> &dof_handler);
 
   /**
    * Sizes of the multi-level vectors.
@@ -541,49 +548,44 @@ protected:
    * Mapping for the copy_to_mg() and copy_from_mg() functions. Here only
    * index pairs locally owned is stored.
    *
-   * The data is organized as follows: one vector per level. Each element of
-   * these vectors contains first the global index, then the level index.
+   * The data is organized as follows: one table per level. This table has two
+   * rows. The first row contains the global index, the second one the level
+   * index.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>> copy_indices;
-
+  std::vector<Table<2, unsigned int>> copy_indices;
 
   /**
    * Same as above, but used to transfer solution vectors.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>>
-    solution_copy_indices;
+  std::vector<Table<2, unsigned int>> solution_copy_indices;
 
   /**
    * Additional degrees of freedom for the copy_to_mg() function. These are
    * the ones where the global degree of freedom is locally owned and the
    * level degree of freedom is not.
    *
-   * Organization of the data is like for @p copy_indices_mine.
+   * Organization of the data is like for @p copy_indices.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>>
-    copy_indices_global_mine;
+  std::vector<Table<2, unsigned int>> copy_indices_global_mine;
 
   /**
    * Same as above, but used to transfer solution vectors.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>>
-    solution_copy_indices_global_mine;
+  std::vector<Table<2, unsigned int>> solution_copy_indices_global_mine;
 
   /**
    * Additional degrees of freedom for the copy_from_mg() function. These are
    * the ones where the level degree of freedom is locally owned and the
    * global degree of freedom is not.
    *
-   * Organization of the data is like for @p copy_indices_mine.
+   * Organization of the data is like for @p copy_indices.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>>
-    copy_indices_level_mine;
+  std::vector<Table<2, unsigned int>> copy_indices_level_mine;
 
   /**
    * Same as above, but used to transfer solution vectors.
    */
-  std::vector<std::vector<std::pair<unsigned int, unsigned int>>>
-    solution_copy_indices_level_mine;
+  std::vector<Table<2, unsigned int>> solution_copy_indices_level_mine;
 
   /**
    * This variable stores whether the copy operation from the global to the
@@ -640,6 +642,14 @@ protected:
    */
   mutable MGLevelObject<LinearAlgebra::distributed::Vector<Number>>
     solution_ghosted_level_vector;
+
+private:
+  /**
+   * This function is called to make sure that build() has been invoked.
+   */
+  template <int dim, int spacedim>
+  void
+  assert_built(const DoFHandler<dim, spacedim> &dof_handler) const;
 };
 
 
@@ -673,37 +683,17 @@ public:
    */
   MGTransferPrebuilt(const MGConstrainedDoFs &mg_constrained_dofs);
 
-  /**
-   * Constructor with constraints. Equivalent to the default constructor
-   * followed by initialize_constraints().
-   *
-   * @deprecated @p constraints is unused.
-   */
-  DEAL_II_DEPRECATED
-  MGTransferPrebuilt(const AffineConstraints<double> &constraints,
-                     const MGConstrainedDoFs &        mg_constrained_dofs);
-
   /**
    * Destructor.
    */
   virtual ~MGTransferPrebuilt() override = default;
 
   /**
-   * Initialize the constraints to be used in build_matrices().
+   * Initialize the constraints to be used in build().
    */
   void
   initialize_constraints(const MGConstrainedDoFs &mg_constrained_dofs);
 
-  /**
-   * Initialize the constraints to be used in build_matrices().
-   *
-   * @deprecated @p constraints is unused.
-   */
-  DEAL_II_DEPRECATED
-  void
-  initialize_constraints(const AffineConstraints<double> &constraints,
-                         const MGConstrainedDoFs &        mg_constrained_dofs);
-
   /**
    * Reset the object to the state it had right after the default constructor.
    */
@@ -711,11 +701,21 @@ public:
   clear();
 
   /**
-   * Actually build the prolongation matrices for each level.
+   * Actually build the information required for the transfer operations. Needs
+   * to be called before prolongate() or restrict_and_add() can be used.
    */
   template <int dim, int spacedim>
   void
-  build_matrices(const DoFHandler<dim, spacedim> &mg_dof);
+  build(const DoFHandler<dim, spacedim> &dof_handler);
+
+  /**
+   * Actually build the prolongation matrices for each level.
+   *
+   * @deprecated use build() instead.
+   */
+  template <int dim, int spacedim>
+  DEAL_II_DEPRECATED void
+  build_matrices(const DoFHandler<dim, spacedim> &dof_handler);
 
   /**
    * Prolongate a vector from level <tt>to_level-1</tt> to level
@@ -759,7 +759,7 @@ public:
   DeclException0(ExcNoProlongation);
 
   /**
-   * You have to call build_matrices() before using this object.
+   * You have to call build() before using this object.
    */
   DeclException0(ExcMatricesNotBuilt);
 
