@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2003 - 2018 by the deal.II authors
+// Copyright (C) 2003 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -17,6 +17,8 @@
 #ifndef dealii_mg_transfer_block_templates_h
 #define dealii_mg_transfer_block_templates_h
 
+#include <deal.II/base/config.h>
+
 #include <deal.II/dofs/dof_accessor.h>
 
 #include <deal.II/fe/fe.h>
@@ -44,12 +46,12 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlockSelect<number>::copy_from_mg(
-  const DoFHandler<dim, spacedim> &    mg_dof_handler,
+  const DoFHandler<dim, spacedim> &    dof_handler,
   BlockVector<number2> &               dst,
   const MGLevelObject<Vector<number>> &src) const
 {
   for (unsigned int level = 0;
-       level < mg_dof_handler.get_triangulation().n_levels();
+       level < dof_handler.get_triangulation().n_levels();
        ++level)
     for (IT i = copy_indices[selected_block][level].begin();
          i != copy_indices[selected_block][level].end();
@@ -63,12 +65,12 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlockSelect<number>::copy_from_mg(
-  const DoFHandler<dim, spacedim> &    mg_dof_handler,
+  const DoFHandler<dim, spacedim> &    dof_handler,
   Vector<number2> &                    dst,
   const MGLevelObject<Vector<number>> &src) const
 {
   for (unsigned int level = 0;
-       level < mg_dof_handler.get_triangulation().n_levels();
+       level < dof_handler.get_triangulation().n_levels();
        ++level)
     for (IT i = copy_indices[selected_block][level].begin();
          i != copy_indices[selected_block][level].end();
@@ -82,12 +84,12 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlockSelect<number>::copy_from_mg_add(
-  const DoFHandler<dim, spacedim> &    mg_dof_handler,
+  const DoFHandler<dim, spacedim> &    dof_handler,
   BlockVector<number2> &               dst,
   const MGLevelObject<Vector<number>> &src) const
 {
   for (unsigned int level = 0;
-       level < mg_dof_handler.get_triangulation().n_levels();
+       level < dof_handler.get_triangulation().n_levels();
        ++level)
     for (IT i = copy_indices[selected_block][level].begin();
          i != copy_indices[selected_block][level].end();
@@ -101,12 +103,12 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlockSelect<number>::copy_from_mg_add(
-  const DoFHandler<dim, spacedim> &    mg_dof_handler,
+  const DoFHandler<dim, spacedim> &    dof_handler,
   Vector<number2> &                    dst,
   const MGLevelObject<Vector<number>> &src) const
 {
   for (unsigned int level = 0;
-       level < mg_dof_handler.get_triangulation().n_levels();
+       level < dof_handler.get_triangulation().n_levels();
        ++level)
     for (IT i = copy_indices[selected_block][level].begin();
          i != copy_indices[selected_block][level].end();
@@ -132,14 +134,14 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlock<number>::copy_from_mg(
-  const DoFHandler<dim, spacedim> &         mg_dof_handler,
+  const DoFHandler<dim, spacedim> &         dof_handler,
   BlockVector<number2> &                    dst,
   const MGLevelObject<BlockVector<number>> &src) const
 {
   for (unsigned int block = 0; block < selected.size(); ++block)
     if (selected[block])
       for (unsigned int level = 0;
-           level < mg_dof_handler.get_triangulation().n_levels();
+           level < dof_handler.get_triangulation().n_levels();
            ++level)
         for (IT i = copy_indices[block][level].begin();
              i != copy_indices[block][level].end();
@@ -154,14 +156,14 @@ template <typename number>
 template <int dim, typename number2, int spacedim>
 void
 MGTransferBlock<number>::copy_from_mg_add(
-  const DoFHandler<dim, spacedim> &         mg_dof_handler,
+  const DoFHandler<dim, spacedim> &         dof_handler,
   BlockVector<number2> &                    dst,
   const MGLevelObject<BlockVector<number>> &src) const
 {
   for (unsigned int block = 0; block < selected.size(); ++block)
     if (selected[block])
       for (unsigned int level = 0;
-           level < mg_dof_handler.get_triangulation().n_levels();
+           level < dof_handler.get_triangulation().n_levels();
            ++level)
         for (IT i = copy_indices[block][level].begin();
              i != copy_indices[block][level].end();
