@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2016 - 2018 by the deal.II authors
+// Copyright (C) 2016 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -89,9 +89,24 @@ public:
 
   /**
    * Actually build the information for the prolongation for each level.
+   *
+   * The optional second argument of external partitioners allows the user to
+   * suggest vector partitioning on the levels. In case the partitioners
+   * are found to contain all ghost unknowns that are visited through the
+   * transfer, the given partitioners are chosen. This ensures compatibility
+   * of vectors during prolongate and restrict with external partitioners as
+   * given by the user, which in turn saves some copy operations. However, in
+   * case there are unknowns missing -- and this is typically the case at some
+   * point during h-coarsening since processors will need to drop out and
+   * thus children's unknowns on some processor will be needed as ghosts to a
+   * parent cell on another processor -- the provided external partitioners are
+   * ignored and internal variants are used instead.
    */
   void
-  build(const DoFHandler<dim, dim> &mg_dof);
+  build(const DoFHandler<dim, dim> &dof_handler,
+        const std::vector<std::shared_ptr<const Utilities::MPI::Partitioner>>
+          &external_partitioners =
+            std::vector<std::shared_ptr<const Utilities::MPI::Partitioner>>());
 
   /**
    * Prolongate a vector from level <tt>to_level-1</tt> to level
@@ -138,16 +153,19 @@ public:
     const LinearAlgebra::distributed::Vector<Number> &src) const override;
 
   /**
-   * Restrict fine-mesh field @p src to each multigrid level in @p mg_dof and
+   * Restrict fine-mesh field @p src to each multigrid level in @p dof_handler and
    * store the result in @p dst.
    *
-   * If @p dst is empty or has incorrect locally owned size, it will be
-   * resized to locally relevant vectors on each level.
+   * The argument @p dst has to be initialized with the correct size according
+   * to the number of levels of the triangulation.
+   *
+   * If an inner vector of @p dst is empty or has incorrect locally owned size,
+   * it will be resized to locally relevant degrees of freedom on each level.
    */
   template <typename Number2, int spacedim>
   void
   interpolate_to_mg(
-    const DoFHandler<dim, spacedim> &                          mg_dof,
+    const DoFHandler<dim, spacedim> &                          dof_handler,
     MGLevelObject<LinearAlgebra::distributed::Vector<Number>> &dst,
     const LinearAlgebra::distributed::Vector<Number2> &        src) const;
 
@@ -245,6 +263,15 @@ private:
    */
   std::vector<std::vector<std::vector<unsigned short>>> dirichlet_indices;
 
+  /**
+   * A vector that holds shared pointers to the partitioners of the
+   * transfer. These partitioners might be shared with what was passed in from
+   * the outside through build() or be shared with the level vectors inherited
+   * from MGLevelGlobalTransfer.
+   */
+  MGLevelObject<std::shared_ptr<const Utilities::MPI::Partitioner>>
+    vector_partitioners;
+
   /**
    * Perform the prolongation operation.
    */
@@ -333,13 +360,13 @@ public:
    * Actually build the information for the prolongation for each level.
    */
   void
-  build(const DoFHandler<dim, dim> &mg_dof);
+  build(const DoFHandler<dim, dim> &dof_handler);
 
   /**
    * Same as above for the case that each block has its own DoFHandler.
    */
   void
-  build(const std::vector<const DoFHandler<dim, dim> *> &mg_dof);
+  build(const std::vector<const DoFHandler<dim, dim> *> &dof_handler);
 
   /**
    * Prolongate a vector from level <tt>to_level-1</tt> to level
@@ -398,7 +425,7 @@ public:
   template <typename Number2, int spacedim>
   void
   copy_to_mg(
-    const DoFHandler<dim, spacedim> &                               mg_dof,
+    const DoFHandler<dim, spacedim> &                               dof_handler,
     MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &dst,
     const LinearAlgebra::distributed::BlockVector<Number2> &        src) const;
 
@@ -408,7 +435,7 @@ public:
   template <typename Number2, int spacedim>
   void
   copy_to_mg(
-    const std::vector<const DoFHandler<dim, spacedim> *> &          mg_dof,
+    const std::vector<const DoFHandler<dim, spacedim> *> &          dof_handler,
     MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &dst,
     const LinearAlgebra::distributed::BlockVector<Number2> &        src) const;
 
@@ -418,7 +445,7 @@ public:
   template <typename Number2, int spacedim>
   void
   copy_from_mg(
-    const DoFHandler<dim, spacedim> &                 mg_dof,
+    const DoFHandler<dim, spacedim> &                 dof_handler,
     LinearAlgebra::distributed::BlockVector<Number2> &dst,
     const MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &src)
     const;
@@ -429,7 +456,7 @@ public:
   template <typename Number2, int spacedim>
   void
   copy_from_mg(
-    const std::vector<const DoFHandler<dim, spacedim> *> &mg_dof,
+    const std::vector<const DoFHandler<dim, spacedim> *> &dof_handler,
     LinearAlgebra::distributed::BlockVector<Number2> &    dst,
     const MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &src)
     const;
@@ -482,8 +509,8 @@ MGTransferMatrixFree<dim, Number>::interpolate_to_mg(
          ExcDimensionMismatch(
            max_level, dof_handler.get_triangulation().n_global_levels() - 1));
 
-  const parallel::Triangulation<dim, spacedim> *p_tria =
-    (dynamic_cast<const parallel::Triangulation<dim, spacedim> *>(
+  const parallel::TriangulationBase<dim, spacedim> *p_tria =
+    (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
       &dof_handler.get_triangulation()));
   MPI_Comm mpi_communicator =
     p_tria != nullptr ? p_tria->get_communicator() : MPI_COMM_SELF;
@@ -537,7 +564,7 @@ MGTransferMatrixFree<dim, Number>::interpolate_to_mg(
             // if we get to a cell without children (== active), we can
             // skip it as there values should be already set by the
             // equivalent of copy_to_mg()
-            if (!cell->has_children())
+            if (cell->is_active())
               continue;
 
             std::fill(dof_values_coarse.begin(), dof_values_coarse.end(), 0.);
@@ -570,7 +597,7 @@ template <int dim, typename Number>
 template <typename Number2, int spacedim>
 void
 MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
-  const DoFHandler<dim, spacedim> &                               mg_dof,
+  const DoFHandler<dim, spacedim> &                               dof_handler,
   MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &dst,
   const LinearAlgebra::distributed::BlockVector<Number2> &        src) const
 {
@@ -581,7 +608,7 @@ MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
            "DoFHandler for each block, but this method assumes that "
            "the same DoFHandler is used for all the blocks!"));
   const std::vector<const DoFHandler<dim, spacedim> *> mg_dofs(src.n_blocks(),
-                                                               &mg_dof);
+                                                               &dof_handler);
 
   copy_to_mg(mg_dofs, dst, src);
 }
@@ -590,12 +617,12 @@ template <int dim, typename Number>
 template <typename Number2, int spacedim>
 void
 MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
-  const std::vector<const DoFHandler<dim, spacedim> *> &          mg_dof,
+  const std::vector<const DoFHandler<dim, spacedim> *> &          dof_handler,
   MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &dst,
   const LinearAlgebra::distributed::BlockVector<Number2> &        src) const
 {
   const unsigned int n_blocks = src.n_blocks();
-  AssertDimension(mg_dof.size(), n_blocks);
+  AssertDimension(dof_handler.size(), n_blocks);
 
   if (n_blocks == 0)
     return;
@@ -607,13 +634,14 @@ MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
   // dst == defect level block vector. At first run this vector is not
   // initialized. Do this below:
   {
-    const parallel::Triangulation<dim, spacedim> *tria =
-      (dynamic_cast<const parallel::Triangulation<dim, spacedim> *>(
-        &(mg_dof[0]->get_triangulation())));
+    const parallel::TriangulationBase<dim, spacedim> *tria =
+      (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
+        &(dof_handler[0]->get_triangulation())));
     for (unsigned int i = 1; i < n_blocks; ++i)
-      AssertThrow((dynamic_cast<const parallel::Triangulation<dim, spacedim> *>(
-                     &(mg_dof[0]->get_triangulation())) == tria),
-                  ExcMessage("The DoFHandler use different Triangulations!"));
+      AssertThrow(
+        (dynamic_cast<const parallel::TriangulationBase<dim, spacedim> *>(
+           &(dof_handler[0]->get_triangulation())) == tria),
+        ExcMessage("The DoFHandler use different Triangulations!"));
 
     MGLevelObject<bool> do_reinit;
     do_reinit.resize(min_level, max_level);
@@ -628,9 +656,10 @@ MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
         for (unsigned int b = 0; b < n_blocks; ++b)
           {
             LinearAlgebra::distributed::Vector<Number> &v = dst[level].block(b);
-            if (v.size() != mg_dof[b]->locally_owned_mg_dofs(level).size() ||
+            if (v.size() !=
+                  dof_handler[b]->locally_owned_mg_dofs(level).size() ||
                 v.local_size() !=
-                  mg_dof[b]->locally_owned_mg_dofs(level).n_elements())
+                  dof_handler[b]->locally_owned_mg_dofs(level).n_elements())
               {
                 do_reinit[level] = true;
                 break; // b
@@ -647,7 +676,7 @@ MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
               {
                 LinearAlgebra::distributed::Vector<Number> &v =
                   dst[level].block(b);
-                v.reinit(mg_dof[b]->locally_owned_mg_dofs(level),
+                v.reinit(dof_handler[b]->locally_owned_mg_dofs(level),
                          tria != nullptr ? tria->get_communicator() :
                                            MPI_COMM_SELF);
               }
@@ -667,7 +696,7 @@ MGTransferBlockMatrixFree<dim, Number>::copy_to_mg(
       for (unsigned int l = min_level; l <= max_level; ++l)
         dst_non_block[l].reinit(dst[l].block(b));
       const unsigned int data_block = same_for_all ? 0 : b;
-      matrix_free_transfer_vector[data_block].copy_to_mg(*mg_dof[b],
+      matrix_free_transfer_vector[data_block].copy_to_mg(*dof_handler[b],
                                                          dst_non_block,
                                                          src.block(b));
 
@@ -680,14 +709,14 @@ template <int dim, typename Number>
 template <typename Number2, int spacedim>
 void
 MGTransferBlockMatrixFree<dim, Number>::copy_from_mg(
-  const DoFHandler<dim, spacedim> &                                     mg_dof,
-  LinearAlgebra::distributed::BlockVector<Number2> &                    dst,
+  const DoFHandler<dim, spacedim> &                 dof_handler,
+  LinearAlgebra::distributed::BlockVector<Number2> &dst,
   const MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &src)
   const
 {
   AssertDimension(matrix_free_transfer_vector.size(), 1);
   const std::vector<const DoFHandler<dim, spacedim> *> mg_dofs(dst.n_blocks(),
-                                                               &mg_dof);
+                                                               &dof_handler);
 
   copy_from_mg(mg_dofs, dst, src);
 }
@@ -696,13 +725,13 @@ template <int dim, typename Number>
 template <typename Number2, int spacedim>
 void
 MGTransferBlockMatrixFree<dim, Number>::copy_from_mg(
-  const std::vector<const DoFHandler<dim, spacedim> *> &                mg_dof,
-  LinearAlgebra::distributed::BlockVector<Number2> &                    dst,
+  const std::vector<const DoFHandler<dim, spacedim> *> &dof_handler,
+  LinearAlgebra::distributed::BlockVector<Number2> &    dst,
   const MGLevelObject<LinearAlgebra::distributed::BlockVector<Number>> &src)
   const
 {
   const unsigned int n_blocks = dst.n_blocks();
-  AssertDimension(mg_dof.size(), n_blocks);
+  AssertDimension(dof_handler.size(), n_blocks);
 
   if (n_blocks == 0)
     return;
@@ -725,7 +754,7 @@ MGTransferBlockMatrixFree<dim, Number>::copy_from_mg(
           src_non_block[l] = src[l].block(b);
         }
       const unsigned int data_block = same_for_all ? 0 : b;
-      matrix_free_transfer_vector[data_block].copy_from_mg(*mg_dof[b],
+      matrix_free_transfer_vector[data_block].copy_from_mg(*dof_handler[b],
                                                            dst.block(b),
                                                            src_non_block);
     }
