@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 1999 - 2018 by the deal.II authors
+// Copyright (C) 1999 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -153,35 +153,6 @@ public:
   using vector_type       = VectorType;
   using const_vector_type = const VectorType;
 
-  /**
-   * Constructor. The DoFHandler is used to check whether the provided
-   * minlevel and maxlevel are in the range of valid levels.
-   * If maxlevel is set to the default value, the highest valid
-   * level is used.
-   * <tt>transfer</tt> is an object performing prolongation and
-   * restriction.
-   *
-   * This function already initializes the vectors which will be used later in
-   * the course of the computations. You should therefore create objects of
-   * this type as late as possible.
-   *
-   * @deprecated Use the other constructor instead.
-   * The DoFHandler is actually not needed.
-   */
-  template <int dim>
-#ifndef _MSC_VER
-  DEAL_II_DEPRECATED
-#endif
-  Multigrid(const DoFHandler<dim> &             mg_dof_handler,
-            const MGMatrixBase<VectorType> &    matrix,
-            const MGCoarseGridBase<VectorType> &coarse,
-            const MGTransferBase<VectorType> &  transfer,
-            const MGSmootherBase<VectorType> &  pre_smooth,
-            const MGSmootherBase<VectorType> &  post_smooth,
-            const unsigned int                  minlevel = 0,
-            const unsigned int maxlevel = numbers::invalid_unsigned_int,
-            Cycle              cycle    = v_cycle);
-
   /**
    * Constructor. <tt>transfer</tt> is an object performing prolongation and
    * restriction. For levels in [minlevel, maxlevel] matrix has to contain
@@ -301,16 +272,6 @@ public:
    */
   void set_cycle(Cycle);
 
-  /**
-   * @deprecated Debug output will go away. Use signals instead.
-   *
-   * Set the debug level. Higher values will create more debugging output
-   * during the multigrid cycles.
-   */
-  DEAL_II_DEPRECATED
-  void
-  set_debug(const unsigned int);
-
   /**
    * Connect a function to mg::Signals::coarse_solve.
    */
@@ -468,11 +429,6 @@ private:
    */
   SmartPointer<const MGMatrixBase<VectorType>, Multigrid<VectorType>> edge_up;
 
-  /**
-   * Level for debug output. Defaults to zero and can be set by set_debug().
-   */
-  unsigned int debug;
-
   template <int dim, class OtherVectorType, class TRANSFER>
   friend class PreconditionMG;
 };
@@ -577,7 +533,7 @@ public:
   /**
    * Return the MPI communicator object in use with this preconditioner.
    */
-  MPI_Comm
+  const MPI_Comm &
   get_mpi_communicator() const;
 
   /**
@@ -636,40 +592,6 @@ private:
 /* --------------------------- inline functions --------------------- */
 
 
-template <typename VectorType>
-template <int dim>
-Multigrid<VectorType>::Multigrid(const DoFHandler<dim> &         mg_dof_handler,
-                                 const MGMatrixBase<VectorType> &matrix,
-                                 const MGCoarseGridBase<VectorType> &coarse,
-                                 const MGTransferBase<VectorType> &  transfer,
-                                 const MGSmootherBase<VectorType> &  pre_smooth,
-                                 const MGSmootherBase<VectorType> &post_smooth,
-                                 const unsigned int                min_level,
-                                 const unsigned int                max_level,
-                                 Cycle                             cycle)
-  : cycle_type(cycle)
-  , minlevel(min_level)
-  , matrix(&matrix, typeid(*this).name())
-  , coarse(&coarse, typeid(*this).name())
-  , transfer(&transfer, typeid(*this).name())
-  , pre_smooth(&pre_smooth, typeid(*this).name())
-  , post_smooth(&post_smooth, typeid(*this).name())
-  , edge_down(nullptr, typeid(*this).name())
-  , edge_up(nullptr, typeid(*this).name())
-  , debug(0)
-{
-  const unsigned int dof_handler_max_level =
-    mg_dof_handler.get_triangulation().n_global_levels() - 1;
-  if (max_level == numbers::invalid_unsigned_int)
-    maxlevel = dof_handler_max_level;
-  else
-    maxlevel = max_level;
-
-  reinit(minlevel, maxlevel);
-}
-
-
-
 template <typename VectorType>
 Multigrid<VectorType>::Multigrid(const MGMatrixBase<VectorType> &    matrix,
                                  const MGCoarseGridBase<VectorType> &coarse,
@@ -689,7 +611,6 @@ Multigrid<VectorType>::Multigrid(const MGMatrixBase<VectorType> &    matrix,
   , edge_in(nullptr, typeid(*this).name())
   , edge_down(nullptr, typeid(*this).name())
   , edge_up(nullptr, typeid(*this).name())
-  , debug(0)
 {
   if (max_level == numbers::invalid_unsigned_int)
     maxlevel = matrix.get_maxlevel();
@@ -929,14 +850,14 @@ PreconditionMG<dim, VectorType, TRANSFER>::locally_owned_domain_indices(
 
 
 template <int dim, typename VectorType, class TRANSFER>
-MPI_Comm
+const MPI_Comm &
 PreconditionMG<dim, VectorType, TRANSFER>::get_mpi_communicator() const
 {
-  // currently parallel GMG works with distributed Triangulation only,
+  // currently parallel GMG works with parallel triangulations only,
   // so it should be a safe bet to use it to query MPI communicator:
   const Triangulation<dim> &tria = dof_handler_vector[0]->get_triangulation();
-  const parallel::distributed::Triangulation<dim> *ptria =
-    dynamic_cast<const parallel::distributed::Triangulation<dim> *>(&tria);
+  const parallel::TriangulationBase<dim> *ptria =
+    dynamic_cast<const parallel::TriangulationBase<dim> *>(&tria);
   Assert(ptria != nullptr, ExcInternalError());
   return ptria->get_communicator();
 }
