@@ -8,27 +8,27 @@
 // it, and/or modify it under the terms of the GNU Lesser General
 // Public License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// The full text of the license can be found in the file LICENSE at
-// the top level of the deal.II distribution.
+// The full text of the license can be found in the file LICENSE.md at
+// the top level directory of deal.II.
 //
 // ---------------------------------------------------------------------
 
-#include <deal.II/base/point.h>
 #include <deal.II/base/function_spherical.h>
 #include <deal.II/base/geometric_utilities.h>
+#include <deal.II/base/point.h>
 
-#include <cmath>
 #include <algorithm>
+#include <cmath>
 
 DEAL_II_NAMESPACE_OPEN
 namespace Functions
 {
-
   // other implementations/notes:
   // https://github.com/apache/commons-math/blob/master/src/main/java/org/apache/commons/math4/geometry/euclidean/threed/SphericalCoordinates.java
   // http://mathworld.wolfram.com/SphericalCoordinates.html
 
-  /*derivation of Hessian in Maxima as function of tensor products of unit vectors:
+  /*derivation of Hessian in Maxima as function of tensor products of unit
+  vectors:
 
   depends(ur,[theta,phi]);
   depends(utheta,theta);
@@ -93,30 +93,30 @@ namespace Functions
 
   namespace
   {
-
     /**
      * Evaluate unit vectors in spherical coordinates
      */
     template <int dim>
-    void set_unit_vectors(const double &cos_theta,
-                          const double &sin_theta,
-                          const double &cos_phi,
-                          const double &sin_phi,
-                          Tensor<1,dim> &unit_r,
-                          Tensor<1,dim> &unit_theta,
-                          Tensor<1,dim> &unit_phi)
+    void
+    set_unit_vectors(const double    cos_theta,
+                     const double    sin_theta,
+                     const double    cos_phi,
+                     const double    sin_phi,
+                     Tensor<1, dim> &unit_r,
+                     Tensor<1, dim> &unit_theta,
+                     Tensor<1, dim> &unit_phi)
     {
-      unit_r[0]     = cos_theta * sin_phi;
-      unit_r[1]     = sin_theta * sin_phi;
-      unit_r[2]     = cos_phi;
+      unit_r[0] = cos_theta * sin_phi;
+      unit_r[1] = sin_theta * sin_phi;
+      unit_r[2] = cos_phi;
 
       unit_theta[0] = -sin_theta;
-      unit_theta[1] =  cos_theta;
+      unit_theta[1] = cos_theta;
       unit_theta[2] = 0.;
 
-      unit_phi[0]   = cos_theta * cos_phi;
-      unit_phi[1]   = sin_theta * cos_phi;
-      unit_phi[2]   = -sin_phi;
+      unit_phi[0] = cos_theta * cos_phi;
+      unit_phi[1] = sin_theta * cos_phi;
+      unit_phi[2] = -sin_phi;
     }
 
 
@@ -124,63 +124,62 @@ namespace Functions
      * calculates out[i][j] += v*(in1[i]*in2[j]+in1[j]*in2[i])
      */
     template <int dim>
-    void add_outer_product(SymmetricTensor<2,dim> &out,
-                           const double &val,
-                           const Tensor<1,dim> &in1,
-                           const Tensor<1,dim> &in2)
+    void add_outer_product(SymmetricTensor<2, dim> &out,
+                           const double             val,
+                           const Tensor<1, dim> &   in1,
+                           const Tensor<1, dim> &   in2)
     {
       if (val != 0.)
         for (unsigned int i = 0; i < dim; i++)
           for (unsigned int j = i; j < dim; j++)
-            out[i][j] += (in1[i]*in2[j]+in1[j]*in2[i])*val;
+            out[i][j] += (in1[i] * in2[j] + in1[j] * in2[i]) * val;
     }
 
     /**
      * calculates out[i][j] += v*in[i]in[j]
      */
     template <int dim>
-    void add_outer_product(SymmetricTensor<2,dim> &out,
-                           const double &val,
-                           const Tensor<1,dim> &in)
+    void add_outer_product(SymmetricTensor<2, dim> &out,
+                           const double             val,
+                           const Tensor<1, dim> &   in)
     {
       if (val != 0.)
         for (unsigned int i = 0; i < dim; i++)
           for (unsigned int j = i; j < dim; j++)
-            out[i][j] += val*in[i]*in[j];
+            out[i][j] += val * in[i] * in[j];
     }
-  }
+  } // namespace
 
 
 
   template <int dim>
-  Spherical<dim>::Spherical(const Point<dim> &p,
+  Spherical<dim>::Spherical(const Point<dim> & p,
                             const unsigned int n_components)
-    :
-    Function<dim>(n_components),
-    coordinate_system_offset(p)
+    : Function<dim>(n_components)
+    , coordinate_system_offset(p)
   {
-    AssertThrow(dim==3,
-                ExcNotImplemented());
+    AssertThrow(dim == 3, ExcNotImplemented());
   }
 
 
 
   template <int dim>
   double
-  Spherical<dim>::value (const Point<dim>   &p_,
-                         const unsigned int component) const
+  Spherical<dim>::value(const Point<dim> & p_,
+                        const unsigned int component) const
   {
-    const Point<dim> p = p_ - coordinate_system_offset;
-    const std::array<double, dim> sp = GeometricUtilities::Coordinates::to_spherical(p);
+    const Point<dim>              p = p_ - coordinate_system_offset;
+    const std::array<double, dim> sp =
+      GeometricUtilities::Coordinates::to_spherical(p);
     return svalue(sp, component);
   }
 
 
 
   template <int dim>
-  Tensor<1,dim>
-  Spherical<dim>::gradient (const Point<dim>   &/*p_*/,
-                            const unsigned int /*component*/) const
+  Tensor<1, dim>
+  Spherical<dim>::gradient(const Point<dim> & /*p_*/,
+                           const unsigned int /*component*/) const
 
   {
     Assert(false, ExcNotImplemented());
@@ -190,13 +189,13 @@ namespace Functions
 
 
   template <>
-  Tensor<1,3>
-  Spherical<3>::gradient (const Point<3>   &p_,
-                          const unsigned int component) const
+  Tensor<1, 3>
+  Spherical<3>::gradient(const Point<3> &p_, const unsigned int component) const
   {
-    constexpr int dim = 3;
-    const Point<dim> p = p_ - coordinate_system_offset;
-    const std::array<double, dim> sp = GeometricUtilities::Coordinates::to_spherical(p);
+    constexpr int                 dim = 3;
+    const Point<dim>              p   = p_ - coordinate_system_offset;
+    const std::array<double, dim> sp =
+      GeometricUtilities::Coordinates::to_spherical(p);
     const std::array<double, dim> sg = sgradient(sp, component);
 
     // somewhat backwards, but we need cos/sin's for unit vectors
@@ -205,12 +204,11 @@ namespace Functions
     const double cos_phi   = std::cos(sp[2]);
     const double sin_phi   = std::sin(sp[2]);
 
-    Tensor<1,dim> unit_r, unit_theta, unit_phi;
-    set_unit_vectors(cos_theta, sin_theta,
-                     cos_phi, sin_phi,
-                     unit_r, unit_theta, unit_phi);
+    Tensor<1, dim> unit_r, unit_theta, unit_phi;
+    set_unit_vectors(
+      cos_theta, sin_theta, cos_phi, sin_phi, unit_r, unit_theta, unit_phi);
 
-    Tensor<1,dim> res;
+    Tensor<1, dim> res;
 
     if (sg[0] != 0.)
       {
@@ -219,15 +217,13 @@ namespace Functions
 
     if (sg[1] * sin_phi != 0.)
       {
-        Assert (sp[0] != 0.,
-                ExcDivideByZero());
+        Assert(sp[0] != 0., ExcDivideByZero());
         res += unit_theta * sg[1] / (sp[0] * sin_phi);
       }
 
     if (sg[2] != 0.)
       {
-        Assert (sp[0] != 0.,
-                ExcDivideByZero());
+        Assert(sp[0] != 0., ExcDivideByZero());
         res += unit_phi * sg[2] / sp[0];
       }
 
@@ -237,9 +233,9 @@ namespace Functions
 
 
   template <int dim>
-  SymmetricTensor<2,dim>
-  Spherical<dim>::hessian (const Point<dim> &/*p*/,
-                           const unsigned int /*component*/) const
+  SymmetricTensor<2, dim>
+  Spherical<dim>::hessian(const Point<dim> & /*p*/,
+                          const unsigned int /*component*/) const
   {
     Assert(false, ExcNotImplemented());
     return {};
@@ -248,14 +244,14 @@ namespace Functions
 
 
   template <>
-  SymmetricTensor<2,3>
-  Spherical<3>::hessian (const Point<3> &p_,
-                         const unsigned int component) const
+  SymmetricTensor<2, 3>
+  Spherical<3>::hessian(const Point<3> &p_, const unsigned int component) const
 
   {
-    constexpr int dim = 3;
-    const Point<dim> p = p_ - coordinate_system_offset;
-    const std::array<double, dim> sp = GeometricUtilities::Coordinates::to_spherical(p);
+    constexpr int                 dim = 3;
+    const Point<dim>              p   = p_ - coordinate_system_offset;
+    const std::array<double, dim> sp =
+      GeometricUtilities::Coordinates::to_spherical(p);
     const std::array<double, dim> sg = sgradient(sp, component);
     const std::array<double, 6>   sh = shessian(sp, component);
 
@@ -266,59 +262,41 @@ namespace Functions
     const double sin_phi   = std::sin(sp[2]);
     const double r         = sp[0];
 
-    Tensor<1,dim> unit_r, unit_theta, unit_phi;
-    set_unit_vectors(cos_theta, sin_theta,
-                     cos_phi, sin_phi,
-                     unit_r, unit_theta, unit_phi);
+    Tensor<1, dim> unit_r, unit_theta, unit_phi;
+    set_unit_vectors(
+      cos_theta, sin_theta, cos_phi, sin_phi, unit_r, unit_theta, unit_phi);
 
-    const double sin_phi2 = sin_phi*sin_phi;
-    const double r2 = r*r;
-    Assert (r != 0.,
-            ExcDivideByZero());
+    const double sin_phi2 = sin_phi * sin_phi;
+    const double r2       = r * r;
+    Assert(r != 0., ExcDivideByZero());
 
-    const double c_utheta2     = sg[0]/r +
-                                 ((sin_phi!= 0.) ?
-                                  (cos_phi*sg[2])/(r2*sin_phi)+sh[1]/(r2*sin_phi2) :
-                                  0.);
-    const double c_utheta_ur   = ((sin_phi != 0.) ?
-                                  (r*sh[3]-sg[1])/(r2*sin_phi) :
-                                  0.);
-    const double c_utheta_uphi = ((sin_phi != 0.) ?
-                                  (sh[5]*sin_phi-cos_phi*sg[1])/(r2*sin_phi2) :
-                                  0.);
-    const double c_ur2         = sh[0];
-    const double c_ur_uphi     = (r*sh[4]-sg[2])/r2;
-    const double c_uphi2       = (sh[2]+r*sg[0])/r2;
+    const double c_utheta2 =
+      sg[0] / r + ((sin_phi != 0.) ? (cos_phi * sg[2]) / (r2 * sin_phi) +
+                                       sh[1] / (r2 * sin_phi2) :
+                                     0.);
+    const double c_utheta_ur =
+      ((sin_phi != 0.) ? (r * sh[3] - sg[1]) / (r2 * sin_phi) : 0.);
+    const double c_utheta_uphi =
+      ((sin_phi != 0.) ? (sh[5] * sin_phi - cos_phi * sg[1]) / (r2 * sin_phi2) :
+                         0.);
+    const double c_ur2     = sh[0];
+    const double c_ur_uphi = (r * sh[4] - sg[2]) / r2;
+    const double c_uphi2   = (sh[2] + r * sg[0]) / r2;
 
     // go through each tensor product
-    SymmetricTensor<2,dim> res;
+    SymmetricTensor<2, dim> res;
 
-    add_outer_product(res,
-                      c_utheta2,
-                      unit_theta);
+    add_outer_product(res, c_utheta2, unit_theta);
 
-    add_outer_product(res,
-                      c_utheta_ur,
-                      unit_theta,
-                      unit_r);
+    add_outer_product(res, c_utheta_ur, unit_theta, unit_r);
 
-    add_outer_product(res,
-                      c_utheta_uphi,
-                      unit_theta,
-                      unit_phi);
+    add_outer_product(res, c_utheta_uphi, unit_theta, unit_phi);
 
-    add_outer_product(res,
-                      c_ur2,
-                      unit_r);
+    add_outer_product(res, c_ur2, unit_r);
 
-    add_outer_product(res,
-                      c_ur_uphi,
-                      unit_r,
-                      unit_phi);
+    add_outer_product(res, c_ur_uphi, unit_r, unit_phi);
 
-    add_outer_product(res,
-                      c_uphi2,
-                      unit_phi);
+    add_outer_product(res, c_uphi2, unit_phi);
 
     return res;
   }
@@ -327,7 +305,7 @@ namespace Functions
 
   template <int dim>
   std::size_t
-  Spherical<dim>::memory_consumption () const
+  Spherical<dim>::memory_consumption() const
   {
     return sizeof(Spherical<dim>);
   }
@@ -339,8 +317,7 @@ namespace Functions
   Spherical<dim>::svalue(const std::array<double, dim> & /* sp */,
                          const unsigned int /*component*/) const
   {
-    AssertThrow(false,
-                ExcNotImplemented());
+    AssertThrow(false, ExcNotImplemented());
     return 0.;
   }
 
@@ -351,30 +328,28 @@ namespace Functions
   Spherical<dim>::sgradient(const std::array<double, dim> & /* sp */,
                             const unsigned int /*component*/) const
   {
-    AssertThrow(false,
-                ExcNotImplemented());
-    return std::array<double,dim>();
+    AssertThrow(false, ExcNotImplemented());
+    return std::array<double, dim>();
   }
 
 
 
   template <int dim>
   std::array<double, 6>
-  Spherical<dim>::shessian (const std::array<double, dim> & /* sp */,
-                            const unsigned int /*component*/) const
+  Spherical<dim>::shessian(const std::array<double, dim> & /* sp */,
+                           const unsigned int /*component*/) const
   {
-    AssertThrow(false,
-                ExcNotImplemented());
+    AssertThrow(false, ExcNotImplemented());
     return std::array<double, 6>();
   }
 
 
 
-// explicit instantiations
+  // explicit instantiations
   template class Spherical<1>;
   template class Spherical<2>;
   template class Spherical<3>;
 
-}
+} // namespace Functions
 
 DEAL_II_NAMESPACE_CLOSE
