@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2019 by the deal.II authors
+// Copyright (C) 2005 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -94,35 +94,6 @@ IndexSet::IndexSet(const Epetra_BlockMap &map)
 
 
 
-void
-IndexSet::add_range(const size_type begin, const size_type end)
-{
-  Assert((begin < index_space_size) ||
-           ((begin == index_space_size) && (end == index_space_size)),
-         ExcIndexRangeType<size_type>(begin, 0, index_space_size));
-  Assert(end <= index_space_size,
-         ExcIndexRangeType<size_type>(end, 0, index_space_size + 1));
-  Assert(begin <= end, ExcIndexRangeType<size_type>(begin, 0, end));
-
-  if (begin != end)
-    {
-      const Range new_range(begin, end);
-
-      // the new index might be larger than the last index present in the
-      // ranges. Then we can skip the binary search
-      if (ranges.size() == 0 || begin > ranges.back().end)
-        ranges.push_back(new_range);
-      else
-        ranges.insert(Utilities::lower_bound(ranges.begin(),
-                                             ranges.end(),
-                                             new_range),
-                      new_range);
-      is_compressed = false;
-    }
-}
-
-
-
 void
 IndexSet::do_compress() const
 {
@@ -188,6 +159,7 @@ IndexSet::do_compress() const
 
 
 
+#ifndef DOXYGEN
 IndexSet IndexSet::operator&(const IndexSet &is) const
 {
   Assert(size() == is.size(), ExcDimensionMismatch(size(), is.size()));
@@ -231,6 +203,7 @@ IndexSet IndexSet::operator&(const IndexSet &is) const
   result.compress();
   return result;
 }
+#endif
 
 
 
@@ -261,7 +234,25 @@ IndexSet::get_view(const size_type begin, const size_type end) const
   return result;
 }
 
+std::vector<IndexSet>
+IndexSet::split_by_block(
+  const std::vector<types::global_dof_index> &n_indices_per_block) const
+{
+  std::vector<IndexSet> partitioned;
+  const unsigned int    n_blocks = n_indices_per_block.size();
 
+  partitioned.reserve(n_blocks);
+  types::global_dof_index start = 0;
+  types::global_dof_index sum   = 0;
+  for (const auto n_block_indices : n_indices_per_block)
+    {
+      partitioned.push_back(this->get_view(start, start + n_block_indices));
+      start += n_block_indices;
+      sum += partitioned.back().size();
+    }
+  AssertDimension(sum, this->size());
+  return partitioned;
+}
 
 void
 IndexSet::subtract_set(const IndexSet &other)
@@ -335,6 +326,18 @@ IndexSet::subtract_set(const IndexSet &other)
 
 
 
+IndexSet
+IndexSet::tensor_product(const IndexSet &other) const
+{
+  IndexSet set(this->size() * other.size());
+  for (const auto el : *this)
+    set.add_indices(other, el * other.size());
+  set.compress();
+  return set;
+}
+
+
+
 IndexSet::size_type
 IndexSet::pop_back()
 {
@@ -376,16 +379,15 @@ IndexSet::pop_front()
 
 
 void
-IndexSet::add_indices(const IndexSet &other, const unsigned int offset)
+IndexSet::add_indices(const IndexSet &other, const size_type offset)
 {
   if ((this == &other) && (offset == 0))
     return;
 
-  Assert(other.ranges.size() == 0 ||
-           other.ranges.back().end - 1 < index_space_size,
-         ExcIndexRangeType<size_type>(other.ranges.back().end - 1,
-                                      0,
-                                      index_space_size));
+  if (other.ranges.size() != 0)
+    {
+      AssertIndexRange(other.ranges.back().end - 1, index_space_size);
+    }
 
   compress();
   other.compress();
@@ -541,10 +543,10 @@ IndexSet::make_tpetra_map(const MPI_Comm &communicator,
                         "contain all indices exactly once: the sum of "
                         "the number of entries the various processors "
                         "want to store locally is " +
-                        Utilities::to_string(n_global_elements) +
+                        std::to_string(n_global_elements) +
                         " whereas the total size of the object to be "
                         "allocated is " +
-                        Utilities::to_string(size()) +
+                        std::to_string(size()) +
                         ". In other words, there are "
                         "either indices that are not spoken for "
                         "by any processor, or there are indices that are "
@@ -610,10 +612,10 @@ IndexSet::make_trilinos_map(const MPI_Comm &communicator,
                         "contain all indices exactly once: the sum of "
                         "the number of entries the various processors "
                         "want to store locally is " +
-                        Utilities::to_string(n_global_elements) +
+                        std::to_string(n_global_elements) +
                         " whereas the total size of the object to be "
                         "allocated is " +
-                        Utilities::to_string(size()) +
+                        std::to_string(size()) +
                         ". In other words, there are "
                         "either indices that are not spoken for "
                         "by any processor, or there are indices that are "
