@@ -1,6 +1,6 @@
 // ---------------------------------------------------------------------
 //
-// Copyright (C) 2005 - 2019 by the deal.II authors
+// Copyright (C) 2005 - 2020 by the deal.II authors
 //
 // This file is part of the deal.II library.
 //
@@ -18,6 +18,8 @@
 #include <deal.II/base/index_set.h>
 #include <deal.II/base/mpi.h>
 #include <deal.II/base/mpi.templates.h>
+#include <deal.II/base/mpi_compute_index_owner_internal.h>
+#include <deal.II/base/mpi_tags.h>
 #include <deal.II/base/multithread_info.h>
 #include <deal.II/base/utilities.h>
 
@@ -27,12 +29,13 @@
 
 #include <iostream>
 #include <numeric>
+#include <set>
+#include <vector>
 
 #ifdef DEAL_II_WITH_TRILINOS
 #  ifdef DEAL_II_WITH_MPI
 #    include <deal.II/lac/trilinos_parallel_block_vector.h>
 #    include <deal.II/lac/trilinos_vector.h>
-#    include <deal.II/lac/vector_memory.h>
 
 #    include <Epetra_MpiComm.h>
 #  endif
@@ -64,8 +67,51 @@ DEAL_II_NAMESPACE_OPEN
 
 namespace Utilities
 {
+  IndexSet
+  create_evenly_distributed_partitioning(const unsigned int my_partition_id,
+                                         const unsigned int n_partitions,
+                                         const IndexSet::size_type total_size)
+  {
+    const unsigned int remain = total_size % n_partitions;
+
+    const IndexSet::size_type min_size = total_size / n_partitions;
+
+    const IndexSet::size_type begin =
+      min_size * my_partition_id + std::min(my_partition_id, remain);
+    const IndexSet::size_type end =
+      min_size * (my_partition_id + 1) + std::min(my_partition_id + 1, remain);
+    IndexSet result(total_size);
+    result.add_range(begin, end);
+    return result;
+  }
+
   namespace MPI
   {
+    MinMaxAvg
+    min_max_avg(const double my_value, const MPI_Comm &mpi_communicator)
+    {
+      MinMaxAvg result;
+      min_max_avg(ArrayView<const double>(my_value),
+                  ArrayView<MinMaxAvg>(result),
+                  mpi_communicator);
+
+      return result;
+    }
+
+
+
+    std::vector<MinMaxAvg>
+    min_max_avg(const std::vector<double> &my_values,
+                const MPI_Comm &           mpi_communicator)
+    {
+      std::vector<MinMaxAvg> results(my_values.size());
+      min_max_avg(my_values, results, mpi_communicator);
+
+      return results;
+    }
+
+
+
 #ifdef DEAL_II_WITH_MPI
     unsigned int
     n_mpi_processes(const MPI_Comm &mpi_communicator)
@@ -100,6 +146,16 @@ namespace Utilities
 
 
 
+    void
+    free_communicator(MPI_Comm &mpi_communicator)
+    {
+      // MPI_Comm_free will set the argument to MPI_COMM_NULL automatically.
+      const int ierr = MPI_Comm_free(&mpi_communicator);
+      AssertThrowMPI(ierr);
+    }
+
+
+
     int
     create_group(const MPI_Comm & comm,
                  const MPI_Group &group,
@@ -183,8 +239,8 @@ namespace Utilities
 
 
     std::vector<IndexSet>
-    create_ascending_partitioning(const MPI_Comm &           comm,
-                                  const IndexSet::size_type &local_size)
+    create_ascending_partitioning(const MPI_Comm &          comm,
+                                  const IndexSet::size_type local_size)
     {
       const unsigned int                     n_proc = n_mpi_processes(comm);
       const std::vector<IndexSet::size_type> sizes =
@@ -204,6 +260,78 @@ namespace Utilities
       return res;
     }
 
+    IndexSet
+    create_evenly_distributed_partitioning(const MPI_Comm &          comm,
+                                           const IndexSet::size_type total_size)
+    {
+      const unsigned int this_proc = this_mpi_process(comm);
+      const unsigned int n_proc    = n_mpi_processes(comm);
+
+      return Utilities::create_evenly_distributed_partitioning(this_proc,
+                                                               n_proc,
+                                                               total_size);
+    }
+
+
+
+    /**
+     * A re-implementation of compute_point_to_point_communication_pattern
+     * using a ConsensusAlgorithm.
+     */
+    class ConsensusAlgorithmsProcessTargets
+      : public ConsensusAlgorithms::Process<unsigned int, unsigned int>
+    {
+    public:
+      ConsensusAlgorithmsProcessTargets(const std::vector<unsigned int> &target)
+        : target(target)
+      {}
+
+      using T1 = unsigned int;
+      using T2 = unsigned int;
+
+      virtual void
+      answer_request(const unsigned int other_rank,
+                     const std::vector<T1> &,
+                     std::vector<T2> &) override
+      {
+        this->sources.push_back(other_rank);
+      }
+
+      /**
+       * Simply return the user-provided list.
+       *
+       * @return List of processes this process wants to send requests to.
+       */
+      virtual std::vector<unsigned int>
+      compute_targets() override
+      {
+        return target;
+      }
+
+      /**
+       * The result of the consensus algorithm.
+       * @return Sorted list of ranks of processes wanting to send a request to
+       *         this process.
+       */
+      std::vector<unsigned int>
+      get_result()
+      {
+        std::sort(sources.begin(), sources.end());
+        return sources;
+      }
+
+    private:
+      /**
+       * List of processes this process wants to send requests to.
+       */
+      const std::vector<unsigned int> &target;
+
+      /**
+       * List of ranks of processes wanting to send a request to this process.
+       */
+      std::vector<unsigned int> sources;
+    };
+
 
 
     std::vector<unsigned int>
@@ -213,17 +341,35 @@ namespace Utilities
     {
       const unsigned int myid    = Utilities::MPI::this_mpi_process(mpi_comm);
       const unsigned int n_procs = Utilities::MPI::n_mpi_processes(mpi_comm);
+      (void)myid;
+      (void)n_procs;
 
       for (const unsigned int destination : destinations)
         {
           (void)destination;
-          Assert(destination < n_procs, ExcIndexRange(destination, 0, n_procs));
+          AssertIndexRange(destination, n_procs);
           Assert(destination != myid,
                  ExcMessage(
                    "There is no point in communicating with ourselves."));
         }
 
-#  if DEAL_II_MPI_VERSION_GTE(2, 2)
+#  if DEAL_II_MPI_VERSION_GTE(3, 0)
+
+      ConsensusAlgorithmsProcessTargets process(destinations);
+      ConsensusAlgorithms::NBX<ConsensusAlgorithmsProcessTargets::T1,
+                               ConsensusAlgorithmsProcessTargets::T2>
+        consensus_algorithm(process, mpi_comm);
+      consensus_algorithm.run();
+      return process.get_result();
+
+#  elif DEAL_II_MPI_VERSION_GTE(2, 2)
+
+      static CollectiveMutex      mutex;
+      CollectiveMutex::ScopedLock lock(mutex, mpi_comm);
+
+      const int mpi_tag =
+        internal::Tags::compute_point_to_point_communication_pattern;
+
       // Calculate the number of messages to send to each process
       std::vector<unsigned int> dest_vector(n_procs);
       for (const auto &el : destinations)
@@ -241,34 +387,43 @@ namespace Utilities
       // Send myid to every process in `destinations` vector...
       std::vector<MPI_Request> send_requests(destinations.size());
       for (const auto &el : destinations)
-        MPI_Isend(&myid,
-                  1,
-                  MPI_UNSIGNED,
-                  el,
-                  32766,
-                  mpi_comm,
-                  send_requests.data() + (&el - destinations.data()));
+        {
+          const int ierr =
+            MPI_Isend(&myid,
+                      1,
+                      MPI_UNSIGNED,
+                      el,
+                      mpi_tag,
+                      mpi_comm,
+                      send_requests.data() + (&el - destinations.data()));
+          AssertThrowMPI(ierr);
+        }
 
-      // if no one to receive from, return an empty vector
-      if (n_recv_from == 0)
-        return std::vector<unsigned int>();
 
-      // ...otherwise receive `n_recv_from` times from the processes
+      // Receive `n_recv_from` times from the processes
       // who communicate with this one. Store the obtained id's
       // in the resulting vector
       std::vector<unsigned int> origins(n_recv_from);
       for (auto &el : origins)
-        MPI_Recv(&el,
-                 1,
-                 MPI_UNSIGNED,
-                 MPI_ANY_SOURCE,
-                 32766,
-                 mpi_comm,
-                 MPI_STATUS_IGNORE);
+        {
+          const int ierr = MPI_Recv(&el,
+                                    1,
+                                    MPI_UNSIGNED,
+                                    MPI_ANY_SOURCE,
+                                    mpi_tag,
+                                    mpi_comm,
+                                    MPI_STATUS_IGNORE);
+          AssertThrowMPI(ierr);
+        }
+
+      if (destinations.size() > 0)
+        {
+          const int ierr = MPI_Waitall(destinations.size(),
+                                       send_requests.data(),
+                                       MPI_STATUSES_IGNORE);
+          AssertThrowMPI(ierr);
+        }
 
-      MPI_Waitall(destinations.size(),
-                  send_requests.data(),
-                  MPI_STATUSES_IGNORE);
       return origins;
 #  else
       // let all processors communicate the maximal number of destinations
@@ -330,7 +485,7 @@ namespace Utilities
       for (const unsigned int destination : destinations)
         {
           (void)destination;
-          Assert(destination < n_procs, ExcIndexRange(destination, 0, n_procs));
+          AssertIndexRange(destination, n_procs);
           Assert(destination != Utilities::MPI::this_mpi_process(mpi_comm),
                  ExcMessage(
                    "There is no point in communicating with ourselves."));
@@ -390,67 +545,81 @@ namespace Utilities
                  int *       len,
                  MPI_Datatype *)
       {
-        (void)len;
         const MinMaxAvg *in_lhs    = static_cast<const MinMaxAvg *>(in_lhs_);
         MinMaxAvg *      inout_rhs = static_cast<MinMaxAvg *>(inout_rhs_);
 
-        Assert(*len == 1, ExcInternalError());
+        for (int i = 0; i < *len; i++)
+          {
+            inout_rhs[i].sum += in_lhs[i].sum;
+            if (inout_rhs[i].min > in_lhs[i].min)
+              {
+                inout_rhs[i].min       = in_lhs[i].min;
+                inout_rhs[i].min_index = in_lhs[i].min_index;
+              }
+            else if (inout_rhs[i].min == in_lhs[i].min)
+              {
+                // choose lower cpu index when tied to make operator commutative
+                if (inout_rhs[i].min_index > in_lhs[i].min_index)
+                  inout_rhs[i].min_index = in_lhs[i].min_index;
+              }
 
-        inout_rhs->sum += in_lhs->sum;
-        if (inout_rhs->min > in_lhs->min)
-          {
-            inout_rhs->min       = in_lhs->min;
-            inout_rhs->min_index = in_lhs->min_index;
-          }
-        else if (inout_rhs->min == in_lhs->min)
-          {
-            // choose lower cpu index when tied to make operator commutative
-            if (inout_rhs->min_index > in_lhs->min_index)
-              inout_rhs->min_index = in_lhs->min_index;
-          }
-
-        if (inout_rhs->max < in_lhs->max)
-          {
-            inout_rhs->max       = in_lhs->max;
-            inout_rhs->max_index = in_lhs->max_index;
-          }
-        else if (inout_rhs->max == in_lhs->max)
-          {
-            // choose lower cpu index when tied to make operator commutative
-            if (inout_rhs->max_index > in_lhs->max_index)
-              inout_rhs->max_index = in_lhs->max_index;
+            if (inout_rhs[i].max < in_lhs[i].max)
+              {
+                inout_rhs[i].max       = in_lhs[i].max;
+                inout_rhs[i].max_index = in_lhs[i].max_index;
+              }
+            else if (inout_rhs[i].max == in_lhs[i].max)
+              {
+                // choose lower cpu index when tied to make operator commutative
+                if (inout_rhs[i].max_index > in_lhs[i].max_index)
+                  inout_rhs[i].max_index = in_lhs[i].max_index;
+              }
           }
       }
     } // namespace
 
 
 
-    MinMaxAvg
-    min_max_avg(const double my_value, const MPI_Comm &mpi_communicator)
+    void
+    min_max_avg(const ArrayView<const double> &my_values,
+                const ArrayView<MinMaxAvg> &   result,
+                const MPI_Comm &               mpi_communicator)
     {
       // If MPI was not started, we have a serial computation and cannot run
       // the other MPI commands
-      if (job_supports_mpi() == false)
+      if (job_supports_mpi() == false ||
+          Utilities::MPI::n_mpi_processes(mpi_communicator) <= 1)
         {
-          MinMaxAvg result;
-          result.sum       = my_value;
-          result.avg       = my_value;
-          result.min       = my_value;
-          result.max       = my_value;
-          result.min_index = 0;
-          result.max_index = 0;
-
-          return result;
+          for (unsigned int i = 0; i < my_values.size(); i++)
+            {
+              result[i].sum       = my_values[i];
+              result[i].avg       = my_values[i];
+              result[i].min       = my_values[i];
+              result[i].max       = my_values[i];
+              result[i].min_index = 0;
+              result[i].max_index = 0;
+            }
+          return;
         }
 
+      AssertDimension(Utilities::MPI::min(my_values.size(), mpi_communicator),
+                      Utilities::MPI::max(my_values.size(), mpi_communicator));
+
+      AssertDimension(my_values.size(), result.size());
+
+
+
       // To avoid uninitialized values on some MPI implementations, provide
       // result with a default value already...
-      MinMaxAvg result = {0.,
-                          std::numeric_limits<double>::max(),
-                          -std::numeric_limits<double>::max(),
-                          0,
-                          0,
-                          0.};
+      MinMaxAvg dummy = {0.,
+                         std::numeric_limits<double>::max(),
+                         -std::numeric_limits<double>::max(),
+                         0,
+                         0,
+                         0.};
+
+      for (auto &i : result)
+        i = dummy;
 
       const unsigned int my_id =
         dealii::Utilities::MPI::this_mpi_process(mpi_communicator);
@@ -464,21 +633,28 @@ namespace Utilities
                       &op);
       AssertThrowMPI(ierr);
 
-      MinMaxAvg in;
-      in.sum = in.min = in.max = my_value;
-      in.min_index = in.max_index = my_id;
+      std::vector<MinMaxAvg> in(my_values.size());
+
+      for (unsigned int i = 0; i < my_values.size(); i++)
+        {
+          in[i].sum = in[i].min = in[i].max = my_values[i];
+          in[i].min_index = in[i].max_index = my_id;
+        }
 
       MPI_Datatype type;
-      int          lengths[]       = {3, 2};
-      MPI_Aint     displacements[] = {0, offsetof(MinMaxAvg, min_index)};
-      MPI_Datatype types[]         = {MPI_DOUBLE, MPI_INT};
+      int          lengths[]       = {3, 2, 1};
+      MPI_Aint     displacements[] = {0,
+                                  offsetof(MinMaxAvg, min_index),
+                                  offsetof(MinMaxAvg, avg)};
+      MPI_Datatype types[]         = {MPI_DOUBLE, MPI_INT, MPI_DOUBLE};
 
-      ierr = MPI_Type_create_struct(2, lengths, displacements, types, &type);
+      ierr = MPI_Type_create_struct(3, lengths, displacements, types, &type);
       AssertThrowMPI(ierr);
 
       ierr = MPI_Type_commit(&type);
       AssertThrowMPI(ierr);
-      ierr = MPI_Allreduce(&in, &result, 1, type, op, mpi_communicator);
+      ierr = MPI_Allreduce(
+        in.data(), result.data(), my_values.size(), type, op, mpi_communicator);
       AssertThrowMPI(ierr);
 
       ierr = MPI_Type_free(&type);
@@ -487,11 +663,11 @@ namespace Utilities
       ierr = MPI_Op_free(&op);
       AssertThrowMPI(ierr);
 
-      result.avg = result.sum / numproc;
-
-      return result;
+      for (auto &r : result)
+        r.avg = r.sum / numproc;
     }
 
+
 #else
 
     unsigned int
@@ -509,13 +685,22 @@ namespace Utilities
     }
 
 
+
     std::vector<IndexSet>
     create_ascending_partitioning(const MPI_Comm & /*comm*/,
-                                  const IndexSet::size_type &local_size)
+                                  const IndexSet::size_type local_size)
     {
       return std::vector<IndexSet>(1, complete_index_set(local_size));
     }
 
+    IndexSet
+    create_evenly_distributed_partitioning(const MPI_Comm & /*comm*/,
+                                           const IndexSet::size_type total_size)
+    {
+      return complete_index_set(total_size);
+    }
+
+
 
     MPI_Comm
     duplicate_communicator(const MPI_Comm &mpi_communicator)
@@ -525,19 +710,28 @@ namespace Utilities
 
 
 
-    MinMaxAvg
-    min_max_avg(const double my_value, const MPI_Comm &)
+    void
+    free_communicator(MPI_Comm & /*mpi_communicator*/)
+    {}
+
+
+
+    void
+    min_max_avg(const ArrayView<const double> &my_values,
+                const ArrayView<MinMaxAvg> &   result,
+                const MPI_Comm &)
     {
-      MinMaxAvg result;
+      AssertDimension(my_values.size(), result.size());
 
-      result.sum       = my_value;
-      result.avg       = my_value;
-      result.min       = my_value;
-      result.max       = my_value;
-      result.min_index = 0;
-      result.max_index = 0;
-
-      return result;
+      for (unsigned int i = 0; i < my_values.size(); i++)
+        {
+          result[i].sum       = my_values[i];
+          result[i].avg       = my_values[i];
+          result[i].min       = my_values[i];
+          result[i].max       = my_values[i];
+          result[i].min_index = 0;
+          result[i].max_index = 0;
+        }
     }
 
 #endif
@@ -640,7 +834,7 @@ namespace Utilities
 #ifdef DEAL_II_WITH_MPI
           // we need to figure out how many MPI processes there are on the
           // current node, as well as how many CPU cores we have. for the
-          // first task, check what get_hostname() returns and then to an
+          // first task, check what get_hostname() returns and then do an
           // allgather so each processor gets the answer
           //
           // in calculating the length of the string, don't forget the
@@ -703,6 +897,33 @@ namespace Utilities
     }
 
 
+
+    void
+    MPI_InitFinalize::register_request(MPI_Request &request)
+    {
+      // insert if it is not in the set already:
+      requests.insert(&request);
+    }
+
+
+
+    void
+    MPI_InitFinalize::unregister_request(MPI_Request &request)
+    {
+      Assert(
+        requests.find(&request) != requests.end(),
+        ExcMessage(
+          "You tried to call unregister_request() with an invalid request."));
+
+      requests.erase(&request);
+    }
+
+
+
+    std::set<MPI_Request *> MPI_InitFinalize::requests;
+
+
+
     MPI_InitFinalize::~MPI_InitFinalize()
     {
       // make memory pool release all PETSc/Trilinos/MPI-based vectors that
@@ -711,8 +932,14 @@ namespace Utilities
       // would run after MPI_Finalize is called, leading to errors
 
 #ifdef DEAL_II_WITH_MPI
-      // Start with the deal.II MPI vectors (need to do this before finalizing
-      // PETSc because it finalizes MPI).  Delete vectors from the pools:
+      // Before exiting, wait for nonblocking communication to complete:
+      for (auto request : requests)
+        {
+          const int ierr = MPI_Wait(request, MPI_STATUS_IGNORE);
+          AssertThrowMPI(ierr);
+        }
+
+      // Start with deal.II MPI vectors and delete vectors from the pools:
       GrowingVectorMemory<
         LinearAlgebra::distributed::Vector<double>>::release_unused_memory();
       GrowingVectorMemory<LinearAlgebra::distributed::BlockVector<double>>::
@@ -778,13 +1005,14 @@ namespace Utilities
 #ifdef DEAL_II_WITH_MPI
       if (job_supports_mpi() == true)
         {
-          if (std::uncaught_exception())
+#  if __cpp_lib_uncaught_exceptions >= 201411
+          // std::uncaught_exception() is deprecated in c++17
+          if (std::uncaught_exceptions() > 0)
+#  else
+          if (std::uncaught_exception() == true)
+#  endif
             {
-              std::cerr
-                << "ERROR: Uncaught exception in MPI_InitFinalize on proc "
-                << this_mpi_process(MPI_COMM_WORLD)
-                << ". Skipping MPI_Finalize() to avoid a deadlock."
-                << std::endl;
+              // do not try to call MPI_Finalize to avoid a deadlock.
             }
           else
             {
@@ -814,6 +1042,135 @@ namespace Utilities
 
 
 
+    std::vector<unsigned int>
+    compute_index_owner(const IndexSet &owned_indices,
+                        const IndexSet &indices_to_look_up,
+                        const MPI_Comm &comm)
+    {
+      Assert(owned_indices.size() == indices_to_look_up.size(),
+             ExcMessage("IndexSets have to have the same sizes."));
+
+      Assert(
+        owned_indices.size() == Utilities::MPI::max(owned_indices.size(), comm),
+        ExcMessage("IndexSets have to have the same size on all processes."));
+
+      std::vector<unsigned int> owning_ranks(indices_to_look_up.n_elements());
+
+      // Step 1: setup dictionary
+      // The input owned_indices can be partitioned arbitrarily. In the
+      // dictionary, the index set is statically repartitioned among the
+      // processes again and extended with information with the actual owner
+      // of that the index.
+      internal::ComputeIndexOwner::ConsensusAlgorithmsPayload process(
+        owned_indices, indices_to_look_up, comm, owning_ranks);
+
+      // Step 2: read dictionary
+      // Communicate with the process who owns the index in the static
+      // partition (i.e. in the dictionary). This process returns the actual
+      // owner of the index.
+      ConsensusAlgorithms::Selector<
+        std::pair<types::global_dof_index, types::global_dof_index>,
+        unsigned int>
+        consensus_algorithm(process, comm);
+      consensus_algorithm.run();
+
+      return owning_ranks;
+    }
+
+
+
+    CollectiveMutex::CollectiveMutex()
+      : locked(false)
+      , request(MPI_REQUEST_NULL)
+    {
+      Utilities::MPI::MPI_InitFinalize::register_request(request);
+    }
+
+
+
+    CollectiveMutex::~CollectiveMutex()
+    {
+      Assert(
+        !locked,
+        ExcMessage(
+          "Error: MPI::CollectiveMutex is still locked while being destroyed!"));
+
+      Utilities::MPI::MPI_InitFinalize::unregister_request(request);
+    }
+
+
+
+    void
+    CollectiveMutex::lock(MPI_Comm comm)
+    {
+      (void)comm;
+
+      Assert(
+        !locked,
+        ExcMessage(
+          "Error: MPI::CollectiveMutex needs to be unlocked before lock()"));
+
+#ifdef DEAL_II_WITH_MPI
+
+      // TODO: For now, we implement this mutex with a blocking barrier
+      // in the lock and unlock. It needs to be tested, if we can move
+      // to a nonblocking barrier (code disabled below).
+
+      const int ierr = MPI_Barrier(comm);
+      AssertThrowMPI(ierr);
+
+#  if 0 && DEAL_II_MPI_VERSION_GTE(3, 0)
+      // wait for non-blocking barrier to finish. This is a noop the
+      // first time we lock().
+      const int ierr = MPI_Wait(&request, MPI_STATUS_IGNORE);
+      AssertThrowMPI(ierr);
+#  else
+      // nothing to do as blocking barrier already completed
+#  endif
+#endif
+
+      locked = true;
+    }
+
+
+
+    void
+    CollectiveMutex::unlock(MPI_Comm comm)
+    {
+      (void)comm;
+
+      Assert(
+        locked,
+        ExcMessage(
+          "Error: MPI::CollectiveMutex needs to be locked before unlock()"));
+
+#ifdef DEAL_II_WITH_MPI
+
+      // TODO: For now, we implement this mutex with a blocking barrier
+      // in the lock and unlock. It needs to be tested, if we can move
+      // to a nonblocking barrier (code disabled below):
+
+#  if 0 && DEAL_II_MPI_VERSION_GTE(3, 0)
+      const int ierr = MPI_Ibarrier(comm, &request);
+      AssertThrowMPI(ierr);
+#  else
+      const int ierr = MPI_Barrier(comm);
+      AssertThrowMPI(ierr);
+#  endif
+#endif
+
+      locked = false;
+    }
+
+
+    template std::vector<unsigned int>
+    compute_set_union(const std::vector<unsigned int> &vec,
+                      const MPI_Comm &                 comm);
+
+
+    template std::set<unsigned int>
+    compute_set_union(const std::set<unsigned int> &set, const MPI_Comm &comm);
+
 #include "mpi.inst"
   } // end of namespace MPI
 } // end of namespace Utilities
